/*!
    @header BTBrushTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/27/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>
#import <CoreFoundation/CFArray.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColor.h>
#import <SMKCGExtensions/CGEFloat.h>
#import <SMKCGExtensions/CGEColor.h>
#import <SMKCGExtensions/CGELayer.h>
#import <Foundation/NSMapTable.h>

@protocol BTCGImage;
@protocol BTBrushTouchConsumer, BTBrushTouchProvider;
@class NSData, NSArray;
@class NSTimer;

/*!
    @class BTBrushTool
    @discussion The tool has three modes: image (repeating compositing of an image), color (brush- & spray-like drawing) and eraser. To switch to the image mode, set "image" attribute to a non-nil value. To switch to the color mode, set "image" attribute to nil and set "color" attribute to a non-nil value. To switch to the eraser mode, set both "image" and "color" attributes to nil. Note: this tool does not work on Panther.
                Stage cycle: BTWaitingToolStage -> BTDrawingToolStage.
*/
@interface BTBrushTool : BTTool<BTMouseTracker>
{
@private
    NSSize _size;
    CGSize _sizeOscillation;
    CGFloat _spacing;
    CGFloat _spacingOscillation;
    CGSize _positionOscillation;
    CGFloat _rotationAngle;
    CGFloat _rotationAngleOscillation;
    NSTimeInterval _touchInterval;
    
    id<BTCGImage> _image;
    
    CGEColorRef _color;
    CGEColorRef _alternateColor;
    NSData* _colorHSVAOscillation;
    NSData* _colorNativeOscillation;
    
    NSSize _userPatternPhase;
    
    NSArray* _overlays;
    CFMutableArrayRef _overlayLayers;
    CGContextRef _overlayLayersReferenceContext;
    
    id<BTCGImage> _colorizationImage;
    CGFloat _colorizationGrayMidPoint;
    CGColorRef _colorizationShadowColor;
    CGColorRef _colorizationLightColor;
    
    CGPathRef _shapePrototypePath;
    CGRect _shapePrototypePathBounds;
    
    CGImageRef _mask;
    
    CGFloat _opacity;
    CGFloat _opacityOscillation;
    
    CGColorRef _shadowColor;
    NSData* _shadowColorHSVAOscillation;
    NSData* _shadowColorNativeOscillation;
    
    CGSize _shadowOffset;
    CGFloat _shadowBlurRadius;
    
    CGImageRef _shadowMask;
    
    NSMapTable* _touchConsumers;
    
    CGELayerRef _padLayer;
    CGELayerRef _alternatePadLayer;
    CGContextRef _padLayerReferenceContext;
    CGContextRef _alternatePadLayerReferenceContext;
    CGSize _padLayerScale;
    CGSize _alternatePadLayerScale;
    
    id<BTCGImage> _shadowMaskInput;
    CGELayerRef _shadowMaskRGBLayer;
    
    id<BTCGImage> _maskInput;
    CGELayerRef _maskRGBLayer;
    
    CGImageRef _cachedShadowMask;
    CGSize _cachedShadowMaskSize;
    
    CGImageRef _cachedMask;
    CGSize _cachedMaskSize;
    
    NSPoint _imageStartPoint;
    unsigned char _guidingMode;
    
    NSTimer* _touchTimer;

    NSCursor* _willDrawCursor;
    NSCursor* _doDrawCursor;
    
	NSCursor* _willDrawShapeContourCursor;
	NSCursor* _doDrawShapeContourCursor;

	CGPathRef _willDrawCursorShapeContour;
	CGPathRef _doDrawCursorShapeContour;
	
	CGRect _willDrawCursorShapeContourBounds;
	CGRect _doDrawCursorShapeContourBounds;
    
    BOOL _padding : 1;
    BOOL _isPadFlippedHorizontally : 1;
    BOOL _isPadFlippedVertically : 1;
    
    BOOL _isTracking : 1;
    BOOL _isDrawing : 1;
    
    BOOL _isWillDrawCursorInitialized : 1;
    BOOL _isDoDrawCursorInitialized : 1;
    
    BOOL _isColorizationShadowColorInitialized : 1;
    BOOL _isColorizationLightColorInitialized : 1;
    
    BOOL _isUsingAlternateColor : 1;
}

- (NSCursor*)defaultWillDrawCursor;
- (NSCursor*)willDrawCursor;
- (void)setWillDrawCursor:(NSCursor*)willDrawCursor;

- (NSCursor*)defaultDoDrawCursor;
- (NSCursor*)doDrawCursor;
- (void)setDoDrawCursor:(NSCursor*)doDrawCursor;

- (id /* CGPathRef */)willDrawCursorShapeContour;
- (void)setWillDrawCursorShapeContour:(id /* CGPathRef */)path;

- (id /* CGPathRef */)doDrawCursorShapeContour;
- (void)setDoDrawCursorShapeContour:(id /* CGPathRef */)path;

- (NSSize)defaultSize;
- (NSSize)size;
- (void)setSize:(NSSize)size;

- (NSSize)sizeOscillation;
- (void)setSizeOscillation:(NSSize)sizeOscillation;

- (CGFloat)spacing;
- (void)setSpacing:(CGFloat)spacing;

- (CGFloat)spacingOscillation;
- (void)setSpacingOscillation:(CGFloat)spacingOscillation;

- (NSSize)positionOscillation;
- (void)setPositionOscillation:(NSSize)positionOscillation;

- (CGFloat)rotationAngle;
- (void)setRotationAngle:(CGFloat)rotationAngle;

- (CGFloat)rotationAngleOscillation;
- (void)setRotationAngleOscillation:(CGFloat)rotationAngleOscillation;

- (NSTimeInterval)touchInterval;
- (void)setTouchInterval:(NSTimeInterval)touchInterval;

// `image' & `color' attributes are mutually exclusive: only one of them works at a time and `image' has priority over `color'. If both `image' & `color' are set to nil then tool switches to the eraser mode.
- (id<BTCGImage>)image;
- (void)setImage:(id<BTCGImage>)image;
- (id /* BTPointer* with CGEColorRef */)color;
- (void)setColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)color;
- (id /* BTPointer* with CGEColorRef */)alternateColor;
- (void)setAlternateColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)alternateColor;

// Note: this pattern phase affects only user pattern CGEColor, it does not affect pattern CGColor nor CGColor-based CGEColor with underlying pattern CGColor (`CGContextSetPatternPhase()' should be used in latter case).
- (NSSize)userPatternPhase;
- (void)setUserPatternPhase:(NSSize)userPatternPhase;

// Brush mode only. If padding is YES, the receiver draws pre-rendered pad in each position. Otherwise, the receiver draws supplied graphics in each position thus allowing it (graphics) to be different each time.
- (BOOL)defaultPadding;
- (BOOL)padding;
- (void)setPadding:(BOOL)paddingFlag;

- (BOOL)isPadFlippedHorizontally;
- (void)setPadFlippedHorizontally:(BOOL)isPadFlippedHorizontally;

- (BOOL)isPadFlippedVertically;
- (void)setPadFlippedVertically:(BOOL)isPadFlippedVertically;

// Overlays are array of dictionaries with following key-value structure:
//      overlayImage -- CGImageRef -- overlay image;
//      overlayBlendMode -- NSString* -- blend mode for overlay drawings:
//              Normal -- corresponds kCGBlendModeNormal,
//              Multiply -- corresponds kCGBlendModeMultiply,
//              Screen -- corresponds kCGBlendModeScreen,
//              Overlay -- corresponds kCGBlendModeOverlay,
//              Darken -- corresponds kCGBlendModeDarken,
//              Lighten -- corresponds kCGBlendModeLighten,
//              Color Dodge -- corresponds kCGBlendModeColorDodge,
//              Color Burn -- corresponds kCGBlendModeColorBurn,
//              Soft Light -- corresponds kCGBlendModeSoftLight,
//              Hard Light -- corresponds kCGBlendModeHardLight,
//              Difference -- corresponds kCGBlendModeDifference,
//              Exclusion -- corresponds kCGBlendModeExclusion,
//              Hue -- corresponds kCGBlendModeHue,
//              Saturation -- corresponds kCGBlendModeSaturation,
//              Color -- corresponds kCGBlendModeColor,
//              Luminosity -- corresponds kCGBlendModeLuminosity.
- (NSArray*)overlays;
- (void)setOverlays:(NSArray*)overlays;

- (id<BTCGImage>)colorizationImage;
- (void)setColorizationImage:(id<BTCGImage>)colorizationImage;

- (CGFloat)defaultColorizationGrayMidPoint;
- (CGFloat)colorizationGrayMidPoint;
- (void)setColorizationGrayMidPoint:(CGFloat)colorizationGrayMidPoint;

// Colorization shadow color may not be pattern color.
- (id /* CGColorRef */)defaultColorizationShadowColor;
- (id /* CGColorRef */)colorizationShadowColor;
- (void)setColorizationShadowColor:(id /* CGColorRef */)colorizationShadowColor;

// Colorization light color may not be pattern color.
- (id /* CGColorRef */)defaultColorizationLightColor;
- (id /* CGColorRef */)colorizationLightColor;
- (void)setColorizationLightColor:(id /* CGColorRef */)colorizationLightColor;

// HSVA (Hue, Saturation, Value, Alpha) randomization precedes native randomization.
- (NSData*)colorHSVAOscillation;
- (void)setColorHSVAOscillation:(NSData*)colorHSVAOscillation;

- (NSData*)colorNativeOscillation;
- (void)setColorNativeOscillation:(NSData*)colorNativeOscillation;

- (id /* CGPathRef */)shapePrototypePath;
- (void)setShapePrototypePath:(id /* CGPathRef */)shapePrototypePath;

// Result mask is either CG image mask or CG image. If it is a CG image, then it is in the DeviceGray color space, does not have alpha, and is not masked by an image mask or masking color.
- (id /* CGImageRef */)mask;

// `mask' may be either CG image mask, CG image or an object conformed to BTCGImage protocol. If `mask' is a CG image, then it must be in the DeviceGray color space, may not have alpha, and may not be masked by an image mask or masking color.
- (void)setMask:(id /* CGImageRef or id<BTCGImage> */)mask;

- (CGFloat)defaultOpacity;
- (CGFloat)opacity;
- (void)setOpacity:(CGFloat)opacity;

- (CGFloat)opacityOscillation;
- (void)setOpacityOscillation:(CGFloat)opacityOscillation;

- (NSSize)shadowOffset;
- (void)setShadowOffset:(NSSize)shadowOffset;

- (CGFloat)shadowBlurRadius;
- (void)setShadowBlurRadius:(CGFloat)shadowBlurRadius;

// Result mask is either CG image mask or CG image. If it is a CG image, then it is in the DeviceGray color space, does not have alpha, and is not masked by an image mask or masking color.
- (id /* CGImageRef */)shadowMask;

// `shadowMask' may be either CG image mask, CG image or an object conformed to BTCGImage protocol. If `mask' is a CG image, then it must be in the DeviceGray color space, may not have alpha, and may not be masked by an image mask or masking color.
- (void)setShadowMask:(id /* CGImageRef or id<BTCGImage> */)shadowMask;

// Shadow color may not be pattern color.
- (id /* CGColorRef */)shadowColor;
- (void)setShadowColor:(id /* CGColorRef */)shadowColor;

- (NSData*)shadowColorHSVAOscillation;
- (void)setShadowColorHSVAOscillation:(NSData*)shadowColorHSVAOscillation;

- (NSData*)shadowColorNativeOscillation;
- (void)setShadowColorNativeOscillation:(NSData*)shadowColorNativeOscillation;

- (void)startRecordTouchesWithConsumer:(id<BTBrushTouchConsumer>)touchConsumer;
- (void)stopRecordTouchesWithConsumer:(id<BTBrushTouchConsumer>)touchConsumer;

// Records touches along polyline.
// Returns bounding rect.
- (CGRect)recordAlongPolylineWithPoints:(const CGPoint*)points count:(NSUInteger)count withAlternateColor:(BOOL)useAlternateColorFlag withPressure:(CGFloat)pressure to:(id<BTBrushTouchConsumer>)touchConsumer;

- (void)drawWithTouchProvider:(id<BTBrushTouchProvider>)touchProvider inContext:(CGContextRef)context withBounds:(CGRect)contextBounds;

@end

#pragma mark *** Brush Touch Consuming/Providing Protocols ***

@protocol BTBrushTouchConsumer<NSObject>

- (void)addTouchAtPoint:(CGPoint)position withSizeDeviation:(CGSize)sizeDeviation rotationAngleDeviation:(CGFloat)rotationAngleDeviation opacityDeviation:(CGFloat)opacityDeviation alternateColor:(BOOL)alternateColorFlag colorHSVADeviation:(NSData*)colorHSVADeviation colorNativeDeviation:(NSData*)colorNativeDeviation shadowColorHSVADeviation:(NSData*)shadowColorHSVADeviation shadowColorNativeDeviation:(NSData*)shadowColorNativeDeviation;

@end

@protocol BTBrushTouchProvider<NSObject>

- (NSUInteger)numberOfTouches;
- (void)position:(CGPoint*)positionPtr sizeDeviation:(CGSize*)sizeDeviationPtr rotationAngleDeviation:(CGFloat*)rotationAngleDeviationPtr opacityDeviation:(CGFloat*)opacityDeviationPtr alternateColor:(BOOL*)alternateColorFlagPtr colorHSVADeviation:(NSData**)colorHSVADeviationPtr colorNativeDeviation:(NSData**)colorNativeDeviationPtr shadowColorHSVADeviation:(NSData**)shadowColorHSVADeviationPtr shadowColorNativeDeviation:(NSData**)shadowColorNativeDeviationPtr forTouchAtIndex:(NSUInteger)index;

@end

#pragma mark *** Brush Touch Consuming/Providing Classes ***

@interface BTBrushTouchStorage : NSObject<BTBrushTouchConsumer, BTBrushTouchProvider, NSCoding, NSCopying>
{
@private
    NSMutableData* _touchDataOffsets;
    NSMutableData* _touchesData;
}

// Applies `transform' to all positions in the receiver.
- (void)applyAffineTransform:(CGAffineTransform)transform;

// *** Utility Routines ***

// `shadowBlurRadius' may be NAN: means shadow mask instead of regular shadow.
- (CGRect)boundingRectOfTouchAtIndex:(NSUInteger)index forToolSize:(CGSize)toolSize rotationAngle:(CGFloat)rotationAngle shadowOffset:(CGSize)shadowOffset shadowBlurRadius:(CGFloat)shadowBlurRadius;
- (CGRect)boundingRectForToolSize:(CGSize)toolSize rotationAngle:(CGFloat)rotationAngle shadowOffset:(CGSize)shadowOffset shadowBlurRadius:(CGFloat)shadowBlurRadius;

@end

#pragma mark *** Stages ***

/*!
    @const BTDrawingToolStage
    @abstract Tool is drawing.
    @discussion Deactivation type: abortive.
*/
#ifndef BTDrawingToolStage
    #define BTDrawingToolStage @"BTDrawingToolStage"
#endif
