/*!
    @header BTCGEBitmapLayer.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2007/04/17
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <Foundation/NSObject.h>
#import <SMKBitmapTools/BTCGEBitmapLayer.h>
#import <SMKBitmapTools/BTImage.h>

/*!
    @class BTCGEBitmapLayer
    @discussion "-prepareXEditing" & "-finalizeXEditing" are the same as "-beginXEditing" & "-endXEditing" except they don't save/restore context's state (if any). "-beginXEditing" & "-endXEditing" are implemented using corresponding "-prepareXEditing" / "-finalizeXEditing".
*/
@interface BTCGEBitmapLayer : NSObject<BTCGImage, BTCocoaImage, BTCGMutableImage, BTCocoaMutableImage, BTRawImage, BTRawMutableImage>
{
@private
    CGEBitmapLayerRef _bitmapLayer;
    NSData* _imageDecode;
    BTImageFormat _imageFormat;
    CMProfileRef _imageColorProfile;
    NSGraphicsContext* _layerGraphicsContext;
}

// `imageDecode' may be nil.
// `imageColorProfile' may be nil.
+ (id)bitmapLayerWithCGEBitmapLayer:(CGEBitmapLayerRef)bitmapLayer decode:(const CGFloat*)imageDecode format:(BTImageFormat)imageFormat colorProfile:(CMProfileRef)imageColorProfile;

// `imageDecode' may be nil.
// `imageColorProfile' may be nil.
- (id)initWithCGEBitmapLayer:(CGEBitmapLayerRef)bitmapLayer decode:(const CGFloat*)imageDecode format:(BTImageFormat)imageFormat colorProfile:(CMProfileRef)imageColorProfile; // Designated initializer.

- (CGEBitmapLayerRef)coreGraphicsBitmapLayer;

- (void)prepareCGEditing;
- (void)finalizeCGEditing;

- (void)prepareCocoaEditing;
- (void)finalizeCocoaEditing;

@end
