/*!
    @header BTCGMutableImage.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2007/01/31
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <Foundation/NSObject.h>
#import <SMKBitmapTools/BTImage.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>

/*!
    @class BTCGMutableImage
    @abstract Generic representation of a Core Graphics image & attached context.
    @discussion "-prepareXEditing" & "-finalizeXEditing" are the same as "-beginXEditing" & "-endXEditing" except they don't save/restore context's state (if any). "-beginXEditing" & "-endXEditing" are implemented using corresponding "-prepareXEditing" / "-finalizeXEditing".
*/
@interface BTCGMutableImage : NSObject<BTCGImage, BTCocoaImage, BTCGMutableImage, BTCocoaMutableImage>
{
@private
    CGImageRef _image;
    CGContextRef _imageContext;
    NSGraphicsContext* _layerGraphicsContext;
    
    BOOL _isImageValid : 1;
    BOOL _isEditing : 1;
}

- (id)initWithCGImage:(CGImageRef)image context:(CGContextRef)imageContext; // Designated initializer.

- (void)prepareCGEditing;
- (void)finalizeCGEditing;

- (void)prepareCocoaEditing;
- (void)finalizeCocoaEditing;

@end
