/*!
    @header BTCreateLassoSelectionTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2007/03/01
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>
#import <SMKCGExtensions/CGEFloat.h>

@class BTCreateFreeHandSelectionTool, BTCreateMagicWandSelectionTool;

/*!
    @class BTCreateLassoSelectionTool
    @discussion Stage cycle: BTWaitingToolStage -> BTCreatingSelectionToolStage.
*/
@interface BTCreateLassoSelectionTool : BTTool<BTToolDelegate>
{
@private
    NSString* _initialCreateSelectionMode;
    NSString* _pendingInitialCreateSelectionMode;
    BTCreateFreeHandSelectionTool* _createFreeHandSelectionTool;
    BTCreateMagicWandSelectionTool* _createMagicWandSelectionTool;
    CGFloat _tolerance;
    NSValue* /* with NSPoint */ _selectionStartPointValue;
    CGPathRef _selectionPath;
    
    NSCursor* _willCreateFreeHandSelectionCursor;
    NSCursor* _doCreateFreeHandSelectionCursor;
    NSCursor* _willCreateMagicLassoSelectionCursor;
    NSCursor* _doCreateMagicLassoSelectionCursor;
    
    BOOL _isCreateFreeHandSelectionToolActivated : 1;
    BOOL _isCreateMagicWandSelectionToolActivated : 1;
    BOOL _switchesMagicLassoAndMagicWandModes : 1;
    BOOL _switchesFreeHandAndMagicLassoModes : 1;
    BOOL _shouldSmoothSelectionPath : 1;
    BOOL _isWillCreateFreeHandSelectionCursorInitialized : 1;
    BOOL _isDoCreateFreeHandSelectionCursorInitialized : 1;
}

- (NSCursor*)defaultWillCreateFreeHandSelectionCursor;
- (NSCursor*)willCreateFreeHandSelectionCursor;
- (void)setWillCreateFreeHandSelectionCursor:(NSCursor*)willCreateFreeHandSelectionCursor;

- (NSCursor*)defaultDoCreateFreeHandSelectionCursor;
- (NSCursor*)doCreateFreeHandSelectionCursor;
- (void)setDoCreateFreeHandSelectionCursor:(NSCursor*)doCreateFreeHandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicWandSelectionCursor;
- (NSCursor*)willCreateMagicWandSelectionCursor;
- (void)setWillCreateMagicWandSelectionCursor:(NSCursor*)willCreateMagicWandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicLassoSelectionCursor;
- (NSCursor*)willCreateMagicLassoSelectionCursor;
- (void)setWillCreateMagicLassoSelectionCursor:(NSCursor*)willCreateMagicLassoSelectionCursor;

- (NSCursor*)defaultDoCreateMagicLassoSelectionCursor;
- (NSCursor*)doCreateMagicLassoSelectionCursor;
- (void)setDoCreateMagicLassoSelectionCursor:(NSCursor*)doCreateMagicLassoSelectionCursor;

- (CGFloat)tolerance;
- (void)setTolerance:(CGFloat)tolerance;

- (NSString*)initialCreateSelectionMode;
- (void)setInitialCreateSelectionMode:(NSString*)initialCreateSelectionMode;

- (NSString*)currentCreateSelectionMode;

- (NSValue* /* with NSPoint */)selectionStartPoint;
- (id /* CGPathRef */)selectionPath;

- (IBAction)selectAllArea:(id)sender;

@end

#pragma mark *** Stages ***

/*!
    @const BTCreatingSelectionToolStage
    @abstract Tool is creating selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTCreatingSelectionToolStage
    #define BTCreatingSelectionToolStage @"BTCreatingSelectionToolStage"
#endif

#pragma mark *** Create Selection Modes ***

/*!
    @const BTFreeHandCreateSelectionMode
*/
#ifndef BTFreeHandCreateSelectionMode
    #define BTFreeHandCreateSelectionMode @"BTFreeHandCreateSelectionMode"
#endif

/*!
    @const BTMagicWandCreateSelectionMode
*/
#ifndef BTMagicWandCreateSelectionMode
    #define BTMagicWandCreateSelectionMode @"BTMagicWandCreateSelectionMode"
#endif

/*!
    @const BTMagicLassoCreateSelectionMode
*/
#ifndef BTMagicLassoCreateSelectionMode
    #define BTMagicLassoCreateSelectionMode @"BTMagicLassoCreateSelectionMode"
#endif
