/*!
    @header BTCreateSelectionTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2008/03/18
    @copyright Copyright (c) 2008 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>
#import <SMKCGExtensions/CGEFloat.h>

@class BTCreateShapeSelectionTool, BTCreateLassoSelectionTool;

/*!
    @class BTCreateSelectionTool
    @discussion Stage cycle: BTWaitingToolStage -> BTCreatingSelectionToolStage.
*/
@interface BTCreateSelectionTool : BTTool<BTToolDelegate>
{
@private
    NSString* _initialCreateSelectionMode;
    BTCreateShapeSelectionTool* _createShapeSelectionTool;
    BTCreateLassoSelectionTool* _createLassoSelectionTool;
    
    BOOL _ignoresCurrentCreateSelectionModeChange : 1;
}

- (NSCursor*)defaultWillCreateFreeHandSelectionCursor;
- (NSCursor*)willCreateFreeHandSelectionCursor;
- (void)setWillCreateFreeHandSelectionCursor:(NSCursor*)willCreateFreeHandSelectionCursor;

- (NSCursor*)defaultDoCreateFreeHandSelectionCursor;
- (NSCursor*)doCreateFreeHandSelectionCursor;
- (void)setDoCreateFreeHandSelectionCursor:(NSCursor*)doCreateFreeHandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicWandSelectionCursor;
- (NSCursor*)willCreateMagicWandSelectionCursor;
- (void)setWillCreateMagicWandSelectionCursor:(NSCursor*)willCreateMagicWandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicLassoSelectionCursor;
- (NSCursor*)willCreateMagicLassoSelectionCursor;
- (void)setWillCreateMagicLassoSelectionCursor:(NSCursor*)willCreateMagicLassoSelectionCursor;

- (NSCursor*)defaultDoCreateMagicLassoSelectionCursor;
- (NSCursor*)doCreateMagicLassoSelectionCursor;
- (void)setDoCreateMagicLassoSelectionCursor:(NSCursor*)doCreateMagicLassoSelectionCursor;

- (NSCursor*)defaultWillCreateShapeSelectionCursor;
- (NSCursor*)willCreateShapeSelectionCursor;
- (void)setWillCreateShapeSelectionCursor:(NSCursor*)willCreateShapeSelectionCursor;

- (NSCursor*)defaultDoCreateShapeSelectionCursor;
- (NSCursor*)doCreateShapeSelectionCursor;
- (void)setDoCreateShapeSelectionCursor:(NSCursor*)doCreateShapeSelectionCursor;

- (NSString*)initialCreateSelectionMode;
- (void)setInitialCreateSelectionMode:(NSString*)initialCreateSelectionMode;

- (NSString*)currentCreateSelectionMode;

- (CGFloat)tolerance;
- (void)setTolerance:(CGFloat)tolerance;

- (id /* CGPathRef */)selectionShapePrototypePath;
- (void)setSelectionShapePrototypePath:(id /* CGPathRef */)shapePrototypePath;

// Proportion = height / width.
- (CGFloat)defaultAlternateProportion;
- (CGFloat)alternateProportion;
- (void)setAlternateProportion:(CGFloat)alternateProportion;

- (BOOL)defaultProportionalModePrimary;
- (BOOL)isProportionalModePrimary;
- (void)setProportionalModePrimary:(BOOL)isProportionalModePrimary;

- (NSValue* /* with NSPoint */)selectionStartPoint;
- (id /* CGPathRef */)selectionPath;

@end

#pragma mark *** Stages ***

/*!
    @const BTCreatingSelectionToolStage
    @abstract Tool is creating selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTCreatingSelectionToolStage
    #define BTCreatingSelectionToolStage @"BTCreatingSelectionToolStage"
#endif

#pragma mark *** Create Selection Modes ***

/*!
    @const BTFreeHandCreateSelectionMode
*/
#ifndef BTFreeHandCreateSelectionMode
    #define BTFreeHandCreateSelectionMode @"BTFreeHandCreateSelectionMode"
#endif

/*!
    @const BTMagicWandCreateSelectionMode
*/
#ifndef BTMagicWandCreateSelectionMode
    #define BTMagicWandCreateSelectionMode @"BTMagicWandCreateSelectionMode"
#endif

/*!
    @const BTMagicLassoCreateSelectionMode
*/
#ifndef BTMagicLassoCreateSelectionMode
    #define BTMagicLassoCreateSelectionMode @"BTMagicLassoCreateSelectionMode"
#endif

/*!
    @const BTShapeCreateSelectionMode
*/
#ifndef BTShapeCreateSelectionMode
    #define BTShapeCreateSelectionMode @"BTShapeCreateSelectionMode"
#endif
