/*!
    @header BTCreateShapeSelectionTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/19/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>

@class BTPointsCreationTool;

/*!
    @class BTCreateShapeSelectionTool
    @discussion Stage cycle: BTWaitingToolStage -> BTCreatingSelectionToolStage.
*/
@interface BTCreateShapeSelectionTool : BTTool<BTToolDelegate, BTMouseTracker>
{
@private
    CGPathRef _selectionShapePrototypePath;
    CGRect _selectionShapePrototypePathBounds;
    BTToolOption _centeringOption;
    NSPoint _selectionStartPoint;
    NSPoint _selectionEndPoint;
    NSRect _feedbackInvalidationRect;
    NSUInteger _effectiveModifierFlags;
    BTPointsCreationTool* _pointsCreationTool;
    
    BOOL _isShapePrototypePathInitialized : 1;
    BOOL _isSelectionPathDefined : 1;
    BOOL _isSelectionPathBeingCreated : 1;
}

- (NSCursor*)defaultWillCreateSelectionCursor;
- (NSCursor*)willCreateSelectionCursor;
- (void)setWillCreateSelectionCursor:(NSCursor*)willCreateSelectionCursor;

- (NSCursor*)defaultDoCreateSelectionCursor;
- (NSCursor*)doCreateSelectionCursor;
- (void)setDoCreateSelectionCursor:(NSCursor*)doCreateSelectionCursor;

- (id /* CGPathRef */)defaultSelectionShapePrototypePath;
- (id /* CGPathRef */)selectionShapePrototypePath;
- (void)setSelectionShapePrototypePath:(id /* CGPathRef */)selectionShapePrototypePath;

- (BTToolOption)defaultCenteringOption;
- (BTToolOption)centeringOption;
- (void)setCenteringOption:(BTToolOption)option;

// Proportion = height / width.
- (CGFloat)defaultAlternateProportion;
- (CGFloat)alternateProportion;
- (void)setAlternateProportion:(CGFloat)alternateProportion;

- (BOOL)defaultProportionalModePrimary;
- (BOOL)isProportionalModePrimary;
- (void)setProportionalModePrimary:(BOOL)isProportionalModePrimary;

- (NSValue* /* with NSPoint */)selectionStartPoint;
- (id /* CGPathRef */)selectionPath;

@end

#pragma mark *** Stages ***

/*!
    @const BTCreatingSelectionToolStage
    @abstract Tool is creating selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTCreatingSelectionToolStage
    #define BTCreatingSelectionToolStage @"BTCreatingSelectionToolStage"
#endif
