/*!
    @header BTDraw3DShapeTool.h
    @project SMKBitmapTools
    @author Oleg Yaros
    @creation_date 2007/05/31
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>

@class BTPointsCreationTool;

/*!
    @class BTDraw3DShapeTool
    @discussion Stage cycle: BTWaitingToolStage -> BTDrawingToolStage.
*/
@interface BTDraw3DShapeTool : BTTool<BTToolDelegate, BTMouseTracker>
{
@protected
    CGPathRef _shape3DPrototypePath;
    CGMutablePathRef _shape3DPath;
    
    CGEColorRef _strokeColor;
    CGEColorRef _alternateStrokeColor;
    CGFloat _lineWidth;
    CGFloat _lineDashPhase;
    NSMutableData* _lineDash;

    CGRect _shape3DPrototypePathBounds;
    // work points
    NSPoint _startPoint;
    NSPoint _endPoint;
    NSPoint _frontShapeStartPoint;

    NSPoint _originalStartPoint;
    NSPoint _originalFrontStartPoint;
    // mouse tracking subtool
    BTPointsCreationTool* _pointsCreationTool;
    
    CGFloat _drawingLineWidth;
    NSUInteger _shapeLineCount;
    
    BOOL _isShapePrototypePathInitialized : 1;
    BOOL _shouldInvalidate3DShapePath : 1;
    BOOL _lastPointShouldBeCorrectedVertically : 1;
    BOOL _lastPointShouldBeCorrectedHorizontally : 1;
    BOOL _isStrokeColorInitialized : 1;
    BOOL _isUsingAlternateStrokeColor : 1;
}

- (NSCursor*)defaultWillDrawCursor;
- (NSCursor*)willDrawCursor;
- (void)setWillDrawCursor:(NSCursor*)willDrawCursor;

- (NSCursor*)defaultDoDrawCursor;
- (NSCursor*)doDrawCursor;
- (void)setDoDrawCursor:(NSCursor*)doDrawCursor;

- (CGFloat)defaultLineWidth;
- (CGFloat)lineWidth;
- (void)setLineWidth:(CGFloat)lineWidth;

- (id /* BTPointer* with CGEColorRef */)defaultStrokeColor;
- (id /* BTPointer* with CGEColorRef */)strokeColor;
- (void)setStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)strokeColor;

- (id /* BTPointer* with CGEColorRef */)alternateStrokeColor;
- (void)setAlternateStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)alternateStrokeColor;

- (CGFloat)defaultLineDashPhase;
- (CGFloat)lineDashPhase;
- (void)setLineDashPhase:(CGFloat)lineDashPhase;

- (NSArray*)defaultLineDash;
- (NSArray*)lineDash;
- (void)setLineDash:(NSArray*)lineDash;

- (id /* CGPathRef */)defaultShapePrototypePath;
- (id /* CGPathRef */)shapePrototypePath;
- (void)setShapePrototypePath:(id /* CGPathRef */)shapePrototypePath;

// Shape line count is equal to maximum between maximal number of shape intersections with a horizontal and maximal number of shape intersections with a vertical. This value is used to determine when shape bounds are too small to contain stroked shape and to adjust line width so stroked shape fits the bounds (and to avoid rendering artifacts of Quartz). The value may be 0: dynamic line width adjusting is turned off.
- (NSUInteger)defaultShapeLineCount;
- (NSUInteger)shapeLineCount;
- (void)setShapeLineCount:(NSUInteger)shapeLineCount;

@end

#pragma mark *** Stages ***

/*!
    @const BTDrawingToolStage
    @abstract Tool is drawing.
    @discussion Deactivation type: abortive.
*/
#ifndef BTDrawingToolStage
    #define BTDrawingToolStage @"BTDrawingToolStage"
#endif
