/*!
    @header BTDrawLineTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2007/02/07
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColor.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>
#import <SMKCGExtensions/CGEFloat.h>
#import <SMKCGExtensions/CGEColor.h>

@class NSData;
@class BTPointsCreationTool;

/*!
    @class BTDrawLineTool
    @discussion Stage cycle: BTWaitingToolStage -> BTDrawingToolStage.
*/
@interface BTDrawLineTool : BTTool<BTToolDelegate, BTMouseTracker>
{
@private
    BTToolOption _positioningOption;
    BTToolOption _centeringOption;
    CGEColorRef _strokeColor;
    BTToolOption _alternateStrokeColorOption;
    CGEColorRef _alternateStrokeColor;
    CGFloat _lineWidth;
    CGLineCap _lineCap;
    CGFloat _lineDashPhase;
    NSData* _lineDash;
    BTPointsCreationTool* _pointsCreationTool;
    
    NSPoint _startPoint;
    NSPoint _endPoint;
    NSRect _feedbackInvalidationRect;
    NSUInteger _effectiveModifierFlags;
    
    BOOL _isStrokeColorInitialized : 1;
}

- (NSCursor*)defaultWillDrawCursor;
- (NSCursor*)willDrawCursor;
- (void)setWillDrawCursor:(NSCursor*)trackingCursor;

- (NSCursor*)defaultDoDrawCursor;
- (NSCursor*)doDrawCursor;
- (void)setDoDrawCursor:(NSCursor*)doDrawCursor;

- (BTToolOption)defaultPositioningOption;
- (BTToolOption)positioningOption;
- (void)setPositioningOption:(BTToolOption)option;

- (BTToolOption)defaultCenteringOption;
- (BTToolOption)centeringOption;
- (void)setCenteringOption:(BTToolOption)option;

- (id /* BTPointer* with CGEColorRef */)defaultStrokeColor;
- (id /* BTPointer* with CGEColorRef */)strokeColor;
- (void)setStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)strokeColor;

- (BTToolOption)defaultAlternateStrokeColorOption;
- (BTToolOption)alternateStrokeColorOption;
- (void)setAlternateStrokeColorOption:(BTToolOption)option;

- (id /* BTPointer* with CGEColorRef */)alternateStrokeColor;
- (void)setAlternateStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)alternateStrokeColor;

- (CGFloat)defaultLineWidth;
- (CGFloat)lineWidth;
- (void)setLineWidth:(CGFloat)lineWidth;

- (NSString*)defaultLineCap;
- (NSString*)lineCap;
- (void)setLineCap:(NSString*)lineCap;

- (CGFloat)defaultLineDashPhase;
- (CGFloat)lineDashPhase;
- (void)setLineDashPhase:(CGFloat)lineDashPhase;

- (NSData*)defaultLineDash;
- (NSData*)lineDash;
- (void)setLineDash:(NSData*)lineDash;

@end

#pragma mark *** Stages ***

/*!
    @const BTDrawingToolStage
    @abstract Tool is drawing.
    @discussion Deactivation type: abortive.
*/
#ifndef BTDrawingToolStage
    #define BTDrawingToolStage @"BTDrawingToolStage"
#endif

#pragma mark *** Line Caps ***

/*!
    @const BTLineCapButt
*/
#define BTLineCapButt @"BTLineCapButt"

/*!
    @const BTLineCapRound
*/
#define BTLineCapRound @"BTLineCapRound"

/*!
    @const BTLineCapSquare
*/
#define BTLineCapSquare @"BTLineCapSquare"
