/*!
    @header BTDrawShapeTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2007/02/02
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>
#import <SMKCGExtensions/CGEFloat.h>
#import <SMKCGExtensions/CGEColor.h>

@class BTCreateShapeSelectionTool;


/*!
    @class BTDrawShapeTool
    @discussion Stage cycle: BTWaitingToolStage -> BTDrawingToolStage.
*/
@interface BTDrawShapeTool : BTTool<BTToolDelegate, BTMouseTracker>
{
@private
    BTToolOption _positioningOption;
    CGEColorRef _strokeColor;
    BTToolOption _alternateStrokeColorOption;
    CGEColorRef _alternateStrokeColor;
    CGEColorRef _fillColor;
    BTToolOption _alternateFillColorOption;
    CGEColorRef _alternateFillColor;
    CGFloat _lineWidth;
    CGLineJoin _lineJoin;
    CGFloat _lineDashPhase;
    NSMutableData* _lineDash;
    NSUInteger _shapeLineCount;
    
    BTCreateShapeSelectionTool* _createShapeSelectionTool;
    
    NSUInteger _effectiveModifierFlags;
    
    BOOL _isStrokeColorInitialized : 1;
    BOOL _isFillColorInitialized : 1;
}

- (NSCursor*)defaultWillDrawCursor;
- (NSCursor*)willDrawCursor;
- (void)setWillDrawCursor:(NSCursor*)trackingCursor;

- (NSCursor*)defaultDoDrawCursor;
- (NSCursor*)doDrawCursor;
- (void)setDoDrawCursor:(NSCursor*)doDrawCursor;

- (BTToolOption)defaultPositioningOption;
- (BTToolOption)positioningOption;
- (void)setPositioningOption:(BTToolOption)option;

- (BTToolOption)defaultCenteringOption;
- (BTToolOption)centeringOption;
- (void)setCenteringOption:(BTToolOption)option;

- (id /* BTPointer* with CGEColorRef */)defaultStrokeColor;
- (id /* BTPointer* with CGEColorRef */)strokeColor;
- (void)setStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)strokeColor;

- (BTToolOption)defaultAlternateStrokeColorOption;
- (BTToolOption)alternateStrokeColorOption;
- (void)setAlternateStrokeColorOption:(BTToolOption)option;

- (id /* BTPointer* with CGEColorRef */)alternateStrokeColor;
- (void)setAlternateStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)alternateStrokeColor;

- (id /* BTPointer* with CGEColorRef */)defaultFillColor;
- (id /* BTPointer* with CGEColorRef */)fillColor;
- (void)setFillColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)fillColor;

- (BTToolOption)defaultAlternateFillColorOption;
- (BTToolOption)alternateFillColorOption;
- (void)setAlternateFillColorOption:(BTToolOption)option;

- (id /* BTPointer* with CGEColorRef */)alternateFillColor;
- (void)setAlternateFillColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)alternateFillColor;

- (CGFloat)defaultLineWidth;
- (CGFloat)lineWidth;
- (void)setLineWidth:(CGFloat)lineWidth;

- (NSString*)defaultLineJoin;
- (NSString*)lineJoin;
- (void)setLineJoin:(NSString*)lineJoin;

- (CGFloat)defaultLineDashPhase;
- (CGFloat)lineDashPhase;
- (void)setLineDashPhase:(CGFloat)lineDashPhase;

- (NSArray*)defaultLineDash;
- (NSArray*)lineDash;
- (void)setLineDash:(NSArray*)lineDash;

- (id /* CGPathRef */)defaultShapePrototypePath;
- (id /* CGPathRef */)shapePrototypePath;
- (void)setShapePrototypePath:(id /* CGPathRef */)shapePrototypePath;

// Shape line count is equal to maximum between maximal number of shape intersections with a horizontal and maximal number of shape intersections with a vertical. This value is used to determine when shape bounds are too small to contain stroked shape and to adjust line width so stroked shape fits the bounds (and to avoid rendering artifacts of Quartz). The value may be 0: dynamic line width adjusting is turned off.
- (NSUInteger)defaultShapeLineCount;
- (NSUInteger)shapeLineCount;
- (void)setShapeLineCount:(NSUInteger)shapeLineCount;

@end


#pragma mark *** Stages ***

/*!
    @const BTDrawingToolStage
    @abstract Tool is drawing.
    @discussion Deactivation type: abortive.
*/
#ifndef BTDrawingToolStage
    #define BTDrawingToolStage @"BTDrawingToolStage"
#endif

#pragma mark *** Line Joins ***

/*!
    @const BTLineJoinMiter
*/
#define BTLineJoinMiter @"BTLineJoinMiter"

/*!
    @const BTLineJoinRound
*/
#define BTLineJoinRound @"BTLineJoinRound"

/*!
    @const BTLineJoinBevel
*/
#define BTLineJoinBevel @"BTLineJoinBevel"
