/*!
    @header BTModifySelectionTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/19/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGAffineTransform.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>

/*!
    @class BTModifySelectionTool
    @discussion Stage cycle: BTWaitingToolStage <-> BTModifyingSelectionToolStage.
*/
@interface BTModifySelectionTool : BTTool<BTMouseTracker>
{
@private
    CGPathRef _selectionPath;
    NSRect _selectionPathBounds;
    
    CGPoint _startImagePoint;
    CGPoint _startSelectionPoint;
    CGPoint _lastImagePoint;
    unsigned char _guidingMode;
    CGSize _deltaFactor;
    CGPoint _selectionOperationCenter;
    CGAffineTransform _baseSelectionAffineTransform;
    CGAffineTransform _selectionAffineTransform;
    
    NSString* _trackingCursorName;
    
    NSCursor* _willMoveSelectionCursor;
    NSCursor* _doMoveSelectionCursor;
    
    NSCursor* _willResizeBottomLeftSelectionCursor;
    NSCursor* _doResizeBottomLeftSelectionCursor;
    NSCursor* _willResizeCenterLeftSelectionCursor;
    NSCursor* _doResizeCenterLeftSelectionCursor;
    NSCursor* _willResizeTopLeftSelectionCursor;
    NSCursor* _doResizeTopLeftSelectionCursor;
    NSCursor* _willResizeTopCenterSelectionCursor;
    NSCursor* _doResizeTopCenterSelectionCursor;
    NSCursor* _willResizeTopRightSelectionCursor;
    NSCursor* _doResizeTopRightSelectionCursor;
    NSCursor* _willResizeCenterRightSelectionCursor;
    NSCursor* _doResizeCenterRightSelectionCursor;
    NSCursor* _willResizeBottomRightSelectionCursor;
    NSCursor* _doResizeBottomRightSelectionCursor;
    NSCursor* _willResizeBottomCenterSelectionCursor;
    NSCursor* _doResizeBottomCenterSelectionCursor;
    
    NSCursor* _willRotateBottomLeftSelectionCursor;
    NSCursor* _doRotateBottomLeftSelectionCursor;
    NSCursor* _willRotateCenterLeftSelectionCursor;
    NSCursor* _doRotateCenterLeftSelectionCursor;
    NSCursor* _willRotateTopLeftSelectionCursor;
    NSCursor* _doRotateTopLeftSelectionCursor;
    NSCursor* _willRotateTopCenterSelectionCursor;
    NSCursor* _doRotateTopCenterSelectionCursor;
    NSCursor* _willRotateTopRightSelectionCursor;
    NSCursor* _doRotateTopRightSelectionCursor;
    NSCursor* _willRotateCenterRightSelectionCursor;
    NSCursor* _doRotateCenterRightSelectionCursor;
    NSCursor* _willRotateBottomRightSelectionCursor;
    NSCursor* _doRotateBottomRightSelectionCursor;
    NSCursor* _willRotateBottomCenterSelectionCursor;
    NSCursor* _doRotateBottomCenterSelectionCursor;
    
    CGFloat _selectionFeedbackLineDashPhase;
    NSTimer* _selectionFeedbackLineDashPhaseTimer;
    
    BOOL _isTracking : 1;
    BOOL _isProportionalModePrimaryForCorners : 1;
    BOOL _isProportionalModePrimaryForSides : 1;
    BOOL _isBaseSelectionFlippedHorizontally : 1;
    BOOL _isSelectionFlippedHorizontally : 1;
    BOOL _isWillMoveSelectionCursorInitialized : 1;
    BOOL _isDoMoveSelectionCursorInitialized : 1;
    BOOL _isWillResizeBottomLeftSelectionCursorInitialized : 1;
    BOOL _isDoResizeBottomLeftSelectionCursorInitialized : 1;
    BOOL _isWillResizeCenterLeftSelectionCursorInitialized : 1;
    BOOL _isDoResizeCenterLeftSelectionCursorInitialized : 1;
    BOOL _isWillResizeTopLeftSelectionCursorInitialized : 1;
    BOOL _isDoResizeTopLeftSelectionCursorInitialized : 1;
    BOOL _isWillResizeTopCenterSelectionCursorInitialized : 1;
    BOOL _isDoResizeTopCenterSelectionCursorInitialized : 1;
    BOOL _isWillResizeTopRightSelectionCursorInitialized : 1;
    BOOL _isDoResizeTopRightSelectionCursorInitialized : 1;
    BOOL _isWillResizeCenterRightSelectionCursorInitialized : 1;
    BOOL _isDoResizeCenterRightSelectionCursorInitialized : 1;
    BOOL _isWillResizeBottomRightSelectionCursorInitialized : 1;
    BOOL _isDoResizeBottomRightSelectionCursorInitialized : 1;
    BOOL _isWillResizeBottomCenterSelectionCursorInitialized : 1;
    BOOL _isDoResizeBottomCenterSelectionCursorInitialized : 1;
    BOOL _isWillRotateBottomLeftSelectionCursorInitialized : 1;
    BOOL _isDoRotateBottomLeftSelectionCursorInitialized : 1;
    BOOL _isWillRotateCenterLeftSelectionCursorInitialized : 1;
    BOOL _isDoRotateCenterLeftSelectionCursorInitialized : 1;
    BOOL _isWillRotateTopLeftSelectionCursorInitialized : 1;
    BOOL _isDoRotateTopLeftSelectionCursorInitialized : 1;
    BOOL _isWillRotateTopCenterSelectionCursorInitialized : 1;
    BOOL _isDoRotateTopCenterSelectionCursorInitialized : 1;
    BOOL _isWillRotateTopRightSelectionCursorInitialized : 1;
    BOOL _isDoRotateTopRightSelectionCursorInitialized : 1;
    BOOL _isWillRotateCenterRightSelectionCursorInitialized : 1;
    BOOL _isDoRotateCenterRightSelectionCursorInitialized : 1;
    BOOL _isWillRotateBottomRightSelectionCursorInitialized : 1;
    BOOL _isDoRotateBottomRightSelectionCursorInitialized : 1;
    BOOL _isWillRotateBottomCenterSelectionCursorInitialized : 1;
    BOOL _isDoRotateBottomCenterSelectionCursorInitialized : 1;
    BOOL _areSelectionPathBoundsCalculated : 1;
    NSUInteger _selectionPart : 4;
    NSUInteger _selectionOperation : 2;
    NSUInteger _selectionAspectRatioMaintainingRule : 2;
}

- (NSCursor*)defaultWillMoveSelectionCursor;
- (NSCursor*)willMoveSelectionCursor;
- (void)setWillMoveSelectionCursor:(NSCursor*)willMoveSelectionCursor;

- (NSCursor*)defaultDoMoveSelectionCursor;
- (NSCursor*)doMoveSelectionCursor;
- (void)setDoMoveSelectionCursor:(NSCursor*)doMoveSelectionCursor;

- (NSCursor*)defaultWillResizeBottomLeftSelectionCursor;
- (NSCursor*)willResizeBottomLeftSelectionCursor;
- (void)setWillResizeBottomLeftSelectionCursor:(NSCursor*)willResizeBottomLeftSelectionCursor;

- (NSCursor*)defaultDoResizeBottomLeftSelectionCursor;
- (NSCursor*)doResizeBottomLeftSelectionCursor;
- (void)setDoResizeBottomLeftSelectionCursor:(NSCursor*)doResizeBottomLeftSelectionCursor;

- (NSCursor*)defaultWillResizeCenterLeftSelectionCursor;
- (NSCursor*)willResizeCenterLeftSelectionCursor;
- (void)setWillResizeCenterLeftSelectionCursor:(NSCursor*)willResizeCenterLeftSelectionCursor;

- (NSCursor*)defaultDoResizeCenterLeftSelectionCursor;
- (NSCursor*)doResizeCenterLeftSelectionCursor;
- (void)setDoResizeCenterLeftSelectionCursor:(NSCursor*)doResizeCenterLeftSelectionCursor;

- (NSCursor*)defaultWillResizeTopLeftSelectionCursor;
- (NSCursor*)willResizeTopLeftSelectionCursor;
- (void)setWillResizeTopLeftSelectionCursor:(NSCursor*)willResizeTopLeftSelectionCursor;

- (NSCursor*)defaultDoResizeTopLeftSelectionCursor;
- (NSCursor*)doResizeTopLeftSelectionCursor;
- (void)setDoResizeTopLeftSelectionCursor:(NSCursor*)doResizeTopLeftSelectionCursor;

- (NSCursor*)defaultWillResizeTopCenterSelectionCursor;
- (NSCursor*)willResizeTopCenterSelectionCursor;
- (void)setWillResizeTopCenterSelectionCursor:(NSCursor*)willResizeTopCenterSelectionCursor;

- (NSCursor*)defaultDoResizeTopCenterSelectionCursor;
- (NSCursor*)doResizeTopCenterSelectionCursor;
- (void)setDoResizeTopCenterSelectionCursor:(NSCursor*)doResizeTopCenterSelectionCursor;

- (NSCursor*)defaultWillResizeTopRightSelectionCursor;
- (NSCursor*)willResizeTopRightSelectionCursor;
- (void)setWillResizeTopRightSelectionCursor:(NSCursor*)willResizeTopRightSelectionCursor;

- (NSCursor*)defaultDoResizeTopRightSelectionCursor;
- (NSCursor*)doResizeTopRightSelectionCursor;
- (void)setDoResizeTopRightSelectionCursor:(NSCursor*)doResizeTopRightSelectionCursor;

- (NSCursor*)defaultWillResizeCenterRightSelectionCursor;
- (NSCursor*)willResizeCenterRightSelectionCursor;
- (void)setWillResizeCenterRightSelectionCursor:(NSCursor*)willResizeCenterRightSelectionCursor;

- (NSCursor*)defaultDoResizeCenterRightSelectionCursor;
- (NSCursor*)doResizeCenterRightSelectionCursor;
- (void)setDoResizeCenterRightSelectionCursor:(NSCursor*)doResizeCenterRightSelectionCursor;

- (NSCursor*)defaultWillResizeBottomRightSelectionCursor;
- (NSCursor*)willResizeBottomRightSelectionCursor;
- (void)setWillResizeBottomRightSelectionCursor:(NSCursor*)willResizeBottomRightSelectionCursor;

- (NSCursor*)defaultDoResizeBottomRightSelectionCursor;
- (NSCursor*)doResizeBottomRightSelectionCursor;
- (void)setDoResizeBottomRightSelectionCursor:(NSCursor*)doResizeBottomRightSelectionCursor;

- (NSCursor*)defaultWillResizeBottomCenterSelectionCursor;
- (NSCursor*)willResizeBottomCenterSelectionCursor;
- (void)setWillResizeBottomCenterSelectionCursor:(NSCursor*)willResizeBottomCenterSelectionCursor;

- (NSCursor*)defaultDoResizeBottomCenterSelectionCursor;
- (NSCursor*)doResizeBottomCenterSelectionCursor;
- (void)setDoResizeBottomCenterSelectionCursor:(NSCursor*)doResizeBottomCenterSelectionCursor;

- (NSCursor*)defaultWillRotateBottomLeftSelectionCursor;
- (NSCursor*)willRotateBottomLeftSelectionCursor;
- (void)setWillRotateBottomLeftSelectionCursor:(NSCursor*)willRotateBottomLeftSelectionCursor;

- (NSCursor*)defaultDoRotateBottomLeftSelectionCursor;
- (NSCursor*)doRotateBottomLeftSelectionCursor;
- (void)setDoRotateBottomLeftSelectionCursor:(NSCursor*)doRotateBottomLeftSelectionCursor;

- (NSCursor*)defaultWillRotateCenterLeftSelectionCursor;
- (NSCursor*)willRotateCenterLeftSelectionCursor;
- (void)setWillRotateCenterLeftSelectionCursor:(NSCursor*)willRotateCenterLeftSelectionCursor;

- (NSCursor*)defaultDoRotateCenterLeftSelectionCursor;
- (NSCursor*)doRotateCenterLeftSelectionCursor;
- (void)setDoRotateCenterLeftSelectionCursor:(NSCursor*)doRotateCenterLeftSelectionCursor;

- (NSCursor*)defaultWillRotateTopLeftSelectionCursor;
- (NSCursor*)willRotateTopLeftSelectionCursor;
- (void)setWillRotateTopLeftSelectionCursor:(NSCursor*)willRotateTopLeftSelectionCursor;

- (NSCursor*)defaultDoRotateTopLeftSelectionCursor;
- (NSCursor*)doRotateTopLeftSelectionCursor;
- (void)setDoRotateTopLeftSelectionCursor:(NSCursor*)doRotateTopLeftSelectionCursor;

- (NSCursor*)defaultWillRotateTopCenterSelectionCursor;
- (NSCursor*)willRotateTopCenterSelectionCursor;
- (void)setWillRotateTopCenterSelectionCursor:(NSCursor*)willRotateTopCenterSelectionCursor;

- (NSCursor*)defaultDoRotateTopCenterSelectionCursor;
- (NSCursor*)doRotateTopCenterSelectionCursor;
- (void)setDoRotateTopCenterSelectionCursor:(NSCursor*)doRotateTopCenterSelectionCursor;

- (NSCursor*)defaultWillRotateTopRightSelectionCursor;
- (NSCursor*)willRotateTopRightSelectionCursor;
- (void)setWillRotateTopRightSelectionCursor:(NSCursor*)willRotateTopRightSelectionCursor;

- (NSCursor*)defaultDoRotateTopRightSelectionCursor;
- (NSCursor*)doRotateTopRightSelectionCursor;
- (void)setDoRotateTopRightSelectionCursor:(NSCursor*)doRotateTopRightSelectionCursor;

- (NSCursor*)defaultWillRotateCenterRightSelectionCursor;
- (NSCursor*)willRotateCenterRightSelectionCursor;
- (void)setWillRotateCenterRightSelectionCursor:(NSCursor*)willRotateCenterRightSelectionCursor;

- (NSCursor*)defaultDoRotateCenterRightSelectionCursor;
- (NSCursor*)doRotateCenterRightSelectionCursor;
- (void)setDoRotateCenterRightSelectionCursor:(NSCursor*)doRotateCenterRightSelectionCursor;

- (NSCursor*)defaultWillRotateBottomRightSelectionCursor;
- (NSCursor*)willRotateBottomRightSelectionCursor;
- (void)setWillRotateBottomRightSelectionCursor:(NSCursor*)willRotateBottomRightSelectionCursor;

- (NSCursor*)defaultDoRotateBottomRightSelectionCursor;
- (NSCursor*)doRotateBottomRightSelectionCursor;
- (void)setDoRotateBottomRightSelectionCursor:(NSCursor*)doRotateBottomRightSelectionCursor;

- (NSCursor*)defaultWillRotateBottomCenterSelectionCursor;
- (NSCursor*)willRotateBottomCenterSelectionCursor;
- (void)setWillRotateBottomCenterSelectionCursor:(NSCursor*)willRotateBottomCenterSelectionCursor;

- (NSCursor*)defaultDoRotateBottomCenterSelectionCursor;
- (NSCursor*)doRotateBottomCenterSelectionCursor;
- (void)setDoRotateBottomCenterSelectionCursor:(NSCursor*)doRotateBottomCenterSelectionCursor;

- (BOOL)defaultProportionalModePrimaryForCorners;
- (BOOL)isProportionalModePrimaryForCorners;
- (void)setProportionalModePrimaryForCorners:(BOOL)isProportionalModePrimaryForCorners;

- (BOOL)defaultProportionalModePrimaryForSides;
- (BOOL)isProportionalModePrimaryForSides;
- (void)setProportionalModePrimaryForSides:(BOOL)isProportionalModePrimaryForSides;

- (id /* CGPathRef */)selectionPath;
- (void)setSelectionPath:(id /* CGPathRef */)selectionPath;

- (NSValue* /* with CGAffineTransform */)selectionAffineTransform;
- (void)setSelectionAffineTransform:(NSValue* /* with CGAffineTransform */)selectionAffineTransform;

- (BOOL)isSelectionFlippedHorizontally;
- (void)setSelectionFlippedHorizontally:(BOOL)selectionHorizontalFlipFlag;

// *** Operations (Active State Only) ***

// `isFlippingHorizontally' specifies whether to interpret flipping operation in `transform' as horizontal flip or as vertical flip with 180 degrees rotation.
- (void)setSelectionAffineTransform:(CGAffineTransform)transform flippedHorizontally:(BOOL)isFlippedHorizontally;
- (void)setSelectionAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;
- (void)pretransformSelectionUsingAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;
- (void)posttransformSelectionUsingAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;

- (void)flipSelectionHorizontally;
- (void)flipSelectionVertically;
- (void)scaleSelectionAboutCenterBy:(CGSize)scaleFactor;
- (void)rotateSelectionAboutCenterBy:(CGFloat)angle;
- (void)moveSelectionBy:(CGSize)offset;

// *** Actions (Active State Only) ***

- (IBAction)moveRight:(id)sender;
- (IBAction)moveLeft:(id)sender;
- (IBAction)moveUp:(id)sender;
- (IBAction)moveDown:(id)sender;

- (IBAction)nudgeRight:(id)sender;
- (IBAction)nudgeLeft:(id)sender;
- (IBAction)nudgeUp:(id)sender;
- (IBAction)nudgeDown:(id)sender;

- (IBAction)flipHorizontally:(id)sender;
- (IBAction)flipVertically:(id)sender;

- (IBAction)rotateBy90DegreesClockwise:(id)sender;
- (IBAction)rotateBy90DegreesCounterClockwise:(id)sender;

- (IBAction)unflip:(id)sender;
- (IBAction)unscale:(id)sender;
- (IBAction)unrotate:(id)sender;
- (IBAction)reset:(id)sender;

@end

#pragma mark *** Stages ***

/*!
    @const BTModifyingSelectionToolStage
    @abstract Tool is modifying selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTModifyingSelectionToolStage
    #define BTModifyingSelectionToolStage @"BTModifyingSelectionToolStage"
#endif

#pragma mark *** Undo Categories ***

/*!
    @const BTSelectionUndoCategory
    @discussion Undoable outputs: selectionAffineTransform, selectionFlippedHorizontally.
*/
#if !defined(BTSelectionUndoCategory)
    #define BTSelectionUndoCategory @"BTSelectionUndoCategory"
#endif // !defined(BTSelectionUndoCategory)
