/*!
    @header BTNSGeometry.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/11/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <Foundation/NSGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>
#import <SMKCGExtensions/CGEFloat.h>
#import <SMKBitmapTools/BTNSObjCRuntime.h>

FOUNDATION_STATIC_INLINE BOOL BTNSEqualPointsWithDelta(NSPoint point1, NSPoint point2, NSSize delta);
FOUNDATION_STATIC_INLINE BOOL BTNSEqualSizesWithDelta(NSSize size1, NSSize size2, NSSize delta);
FOUNDATION_STATIC_INLINE BOOL BTNSEqualRectsWithDelta(NSRect rect1, NSRect rect2, NSSize delta);

FOUNDATION_STATIC_INLINE NSRect BTNSMakeRectWithCorners(NSPoint corner1, NSPoint corner2);

// Note: NSIntegralRect returns NSZeroRect if input rect has zero width or height.
FOUNDATION_STATIC_INLINE NSRect BTNSIntegralRect(NSRect rect);

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

FOUNDATION_STATIC_INLINE BOOL BTNSEqualPointsWithDelta(NSPoint point1, NSPoint point2, NSSize delta)
{
    return (CGEFloatEqualWithDelta(point1.x, point2.x, delta.width) && CGEFloatEqualWithDelta(point1.y, point2.y, delta.height));
}

FOUNDATION_STATIC_INLINE BOOL BTNSEqualSizesWithDelta(NSSize size1, NSSize size2, NSSize delta)
{
    return (CGEFloatEqualWithDelta(size1.width, size2.width, delta.width) && CGEFloatEqualWithDelta(size1.height, size2.height, delta.height));
}

FOUNDATION_STATIC_INLINE BOOL BTNSEqualRectsWithDelta(NSRect rect1, NSRect rect2, NSSize delta)
{
    return (BTNSEqualPointsWithDelta(rect1.origin, rect2.origin, delta) && BTNSEqualSizesWithDelta(rect1.size, rect2.size, delta));
}

#pragma mark -

FOUNDATION_STATIC_INLINE NSRect BTNSMakeRectWithCorners(NSPoint corner1, NSPoint corner2)
{
    CGRect resultRect = CGERectMakeWithCorners(*(CGPoint*)&corner1, *(CGPoint*)&corner2);
    
    return *(NSRect*)&resultRect;
}

FOUNDATION_STATIC_INLINE NSRect BTNSIntegralRect(NSRect rect)
{
    CGRect resultRect = CGRectIntegral(*(CGRect*)&rect);
    
    return *(NSRect*)&resultRect;
}
