/*!
    @header BTPencilTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/27/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>
#import <SMKCGExtensions/CGEColor.h>
#import <Foundation/NSMapTable.h>

@protocol BTPencilStrokeConsumer, BTPencilStrokeProvider;

/*!
    @class BTPencilTool
    @discussion Stage cycle: BTWaitingToolStage -> BTDrawingToolStage.
*/
@interface BTPencilTool : BTTool<BTMouseTracker>
{
@private
    NSPoint _imageStartPoint;
    unsigned char _guidingMode;
    CGEColorRef _strokeColor;
    CGEColorRef _alternateStrokeColor;
    NSCursor* _willDrawCursor;
    NSCursor* _doDrawCursor;
    NSMapTable* _strokeConsumers;
    
    BOOL _isTracking : 1;
    BOOL _isDrawing : 1;
    BOOL _adjustsStrokeToMatchImagePoints : 1;
    BOOL _adjustsStrokeToMatchWindowPoints : 1;
    BOOL _isWillDrawCursorInitialized : 1;
    BOOL _isDoDrawCursorInitialized : 1;
    BOOL _isStrokeColorInitialized : 1;
    BOOL _isUsingAlternateStrokeColor : 1;
    BOOL _isColorPicked : 1;
    BOOL _isStroked : 1;
}

- (NSCursor*)defaultWillDrawCursor;
- (NSCursor*)willDrawCursor;
- (void)setWillDrawCursor:(NSCursor*)willDrawCursor;

- (NSCursor*)defaultDoDrawCursor;
- (NSCursor*)doDrawCursor;
- (void)setDoDrawCursor:(NSCursor*)doDrawCursor;

- (id /* BTPointer* with CGEColorRef */)defaultStrokeColor;
- (id /* BTPointer* with CGEColorRef */)strokeColor;
- (void)setStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)strokeColor;

- (id /* BTPointer* with CGEColorRef */)alternateStrokeColor;
- (void)setAlternateStrokeColor:(id /* CGColorRef or BTPointer* with CGEColorRef */)alternateStrokeColor;

// Result may be nil.
- (NSString*)defaultStrokeAdjustment;
- (NSString*)strokeAdjustment;
- (void)setStrokeAdjustment:(NSString*)strokeAdjustment; // `strokeAdjustment' may be nil.

- (void)startRecordStrokesWithConsumer:(id<BTPencilStrokeConsumer>)strokeConsumer;
- (void)stopRecordStrokesWithConsumer:(id<BTPencilStrokeConsumer>)strokeConsumer;

- (void)drawWithPointProvider:(id<BTPencilStrokeProvider>)strokeProvider inContext:(CGContextRef)context withBounds:(CGRect)contextBounds;

@end

#pragma mark *** Brush Touch Consuming/Providing Protocols ***

@protocol BTPencilStrokeConsumer<NSObject>

- (void)addPencilStrokeFromPoint:(CGPoint)startPoint toPoint:(CGPoint)endPoint;

@end

@protocol BTPencilStrokeProvider<NSObject>

- (NSUInteger)numberOfPencilStrokes;
- (void)getStartPoint:(CGPoint*)startPointPtr endtPoint:(CGPoint*)endPointPtr forPencilStrokeAtIndex:(NSUInteger)index;

@end

#pragma mark *** Stages ***

/*!
    @const BTDrawingToolStage
    @abstract Tool is picking color and / or drawing.
    @discussion Deactivation type: abortive.
*/
#ifndef BTDrawingToolStage
    #define BTDrawingToolStage @"BTDrawingToolStage"
#endif

#pragma mark *** Stroke Adjustments ***

/*!
    @const BTImagePointsStrokeAdjustment
*/
#define BTImagePointsStrokeAdjustment @"BTImagePointsStrokeAdjustment"

/*!
    @const BTWindowPointsStrokeAdjustment
*/
#define BTWindowPointsStrokeAdjustment @"BTWindowPointsStrokeAdjustment"

#pragma mark *** Undo Categories ***

/*!
    @const BTStrokeColorUndoCategory
    @discussion Undoable outputs: strokeColor, alternateStrokeColor.
*/
#define BTStrokeColorUndoCategory @"BTStrokeColorUndoCategory"
