/*!
    @header BTPointer.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/16/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <Foundation/NSObject.h>
#import <SMKBitmapTools/BTNSObjCRuntime.h>

/*!
    @class BTPointer
    @abstract Wrapper for adding retain counting and autorelease abilities to pointers.
    @discussion Examples of usage:
        [BTPointer autoreleasedPointer:&data withDisposer:&free];
        [BTPointer autoreleasedPointer:mapTable withDisposer:(void (*)(void*))&NSFreeMapTable];
        [BTPointer autoreleasedPointer:cfString withDisposer:(void (*)(void*))&CFRelease];
*/
@interface BTPointer : NSObject 
{
@private
    void* _pointer;
    void (*_disposer)(void*);
}

// Creates autoreleased BTPointer with `pointer' & returns `pointer'.
// `disposer' may be nil.
+ (void*)autoreleasedPointer:(void*)pointer withDisposer:(void (*)(void*))disposer;

// Creates & returns autoreleased BTPointer with `pointer'.
// `disposer' may be nil.
+ (id)wrapperForPointer:(void*)pointer withDisposer:(void (*)(void*))disposer;

// `disposer' may be nil.
- (id)initWithPointer:(void*)pointer disposer:(void (*)(void*))disposer; // Designated initializer.
- (id)initWithPointer:(void*)pointer;

- (void*)pointer;

@end
