/*!
    @header BTPointsCreationTool.h
    @project SMKBitmapTools
    @author Oleg Yaros
    @creation_date 2007/06/12
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <SMKBitmapTools/NSViewBTMouseTracking.h>


@class BTDataArray;
@class BTMutableDataArray;

/*!
    @class BTPointsCreationTool 
    @discussion Stage cycle: BTWaitingToolStage -> BTCollectingPointsToolStage.
*/
@interface BTPointsCreationTool : BTTool<BTMouseTracker>
{
@private
    BTMutableDataArray* _pointsDataContainer;
    NSPoint _lastUncorrectedPoint;
    
    NSCursor* _willStartPointsCollectingCursor;
    NSCursor* _doPickPointCursor;
    NSCursor* _doDragPointCursor;
    
    NSUInteger _maxPointsCount;
    NSString* _correctionMode;
    CGFloat _correctionDiagonalTangent;
    CGFloat _alternateCorrectionDiagonalTangent;
    
    BOOL _isCorrectionModePrimary : 1;
    
    BOOL _isTracking : 1;
    unsigned char _isInCorrectionMode : 2;
    
    BOOL _isWillStartPointsCollectingCursorInitialized : 1;
    BOOL _isDoPickPointCursorInitialized : 1;
    BOOL _isDoDragPointCursorInitialized : 1;
    
    BOOL _isStopCreationOnDoubleClick : 1;
    BOOL _isLastPointDynamic : 1;
    BOOL _isWaitingForClick : 1;
    BOOL _isPointCollectingStarted : 1;
}

/* tool cursors */
- (NSCursor*)defaultWillStartPointsCollectingCursor;
- (NSCursor*)willStartPointsCollectingCursor;
- (void)setWillStartPointsCollectingCursor:(NSCursor*)willStartPointsCollectingCursor;

- (NSCursor*)defaultDoPickPointCursor;
- (NSCursor*)doPickPointCursor;
- (void)setDoPickPointCursor:(NSCursor*)doPickPointCursor;

- (NSCursor*)defaultDoDragPointCursor;
- (NSCursor*)doDragPointCursor;
- (void)setDoDragPointCursor:(NSCursor*)doDragPointCursor;

/* tool settings */
- (BOOL)defaultIsLastPointDynamic;
- (BOOL)isLastPointDynamic;
/*!
    @method     setIsLastPointDynamic:
    @discussion YES means that last added point will be alive (contains current mouse pointer coordinates)
*/
- (void)setIsLastPointDynamic:(BOOL)isLastPointDynamic;

- (NSString*)defaultCollectingMode;
- (NSString*)collectingMode;
- (void)setCollectingMode:(NSString*)collectingMode;

- (NSUInteger)defaultMaxPointsCount;
- (NSUInteger)maxPointsCount;
/*!
    @method     setMaxPointsCount:
    @discussion Change maximal points count. double click will be interpreted as tool stopper when "unlimited count" value used (BTUnlimitedMaxPointsCount specified).
*/
- (void)setMaxPointsCount:(NSUInteger)maxPointsCount;

- (BOOL)defaultIsCorrectionModePrimary;
- (BOOL)isCorrectionModePrimary;
- (void)setIsCorrectionModePrimary:(BOOL)isCorrectionModePrimary;

- (NSString*)defaultCorrectionMode;
- (NSString*)correctionMode;
- (void)setCorrectionMode:(NSString*)correctionMode;

- (CGFloat)defaultCorrectionDiagonalTangent;
- (CGFloat)correctionDiagonalTangent;
- (void)setCorrectionDiagonalTangent:(CGFloat)correctionDiagonalTangent;

- (CGFloat)defaultAlternateCorrectionDiagonalTangent;
- (CGFloat)alternateCorrectionDiagonalTangent;
- (void)setAlternateCorrectionDiagonalTangent:(CGFloat)correctionDiagonalTangent;

- (BTDataArray* /* with NSPoint */)pointsArray;

- (IBAction)selectAllArea:(id)sender;

@end

/*!
    @constant BTUnlimitedMaxPointsCount
*/
extern const NSUInteger BTUnlimitedMaxPointsCount;

/*!
    @constant   BTCollectingDragPointsMode
    @discussion Two new points will be collected at starting and ending of mouse drag
*/

extern NSString* const BTCollectingDragPointsMode;

/*!
    @constant   BTCollectingPickPointsMode
    @discussion New point will be collected at mouse click
*/
extern NSString* const BTCollectingPickPointsMode;

#pragma mark *** Stages ***

/*!
    @const BTCollectingPointsToolStage
    @abstract Tool is collecting points.
    @discussion Deactivation type: abortive.
*/
#ifndef BTCollectingPointsToolStage
    #define BTCollectingPointsToolStage @"BTCollectingPointsToolStage"
#endif

#pragma mark *** Correction Modes ***

/*!
    @const BTCorrectionModeHorizontal
*/
#define BTCorrectionModeHorizontal @"BTCorrectionModeHorizontal"

/*!
    @const BTCorrectionModeVertical
*/
#define BTCorrectionModeVertical @"BTCorrectionModeVertical"

/*!
    @const BTCorrectionModeDiagonal
*/
#define BTCorrectionModeDiagonal @"BTCorrectionModeDiagonal"

/*!
    @const BTCorrectionModeHorizontalVertical
*/
#define BTCorrectionModeHorizontalVertical @"BTCorrectionModeHorizontalVertical"

/*!
    @const BTCorrectionModeHorizontalVerticalDiagonal
*/
#define BTCorrectionModeHorizontalVerticalDiagonal @"BTCorrectionModeHorizontalVerticalDiagonal"
