/*!
    @header BTRegionEditingTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/19/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColor.h>
#import <SMKCGExtensions/CGEFloat.h>

@protocol BTCGImage;
@class BTSelectionTool;

/*!
    @class BTRegionEditingTool
    @discussion Stage cycle: BTWaitingToolStage -> BTCreatingSelectionToolStage -> BTWaitingToPositionSelectionPathToolStage -> BTWaitingToModifySelectionToolStage <-> BTModifyingSelectionToolStage.
*/
@interface BTRegionEditingTool : BTTool<BTToolDelegate>
{
@private
    NSString* _operation;
    NSString* _backgroundMode;
    CGColorRef _backgroundColor;
    id<BTCGImage> _imageSelection;
    CGRect _imageSelectionBounds;
    CGAffineTransform _imageSelectionTransform;
    CGAffineTransform _selectionImageToPathTransform;
    
    BTSelectionTool* _selectionTool;
    
    NSDictionary* _modifySelectionKeyBindings;
    
    id _workImageRestorator;
    
    id<BTCGImage> _pendingImageSelection;
    CGRect _pendingImageSelectionBounds;
    CGAffineTransform _pendingImageSelectionTransform;
    
    NSString* _selectionPathPositioningUndoActionNamePrefix;
    
    BOOL _shouldCopy : 1;
    BOOL _shouldDelete : 1;
    BOOL _shouldPaste : 1;
    
    BOOL _didResetImageSelection : 1;
    BOOL _isSelectionPathPositioned : 1;
    BOOL _isSelectionPathPositioningPending : 1;
    BOOL _shouldRestoreWorkImage : 1;
    
    BOOL _areModifySelectionKeyBindingsInitialized : 1;
}

- (NSDictionary*)defaultModifySelectionKeyBindings;
- (NSDictionary*)modifySelectionKeyBindings;
- (void)setModifySelectionKeyBindings:(NSDictionary*)modifySelectionKeyBindings;

- (NSCursor*)defaultWillCreateFreeHandSelectionCursor;
- (NSCursor*)willCreateFreeHandSelectionCursor;
- (void)setWillCreateFreeHandSelectionCursor:(NSCursor*)willCreateFreeHandSelectionCursor;

- (NSCursor*)defaultDoCreateFreeHandSelectionCursor;
- (NSCursor*)doCreateFreeHandSelectionCursor;
- (void)setDoCreateFreeHandSelectionCursor:(NSCursor*)doCreateFreeHandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicWandSelectionCursor;
- (NSCursor*)willCreateMagicWandSelectionCursor;
- (void)setWillCreateMagicWandSelectionCursor:(NSCursor*)willCreateMagicWandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicLassoSelectionCursor;
- (NSCursor*)willCreateMagicLassoSelectionCursor;
- (void)setWillCreateMagicLassoSelectionCursor:(NSCursor*)willCreateMagicLassoSelectionCursor;

- (NSCursor*)defaultDoCreateMagicLassoSelectionCursor;
- (NSCursor*)doCreateMagicLassoSelectionCursor;
- (void)setDoCreateMagicLassoSelectionCursor:(NSCursor*)doCreateMagicLassoSelectionCursor;

- (NSCursor*)defaultWillCreateShapeSelectionCursor;
- (NSCursor*)willCreateShapeSelectionCursor;
- (void)setWillCreateShapeSelectionCursor:(NSCursor*)willCreateShapeSelectionCursor;

- (NSCursor*)defaultDoCreateShapeSelectionCursor;
- (NSCursor*)doCreateShapeSelectionCursor;
- (void)setDoCreateShapeSelectionCursor:(NSCursor*)doCreateShapeSelectionCursor;

- (NSCursor*)defaultWillMoveSelectionCursor;
- (NSCursor*)willMoveSelectionCursor;
- (void)setWillMoveSelectionCursor:(NSCursor*)willMoveSelectionCursor;

- (NSCursor*)defaultDoMoveSelectionCursor;
- (NSCursor*)doMoveSelectionCursor;
- (void)setDoMoveSelectionCursor:(NSCursor*)doMoveSelectionCursor;

- (NSCursor*)defaultWillResizeBottomLeftSelectionCursor;
- (NSCursor*)willResizeBottomLeftSelectionCursor;
- (void)setWillResizeBottomLeftSelectionCursor:(NSCursor*)willResizeBottomLeftSelectionCursor;

- (NSCursor*)defaultDoResizeBottomLeftSelectionCursor;
- (NSCursor*)doResizeBottomLeftSelectionCursor;
- (void)setDoResizeBottomLeftSelectionCursor:(NSCursor*)doResizeBottomLeftSelectionCursor;

- (NSCursor*)defaultWillResizeCenterLeftSelectionCursor;
- (NSCursor*)willResizeCenterLeftSelectionCursor;
- (void)setWillResizeCenterLeftSelectionCursor:(NSCursor*)willResizeCenterLeftSelectionCursor;

- (NSCursor*)defaultDoResizeCenterLeftSelectionCursor;
- (NSCursor*)doResizeCenterLeftSelectionCursor;
- (void)setDoResizeCenterLeftSelectionCursor:(NSCursor*)doResizeCenterLeftSelectionCursor;

- (NSCursor*)defaultWillResizeTopLeftSelectionCursor;
- (NSCursor*)willResizeTopLeftSelectionCursor;
- (void)setWillResizeTopLeftSelectionCursor:(NSCursor*)willResizeTopLeftSelectionCursor;

- (NSCursor*)defaultDoResizeTopLeftSelectionCursor;
- (NSCursor*)doResizeTopLeftSelectionCursor;
- (void)setDoResizeTopLeftSelectionCursor:(NSCursor*)doResizeTopLeftSelectionCursor;

- (NSCursor*)defaultWillResizeTopCenterSelectionCursor;
- (NSCursor*)willResizeTopCenterSelectionCursor;
- (void)setWillResizeTopCenterSelectionCursor:(NSCursor*)willResizeTopCenterSelectionCursor;

- (NSCursor*)defaultDoResizeTopCenterSelectionCursor;
- (NSCursor*)doResizeTopCenterSelectionCursor;
- (void)setDoResizeTopCenterSelectionCursor:(NSCursor*)doResizeTopCenterSelectionCursor;

- (NSCursor*)defaultWillResizeTopRightSelectionCursor;
- (NSCursor*)willResizeTopRightSelectionCursor;
- (void)setWillResizeTopRightSelectionCursor:(NSCursor*)willResizeTopRightSelectionCursor;

- (NSCursor*)defaultDoResizeTopRightSelectionCursor;
- (NSCursor*)doResizeTopRightSelectionCursor;
- (void)setDoResizeTopRightSelectionCursor:(NSCursor*)doResizeTopRightSelectionCursor;

- (NSCursor*)defaultWillResizeCenterRightSelectionCursor;
- (NSCursor*)willResizeCenterRightSelectionCursor;
- (void)setWillResizeCenterRightSelectionCursor:(NSCursor*)willResizeCenterRightSelectionCursor;

- (NSCursor*)defaultDoResizeCenterRightSelectionCursor;
- (NSCursor*)doResizeCenterRightSelectionCursor;
- (void)setDoResizeCenterRightSelectionCursor:(NSCursor*)doResizeCenterRightSelectionCursor;

- (NSCursor*)defaultWillResizeBottomRightSelectionCursor;
- (NSCursor*)willResizeBottomRightSelectionCursor;
- (void)setWillResizeBottomRightSelectionCursor:(NSCursor*)willResizeBottomRightSelectionCursor;

- (NSCursor*)defaultDoResizeBottomRightSelectionCursor;
- (NSCursor*)doResizeBottomRightSelectionCursor;
- (void)setDoResizeBottomRightSelectionCursor:(NSCursor*)doResizeBottomRightSelectionCursor;

- (NSCursor*)defaultWillResizeBottomCenterSelectionCursor;
- (NSCursor*)willResizeBottomCenterSelectionCursor;
- (void)setWillResizeBottomCenterSelectionCursor:(NSCursor*)willResizeBottomCenterSelectionCursor;

- (NSCursor*)defaultDoResizeBottomCenterSelectionCursor;
- (NSCursor*)doResizeBottomCenterSelectionCursor;
- (void)setDoResizeBottomCenterSelectionCursor:(NSCursor*)doResizeBottomCenterSelectionCursor;

- (NSCursor*)defaultWillRotateBottomLeftSelectionCursor;
- (NSCursor*)willRotateBottomLeftSelectionCursor;
- (void)setWillRotateBottomLeftSelectionCursor:(NSCursor*)willRotateBottomLeftSelectionCursor;

- (NSCursor*)defaultDoRotateBottomLeftSelectionCursor;
- (NSCursor*)doRotateBottomLeftSelectionCursor;
- (void)setDoRotateBottomLeftSelectionCursor:(NSCursor*)doRotateBottomLeftSelectionCursor;

- (NSCursor*)defaultWillRotateCenterLeftSelectionCursor;
- (NSCursor*)willRotateCenterLeftSelectionCursor;
- (void)setWillRotateCenterLeftSelectionCursor:(NSCursor*)willRotateCenterLeftSelectionCursor;

- (NSCursor*)defaultDoRotateCenterLeftSelectionCursor;
- (NSCursor*)doRotateCenterLeftSelectionCursor;
- (void)setDoRotateCenterLeftSelectionCursor:(NSCursor*)doRotateCenterLeftSelectionCursor;

- (NSCursor*)defaultWillRotateTopLeftSelectionCursor;
- (NSCursor*)willRotateTopLeftSelectionCursor;
- (void)setWillRotateTopLeftSelectionCursor:(NSCursor*)willRotateTopLeftSelectionCursor;

- (NSCursor*)defaultDoRotateTopLeftSelectionCursor;
- (NSCursor*)doRotateTopLeftSelectionCursor;
- (void)setDoRotateTopLeftSelectionCursor:(NSCursor*)doRotateTopLeftSelectionCursor;

- (NSCursor*)defaultWillRotateTopCenterSelectionCursor;
- (NSCursor*)willRotateTopCenterSelectionCursor;
- (void)setWillRotateTopCenterSelectionCursor:(NSCursor*)willRotateTopCenterSelectionCursor;

- (NSCursor*)defaultDoRotateTopCenterSelectionCursor;
- (NSCursor*)doRotateTopCenterSelectionCursor;
- (void)setDoRotateTopCenterSelectionCursor:(NSCursor*)doRotateTopCenterSelectionCursor;

- (NSCursor*)defaultWillRotateTopRightSelectionCursor;
- (NSCursor*)willRotateTopRightSelectionCursor;
- (void)setWillRotateTopRightSelectionCursor:(NSCursor*)willRotateTopRightSelectionCursor;

- (NSCursor*)defaultDoRotateTopRightSelectionCursor;
- (NSCursor*)doRotateTopRightSelectionCursor;
- (void)setDoRotateTopRightSelectionCursor:(NSCursor*)doRotateTopRightSelectionCursor;

- (NSCursor*)defaultWillRotateCenterRightSelectionCursor;
- (NSCursor*)willRotateCenterRightSelectionCursor;
- (void)setWillRotateCenterRightSelectionCursor:(NSCursor*)willRotateCenterRightSelectionCursor;

- (NSCursor*)defaultDoRotateCenterRightSelectionCursor;
- (NSCursor*)doRotateCenterRightSelectionCursor;
- (void)setDoRotateCenterRightSelectionCursor:(NSCursor*)doRotateCenterRightSelectionCursor;

- (NSCursor*)defaultWillRotateBottomRightSelectionCursor;
- (NSCursor*)willRotateBottomRightSelectionCursor;
- (void)setWillRotateBottomRightSelectionCursor:(NSCursor*)willRotateBottomRightSelectionCursor;

- (NSCursor*)defaultDoRotateBottomRightSelectionCursor;
- (NSCursor*)doRotateBottomRightSelectionCursor;
- (void)setDoRotateBottomRightSelectionCursor:(NSCursor*)doRotateBottomRightSelectionCursor;

- (NSCursor*)defaultWillRotateBottomCenterSelectionCursor;
- (NSCursor*)willRotateBottomCenterSelectionCursor;
- (void)setWillRotateBottomCenterSelectionCursor:(NSCursor*)willRotateBottomCenterSelectionCursor;

- (NSCursor*)defaultDoRotateBottomCenterSelectionCursor;
- (NSCursor*)doRotateBottomCenterSelectionCursor;
- (void)setDoRotateBottomCenterSelectionCursor:(NSCursor*)doRotateBottomCenterSelectionCursor;

// Initial create selection mode may be nil: selection creation is disabled and current selection path is used instead.
- (NSString*)initialCreateSelectionMode;
- (void)setInitialCreateSelectionMode:(NSString*)initialCreateSelectionMode;

// Current create selection mode may be nil: selection creation is disabled and current selection path is used instead.
- (NSString*)currentCreateSelectionMode;

- (CGFloat)selectionTolerance;
- (void)setSelectionTolerance:(CGFloat)tolerance;

- (id /* CGPathRef */)selectionShapePrototypePath;
- (void)setSelectionShapePrototypePath:(id /* CGPathRef */)shapePrototypePath;

// Proportion = height / width.
- (CGFloat)defaultAlternateProportion;
- (CGFloat)alternateProportion;
- (void)setAlternateProportion:(CGFloat)alternateProportion;

- (BOOL)defaultProportionalModePrimaryForCreation;
- (BOOL)isProportionalModePrimaryForCreation;
- (void)setProportionalModePrimaryForCreation:(BOOL)isProportionalModePrimaryForCreation;

- (BOOL)defaultProportionalModePrimaryForCorners;
- (BOOL)isProportionalModePrimaryForCorners;
- (void)setProportionalModePrimaryForCorners:(BOOL)isProportionalModePrimaryForCorners;

- (BOOL)defaultProportionalModePrimaryForSides;
- (BOOL)isProportionalModePrimaryForSides;
- (void)setProportionalModePrimaryForSides:(BOOL)isProportionalModePrimaryForSides;

- (NSString*)backgroundMode;
- (void)setBackgroundMode:(NSString*)backgroundMode;

- (id /* CGColorRef */)backgroundColor;
- (void)setBackgroundColor:(id /* CGColorRef */)backgroundColor;

- (NSString*)operation;
- (void)setOperation:(NSString*)operation;

- (id<BTCGImage>)imageSelection;
- (void)setImageSelection:(id<BTCGImage>)imageSelection;

// Bounds of selected image region in place it was copied or cut from work image or in place it was pasted (without transformations).
// Note: these may differ from bounds of selection path.
- (NSValue* /* with CGRect */)imageSelectionBounds;
- (void)setImageSelectionBounds:(NSValue* /* with CGRect */)imageSelectionBounds;

// To perform transformation of image selection relative to transformation of selection path use following formulae: I1 = I0 * (1 / P0) * P1, where I0 - old image selection transform, I1 - new image selection transform, P0 - old selection path transform, P1 - new selection path transform.
// Note: image selection transform is set to identity upon starting of selection modification.
- (NSValue* /* with CGAffineTransform */)imageSelectionTransform;
- (void)setImageSelectionTransform:(NSValue* /* with CGAffineTransform */)imageSelectionTransform;

// Selection path as it was created or set (without transformations).
- (id /* CGPathRef */)selectionPath;
- (void)setSelectionPath:(id /* CGPathRef */)selectionPath;

// To perform transformation of selection path relative to transformation of image selection use following formulae: P1 = P0 * (1 / I0) * I1, where I0 - old image selection transform, I1 - new image selection transform, P0 - old selection path transform, P1 - new selection path transform.
// Note: selection path transform is set to identity upon starting of selection modification.
- (NSValue* /* with CGAffineTransform */)selectionPathTransform;
- (void)setSelectionPathTransform:(NSValue* /* with CGAffineTransform */)selectionPathTransform;

// Note: selection path horizontal flip flag is set to NO upon starting of selection modification.
- (BOOL)isSelectionPathFlippedHorizontally;
- (void)setSelectionPathFlippedHorizontally:(BOOL)selectionPathHorizontalFlipFlag;

// *** Selection Modifying Stage Operations (Active State Only) ***

// `isFlippingHorizontally' specifies whether to interpret flipping operation in `transform' as horizontal flip or as vertical flip with 180 degrees rotation.
- (void)setSelectionAffineTransform:(CGAffineTransform)transform flippedHorizontally:(BOOL)isFlippedHorizontally;
- (void)setSelectionAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;
- (void)pretransformSelectionUsingAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;
- (void)posttransformSelectionUsingAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;

- (void)flipSelectionHorizontally;
- (void)flipSelectionVertically;
- (void)scaleSelectionAboutCenterBy:(CGSize)scaleFactor;
- (void)rotateSelectionAboutCenterBy:(CGFloat)angle;
- (void)moveSelectionBy:(CGSize)offset;

// *** Actions (Active State Only) ***

- (IBAction)cut:(id)sender;
- (IBAction)copy:(id)sender;
- (IBAction)paste:(id)sender;
- (IBAction)delete:(id)sender;
- (IBAction)stamp:(id)sender;
- (IBAction)finalize:(id)sender;
- (IBAction)copyIntoSelection:(id)sender;
- (IBAction)finalizeSelection:(id)sender;

@end

#pragma mark *** Stages ***

/*!
    @const BTCreatingSelectionToolStage
    @abstract Tool is creating selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTCreatingSelectionToolStage
    #define BTCreatingSelectionToolStage @"BTCreatingSelectionToolStage"
#endif

/*!
    @const BTWaitingToPositionSelectionPathToolStage
    @abstract Tool is waiting for user input to position selection path.
    @discussion Deactivation type: normal.
*/
#ifndef BTWaitingToPositionSelectionPathToolStage
    #define BTWaitingToPositionSelectionPathToolStage @"BTWaitingToPositionSelectionPathToolStage"
#endif

/*!
    @const BTWaitingToModifySelectionToolStage
    @abstract Tool is waiting for user input to modify selection.
    @discussion Deactivation type: normal.
*/
#ifndef BTWaitingToModifySelectionToolStage
    #define BTWaitingToModifySelectionToolStage @"BTWaitingToModifySelectionToolStage"
#endif

/*!
    @const BTModifyingSelectionToolStage
    @abstract Tool is modifying selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTModifyingSelectionToolStage
    #define BTModifyingSelectionToolStage @"BTModifyingSelectionToolStage"
#endif

#pragma mark *** Create Selection Modes ***

/*!
    @const BTFreeHandCreateSelectionMode
*/
#ifndef BTFreeHandCreateSelectionMode
    #define BTFreeHandCreateSelectionMode @"BTFreeHandCreateSelectionMode"
#endif

/*!
    @const BTMagicWandCreateSelectionMode
*/
#ifndef BTMagicWandCreateSelectionMode
    #define BTMagicWandCreateSelectionMode @"BTMagicWandCreateSelectionMode"
#endif

/*!
    @const BTMagicLassoCreateSelectionMode
*/
#ifndef BTMagicLassoCreateSelectionMode
    #define BTMagicLassoCreateSelectionMode @"BTMagicLassoCreateSelectionMode"
#endif

/*!
    @const BTShapeCreateSelectionMode
*/
#ifndef BTShapeCreateSelectionMode
    #define BTShapeCreateSelectionMode @"BTShapeCreateSelectionMode"
#endif

#pragma mark *** Operations ***

/*!
    @const BTCopyOperation
*/
#define BTCopyOperation @"BTCopyOperation"

/*!
    @const BTCutOperation
*/
#define BTCutOperation @"BTCutOperation"

/*!
    @const BTDeleteOperation
*/
#define BTDeleteOperation @"BTDeleteOperation"

/*!
    @const BTPasteOperation
*/
#define BTPasteOperation @"BTPasteOperation"

/*!
    @const BTCopyPasteOperation
*/
#define BTCopyPasteOperation @"BTCopyPasteOperation"

/*!
    @const BTCutPasteOperation
*/
#define BTCutPasteOperation @"BTCutPasteOperation"

#pragma mark *** Background Modes ***

/*!
    @const BTClearBackgroundMode
*/
#define BTClearBackgroundMode @"BTClearBackgroundMode"

/*!
    @const BTExternalColorBackgroundMode
*/
#define BTExternalColorBackgroundMode @"BTExternalColorBackgroundMode"

/*!
    @const BTStartPointColorBackgroundMode
*/
#define BTStartPointColorBackgroundMode @"BTStartPointColorBackgroundMode"

#pragma mark *** Undo Categories ***

/*!
    @const BTSelectionUndoCategory
    @discussion Undoable outputs: selectionPath, selectionPathTransform, selectionPathFlippedHorizontally, imageSelection, imageSelectionBounds, imageSelectionTransform.
*/
#if !defined(BTSelectionUndoCategory)
    #define BTSelectionUndoCategory @"BTSelectionUndoCategory"
#endif // !defined(BTSelectionUndoCategory)

#pragma mark *** Pasteboard Types ***

/*!
    @const BTArchivedImageRegionPboardType
    @abstract Archived image region: image itself with transformation and selection path with transformation.
    @discussion Data format is byte stream as follows:
        - header with version: "BTImageRegionArchive-1.0";
        - image bounds as 4 CFSwappedFloat64s (origin.x, origin.y, size.width, size.height);
        - image transform as 6 CFSwappedFloat64s (a, b, c, d, tx, ty);
        - selection path transform as 6 CFSwappedFloat64s (a, b, c, d, tx, ty);
        - flags as unsigned char:
                - bit 0: if selection path is flipped horizontally;
        - image data UTI string length as UInt64;
        - image data UTI string in UTF8 encoding (region editing tool supports UTIs which are supported by ImageIO.framework);
        - image data length as UInt64;
        - image data;
        - selection path as sequence of path command codes (as unsigned char) and points (as array of pairs of CFSwappedFloat64s) as follows:
                - move to: code 1, one point;
                - line to: code 2, one point;
                - quadratic curve to: code 3, control point and end point;
                - Bezier curve to: code 4, two control points and end point;
                - close sub-path: code 5, no points;
                - end path: code 0, no points.
*/
#define BTArchivedImageRegionPboardType @"BTArchivedImageRegionPboardType"

/*!
    @const BTCreateSelectionModePboardType
    @abstract Create selection mode region editing tool was in during copy / cut action. Actual only if selection creation was enabled.
    @discussion Data is string (use `-[NSPasteboard stringForType:]').
*/
#define BTCreateSelectionModePboardType @"BTCreateSelectionModePboardType"

/*!
    @const BTSelectionRectPrototypePboardType
    @abstract Selection rect prototype. Actual only if region editing tool was in mode `BTShapeCreateSelectionMode' during copy / cut action and shape prototype was rect.
    @discussion Data format is byte stream as follows:
        - rect as 4 CFSwappedFloat64s (origin.x, origin.y, size.width, size.height).
*/
#define BTSelectionRectPrototypePboardType @"BTSelectionRectPrototypePboardType"
