/*!
    @header BTSelectionTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2008/03/18
    @copyright Copyright (c) 2008 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTool.h>
#import <CoreFoundation/CFDictionary.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>
#import <SMKCGExtensions/CGEFloat.h>

@class BTWeakObjectProxy;
@class BTCreateSelectionTool, BTModifySelectionTool;

/*!
    @class BTSelectionTool 
    @discussion Stage cycle: BTWaitingToolStage -> BTCreatingSelectionToolStage -> BTWaitingToModifySelectionToolStage <-> BTModifyingSelectionToolStage.
*/
@interface BTSelectionTool : BTTool<BTToolDelegate>
{
@private
    BTCreateSelectionTool* _createSelectionTool;
    BTModifySelectionTool* _modifySelectionTool;
    BTWeakObjectProxy* _selectionToolProxy;
    CFMutableDictionaryRef _pendingSubtoolSwitchingUndoRegistrations;
    NSEvent* _cursorUpdateBlockingEvent;
    NSString* _lastCreateSelectionUndoActionName;
	NSValue* _selectionStartPoint;
    
    BOOL _isSelectionCreationEnabled : 1;
    BOOL _isExternalSelectionPath : 1;
    BOOL _isCursorUpdatePendingForCreateSelectionTool : 1;
    BOOL _ignoresCurrentCreateSelectionModeUpdate : 1;
}

- (NSCursor*)defaultWillCreateFreeHandSelectionCursor;
- (NSCursor*)willCreateFreeHandSelectionCursor;
- (void)setWillCreateFreeHandSelectionCursor:(NSCursor*)willCreateFreeHandSelectionCursor;

- (NSCursor*)defaultDoCreateFreeHandSelectionCursor;
- (NSCursor*)doCreateFreeHandSelectionCursor;
- (void)setDoCreateFreeHandSelectionCursor:(NSCursor*)doCreateFreeHandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicWandSelectionCursor;
- (NSCursor*)willCreateMagicWandSelectionCursor;
- (void)setWillCreateMagicWandSelectionCursor:(NSCursor*)willCreateMagicWandSelectionCursor;

- (NSCursor*)defaultWillCreateMagicLassoSelectionCursor;
- (NSCursor*)willCreateMagicLassoSelectionCursor;
- (void)setWillCreateMagicLassoSelectionCursor:(NSCursor*)willCreateMagicLassoSelectionCursor;

- (NSCursor*)defaultDoCreateMagicLassoSelectionCursor;
- (NSCursor*)doCreateMagicLassoSelectionCursor;
- (void)setDoCreateMagicLassoSelectionCursor:(NSCursor*)doCreateMagicLassoSelectionCursor;

- (NSCursor*)defaultWillCreateShapeSelectionCursor;
- (NSCursor*)willCreateShapeSelectionCursor;
- (void)setWillCreateShapeSelectionCursor:(NSCursor*)willCreateShapeSelectionCursor;

- (NSCursor*)defaultDoCreateShapeSelectionCursor;
- (NSCursor*)doCreateShapeSelectionCursor;
- (void)setDoCreateShapeSelectionCursor:(NSCursor*)doCreateShapeSelectionCursor;

- (NSCursor*)defaultWillMoveSelectionCursor;
- (NSCursor*)willMoveSelectionCursor;
- (void)setWillMoveSelectionCursor:(NSCursor*)willMoveSelectionCursor;

- (NSCursor*)defaultDoMoveSelectionCursor;
- (NSCursor*)doMoveSelectionCursor;
- (void)setDoMoveSelectionCursor:(NSCursor*)doMoveSelectionCursor;

- (NSCursor*)defaultWillResizeBottomLeftSelectionCursor;
- (NSCursor*)willResizeBottomLeftSelectionCursor;
- (void)setWillResizeBottomLeftSelectionCursor:(NSCursor*)willResizeBottomLeftSelectionCursor;

- (NSCursor*)defaultDoResizeBottomLeftSelectionCursor;
- (NSCursor*)doResizeBottomLeftSelectionCursor;
- (void)setDoResizeBottomLeftSelectionCursor:(NSCursor*)doResizeBottomLeftSelectionCursor;

- (NSCursor*)defaultWillResizeCenterLeftSelectionCursor;
- (NSCursor*)willResizeCenterLeftSelectionCursor;
- (void)setWillResizeCenterLeftSelectionCursor:(NSCursor*)willResizeCenterLeftSelectionCursor;

- (NSCursor*)defaultDoResizeCenterLeftSelectionCursor;
- (NSCursor*)doResizeCenterLeftSelectionCursor;
- (void)setDoResizeCenterLeftSelectionCursor:(NSCursor*)doResizeCenterLeftSelectionCursor;

- (NSCursor*)defaultWillResizeTopLeftSelectionCursor;
- (NSCursor*)willResizeTopLeftSelectionCursor;
- (void)setWillResizeTopLeftSelectionCursor:(NSCursor*)willResizeTopLeftSelectionCursor;

- (NSCursor*)defaultDoResizeTopLeftSelectionCursor;
- (NSCursor*)doResizeTopLeftSelectionCursor;
- (void)setDoResizeTopLeftSelectionCursor:(NSCursor*)doResizeTopLeftSelectionCursor;

- (NSCursor*)defaultWillResizeTopCenterSelectionCursor;
- (NSCursor*)willResizeTopCenterSelectionCursor;
- (void)setWillResizeTopCenterSelectionCursor:(NSCursor*)willResizeTopCenterSelectionCursor;

- (NSCursor*)defaultDoResizeTopCenterSelectionCursor;
- (NSCursor*)doResizeTopCenterSelectionCursor;
- (void)setDoResizeTopCenterSelectionCursor:(NSCursor*)doResizeTopCenterSelectionCursor;

- (NSCursor*)defaultWillResizeTopRightSelectionCursor;
- (NSCursor*)willResizeTopRightSelectionCursor;
- (void)setWillResizeTopRightSelectionCursor:(NSCursor*)willResizeTopRightSelectionCursor;

- (NSCursor*)defaultDoResizeTopRightSelectionCursor;
- (NSCursor*)doResizeTopRightSelectionCursor;
- (void)setDoResizeTopRightSelectionCursor:(NSCursor*)doResizeTopRightSelectionCursor;

- (NSCursor*)defaultWillResizeCenterRightSelectionCursor;
- (NSCursor*)willResizeCenterRightSelectionCursor;
- (void)setWillResizeCenterRightSelectionCursor:(NSCursor*)willResizeCenterRightSelectionCursor;

- (NSCursor*)defaultDoResizeCenterRightSelectionCursor;
- (NSCursor*)doResizeCenterRightSelectionCursor;
- (void)setDoResizeCenterRightSelectionCursor:(NSCursor*)doResizeCenterRightSelectionCursor;

- (NSCursor*)defaultWillResizeBottomRightSelectionCursor;
- (NSCursor*)willResizeBottomRightSelectionCursor;
- (void)setWillResizeBottomRightSelectionCursor:(NSCursor*)willResizeBottomRightSelectionCursor;

- (NSCursor*)defaultDoResizeBottomRightSelectionCursor;
- (NSCursor*)doResizeBottomRightSelectionCursor;
- (void)setDoResizeBottomRightSelectionCursor:(NSCursor*)doResizeBottomRightSelectionCursor;

- (NSCursor*)defaultWillResizeBottomCenterSelectionCursor;
- (NSCursor*)willResizeBottomCenterSelectionCursor;
- (void)setWillResizeBottomCenterSelectionCursor:(NSCursor*)willResizeBottomCenterSelectionCursor;

- (NSCursor*)defaultDoResizeBottomCenterSelectionCursor;
- (NSCursor*)doResizeBottomCenterSelectionCursor;
- (void)setDoResizeBottomCenterSelectionCursor:(NSCursor*)doResizeBottomCenterSelectionCursor;

- (NSCursor*)defaultWillRotateBottomLeftSelectionCursor;
- (NSCursor*)willRotateBottomLeftSelectionCursor;
- (void)setWillRotateBottomLeftSelectionCursor:(NSCursor*)willRotateBottomLeftSelectionCursor;

- (NSCursor*)defaultDoRotateBottomLeftSelectionCursor;
- (NSCursor*)doRotateBottomLeftSelectionCursor;
- (void)setDoRotateBottomLeftSelectionCursor:(NSCursor*)doRotateBottomLeftSelectionCursor;

- (NSCursor*)defaultWillRotateCenterLeftSelectionCursor;
- (NSCursor*)willRotateCenterLeftSelectionCursor;
- (void)setWillRotateCenterLeftSelectionCursor:(NSCursor*)willRotateCenterLeftSelectionCursor;

- (NSCursor*)defaultDoRotateCenterLeftSelectionCursor;
- (NSCursor*)doRotateCenterLeftSelectionCursor;
- (void)setDoRotateCenterLeftSelectionCursor:(NSCursor*)doRotateCenterLeftSelectionCursor;

- (NSCursor*)defaultWillRotateTopLeftSelectionCursor;
- (NSCursor*)willRotateTopLeftSelectionCursor;
- (void)setWillRotateTopLeftSelectionCursor:(NSCursor*)willRotateTopLeftSelectionCursor;

- (NSCursor*)defaultDoRotateTopLeftSelectionCursor;
- (NSCursor*)doRotateTopLeftSelectionCursor;
- (void)setDoRotateTopLeftSelectionCursor:(NSCursor*)doRotateTopLeftSelectionCursor;

- (NSCursor*)defaultWillRotateTopCenterSelectionCursor;
- (NSCursor*)willRotateTopCenterSelectionCursor;
- (void)setWillRotateTopCenterSelectionCursor:(NSCursor*)willRotateTopCenterSelectionCursor;

- (NSCursor*)defaultDoRotateTopCenterSelectionCursor;
- (NSCursor*)doRotateTopCenterSelectionCursor;
- (void)setDoRotateTopCenterSelectionCursor:(NSCursor*)doRotateTopCenterSelectionCursor;

- (NSCursor*)defaultWillRotateTopRightSelectionCursor;
- (NSCursor*)willRotateTopRightSelectionCursor;
- (void)setWillRotateTopRightSelectionCursor:(NSCursor*)willRotateTopRightSelectionCursor;

- (NSCursor*)defaultDoRotateTopRightSelectionCursor;
- (NSCursor*)doRotateTopRightSelectionCursor;
- (void)setDoRotateTopRightSelectionCursor:(NSCursor*)doRotateTopRightSelectionCursor;

- (NSCursor*)defaultWillRotateCenterRightSelectionCursor;
- (NSCursor*)willRotateCenterRightSelectionCursor;
- (void)setWillRotateCenterRightSelectionCursor:(NSCursor*)willRotateCenterRightSelectionCursor;

- (NSCursor*)defaultDoRotateCenterRightSelectionCursor;
- (NSCursor*)doRotateCenterRightSelectionCursor;
- (void)setDoRotateCenterRightSelectionCursor:(NSCursor*)doRotateCenterRightSelectionCursor;

- (NSCursor*)defaultWillRotateBottomRightSelectionCursor;
- (NSCursor*)willRotateBottomRightSelectionCursor;
- (void)setWillRotateBottomRightSelectionCursor:(NSCursor*)willRotateBottomRightSelectionCursor;

- (NSCursor*)defaultDoRotateBottomRightSelectionCursor;
- (NSCursor*)doRotateBottomRightSelectionCursor;
- (void)setDoRotateBottomRightSelectionCursor:(NSCursor*)doRotateBottomRightSelectionCursor;

- (NSCursor*)defaultWillRotateBottomCenterSelectionCursor;
- (NSCursor*)willRotateBottomCenterSelectionCursor;
- (void)setWillRotateBottomCenterSelectionCursor:(NSCursor*)willRotateBottomCenterSelectionCursor;

- (NSCursor*)defaultDoRotateBottomCenterSelectionCursor;
- (NSCursor*)doRotateBottomCenterSelectionCursor;
- (void)setDoRotateBottomCenterSelectionCursor:(NSCursor*)doRotateBottomCenterSelectionCursor;

// Initial create selection mode may be nil: selection creation is disabled and current selection path is used instead.
- (NSString*)initialCreateSelectionMode;
- (void)setInitialCreateSelectionMode:(NSString*)initialCreateSelectionMode;

// Current create selection mode may be nil: selection creation is disabled and current selection path is used instead.
- (NSString*)currentCreateSelectionMode;

- (CGFloat)tolerance;
- (void)setTolerance:(CGFloat)tolerance;

- (id /* CGPathRef */)selectionShapePrototypePath;
- (void)setSelectionShapePrototypePath:(id /* CGPathRef */)shapePrototypePath;

// Proportion = height / width.
- (CGFloat)defaultAlternateProportion;
- (CGFloat)alternateProportion;
- (void)setAlternateProportion:(CGFloat)alternateProportion;

- (BOOL)defaultProportionalModePrimaryForCreation;
- (BOOL)isProportionalModePrimaryForCreation;
- (void)setProportionalModePrimaryForCreation:(BOOL)isProportionalModePrimaryForCreation;

- (BOOL)defaultProportionalModePrimaryForCorners;
- (BOOL)isProportionalModePrimaryForCorners;
- (void)setProportionalModePrimaryForCorners:(BOOL)isProportionalModePrimaryForCorners;

- (BOOL)defaultProportionalModePrimaryForSides;
- (BOOL)isProportionalModePrimaryForSides;
- (void)setProportionalModePrimaryForSides:(BOOL)isProportionalModePrimaryForSides;

- (NSValue* /* with NSPoint */)selectionStartPoint;

- (id /* CGPathRef */)selectionPath;
- (void)setSelectionPath:(id /* CGPathRef */)selectionPath;

- (NSValue* /* with CGAffineTransform */)selectionAffineTransform;
- (void)setSelectionAffineTransform:(NSValue* /* with CGAffineTransform */)selectionAffineTransform;

- (BOOL)isSelectionFlippedHorizontally;
- (void)setSelectionFlippedHorizontally:(BOOL)selectionHorizontalFlipFlag;

// *** Operations (Active State Only) ***

// Starts modifying selection.
// `event' may be nil.
- (void)modifySelectionWithEvent:(NSEvent*)event;
- (void)setSelectionWithPath:(CGPathRef)selectionPath transform:(CGAffineTransform)transform flippedHorizontally:(BOOL)isFlippedHorizontally modifyWithEvent:(NSEvent*)event;

// *** Selection Modifying Stage Operations (Active State Only) ***

// `isFlippingHorizontally' specifies whether to interpret flipping operation in `transform' as horizontal flip or as vertical flip with 180 degrees rotation.
- (void)setSelectionAffineTransform:(CGAffineTransform)transform flippedHorizontally:(BOOL)isFlippedHorizontally;
- (void)setSelectionAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;
- (void)pretransformSelectionUsingAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;
- (void)posttransformSelectionUsingAffineTransform:(CGAffineTransform)transform flippingHorizontally:(BOOL)isFlippingHorizontally;

- (void)flipSelectionHorizontally;
- (void)flipSelectionVertically;
- (void)scaleSelectionAboutCenterBy:(CGSize)scaleFactor;
- (void)rotateSelectionAboutCenterBy:(CGFloat)angle;
- (void)moveSelectionBy:(CGSize)offset;

// *** Actions (Active State Only) ***

- (IBAction)selectAllArea:(id)sender;

@end

#pragma mark *** Stages ***

/*!
    @const BTCreatingSelectionToolStage
    @abstract Tool is creating selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTCreatingSelectionToolStage
    #define BTCreatingSelectionToolStage @"BTCreatingSelectionToolStage"
#endif

/*!
    @const BTWaitingToModifySelectionToolStage
    @abstract Tool is waiting for user input to modify selection.
    @discussion Deactivation type: normal.
*/
#ifndef BTWaitingToModifySelectionToolStage
    #define BTWaitingToModifySelectionToolStage @"BTWaitingToModifySelectionToolStage"
#endif

/*!
    @const BTModifyingSelectionToolStage
    @abstract Tool is modifying selection.
    @discussion Deactivation type: abortive.
*/
#ifndef BTModifyingSelectionToolStage
    #define BTModifyingSelectionToolStage @"BTModifyingSelectionToolStage"
#endif

#pragma mark *** Create Selection Modes ***

/*!
    @const BTFreeHandCreateSelectionMode
*/
#ifndef BTFreeHandCreateSelectionMode
    #define BTFreeHandCreateSelectionMode @"BTFreeHandCreateSelectionMode"
#endif

/*!
    @const BTMagicWandCreateSelectionMode
*/
#ifndef BTMagicWandCreateSelectionMode
    #define BTMagicWandCreateSelectionMode @"BTMagicWandCreateSelectionMode"
#endif

/*!
    @const BTMagicLassoCreateSelectionMode
*/
#ifndef BTMagicLassoCreateSelectionMode
    #define BTMagicLassoCreateSelectionMode @"BTMagicLassoCreateSelectionMode"
#endif

/*!
    @const BTShapeCreateSelectionMode
*/
#ifndef BTShapeCreateSelectionMode
    #define BTShapeCreateSelectionMode @"BTShapeCreateSelectionMode"
#endif

#pragma mark *** Undo Categories ***

/*!
    @const BTSelectionUndoCategory
    @discussion Undoable outputs: selectionPath, selectionAffineTransform, selectionFlippedHorizontally.
*/
#if !defined(BTSelectionUndoCategory)
    #define BTSelectionUndoCategory @"BTSelectionUndoCategory"
#endif // !defined(BTSelectionUndoCategory)
