/*!
    @header BTToolView.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/12/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <SMKBitmapTools/BTTrackingView.h>
#import <SMKBitmapTools/BTTool.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>

@protocol BTImage;

/*!
    @class BTToolView
    @discussion Requirements for view's window:
                    1) the window should accept mouse move events ([window setAcceptsMouseMovedEvents:YES]);
                    2) all cursor rectangle management within the window should be disabled ([window disableCursorRects]).
*/
@interface BTToolView : BTTrackingView<BTToolDelegate, BTImageViewTransformer>
{
@private
    id<BTImage> _image;
    NSDictionary* _imageProperties;
    
    NSDictionary* _availableTools;
    BTTool* _currentTool;
    
    NSCursor* _currentToolCursor;
    BTTrackingZone* _trackingVisibleZone;
    BOOL _isInVisibleZoneAndActive;
    
    NSSize _zoomFactor;
    
    NSEvent* _currentKeyEvent;
}

- (id<BTImage>)image;
- (NSDictionary*)imageProperties;
- (void)setImage:(id<BTImage>)image imageProperties:(NSDictionary*)metaData;

- (NSArray*)availableToolNames;
- (BTTool*)toolForName:(NSString*)toolName;

- (BTTool*)currentTool;
- (BOOL)setCurrentTool:(BTTool*)tool forceSwitch:(BOOL)forceSwicthFlag;
- (BOOL)setCurrentToolByName:(NSString*)toolName forceSwitch:(BOOL)forceSwicthFlag;

// Point for conversion is center of the window pixel pointed by the mouse.
- (NSPoint)convertMouseLocationFromWindow:(NSPoint)mouseLocationInWindow;

- (void)interpretCurrentKeyEventAsCommand:(SEL*)commandSelectorPtr text:(id*)insertStringPtr;

- (void)drawContentRect:(NSRect)rect;
- (void)drawToolRect:(NSRect)rect;

@end
