/*!
    @header NSWindowBTEvents.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2008/02/20
    @copyright Copyright (c) 2008 The Software MacKiev Company. All rights reserved.
*/

#import <AppKit/NSApplication.h>

@class NSEvent;

/*!
    @category NSApplication(BTEvents)
*/
@interface NSApplication(BTEvents)

/*!
    @method BT_nearestModifierFlagsEvent
    @abstract Returns nearest event with valid modifier flags.
    @discussion Returns nearest event with valid modifier flags in the event queue starting with the current event. Returns nil if there is no such event.
*/
- (NSEvent*)BT_nearestModifierFlagsEvent;

/*!
    @method BT_effectiveModifierFlagsWithEvent:
    @abstract Returns modifier flags of `baseEvent', modifier flags of the nearest modifier flags event or the current hardware state of modifier flags as appropriate.
    @param baseEvent The event to query modifier flags for. May be nil: modifier flags of the nearest modifier flags event or the current hardware state of modifier flags as appropriate.
    @result Effective modifier flags for `baseEvent'.
*/
- (NSUInteger)BT_effectiveModifierFlagsWithEvent:(NSEvent*)baseEvent;

/*!
    @method BT_effectiveModifierFlags
    @abstract Returns modifier flags of the nearest modifier flags event or the current hardware state of modifier flags as appropriate.
    @result Effective modifier flags.
*/
- (NSUInteger)BT_effectiveModifierFlags;

@end
