/*!
    @header NSViewBTMouseTracking.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/14/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <AppKit/NSView.h>

@protocol BTMouseTracker;
@class NSEvent;

/*!
    @category NSView(BTMouseTracking)
*/
@interface NSView(BTMouseTracking)

// Returns the NSView object that currently tracks mouse with `-BT_trackMouseWithEvent:tracker:...', or nil if there is none.
+ (NSView*)BT_mouseTrackingView;

// Returns mouse up event which finished or halted mouse tracking; returns nil if tracking was aborted.
- (NSEvent*)BT_trackMouseWithEvent:(NSEvent*)mouseDownEvent tracker:(id<BTMouseTracker>)mouseTracker;
- (NSEvent*)BT_trackMouseWithEvent:(NSEvent*)mouseDownEvent tracker:(id<BTMouseTracker>)mouseTracker startDelay:(NSTimeInterval)delaySeconds;

// Aborts mouse tracking started with `-BT_trackMouseWithEvent:tracker:...'.
- (void)BT_abortMouseTracking;

@end

/*!
    @protocol BTMouseTracker
*/
@protocol BTMouseTracker<NSObject>

- (void)trackMouseDown:(NSEvent*)event inView:(NSView*)trackingView; // This is sent only if dragging is starting.
- (void)trackMouseDragged:(NSEvent*)event inView:(NSView*)trackingView;
- (void)trackMouseUp:(NSEvent*)event inView:(NSView*)trackingView;
- (void)trackFlagsChanged:(NSEvent*)event inView:(NSView*)trackingView;

- (void)haltWithMouseUp:(NSEvent*)event inView:(NSView*)trackingView;

- (BOOL)trackingView:(NSView*)trackingView shouldAutoscrollWithMouseEvent:(NSEvent*)event;

// `didScroll' indicates whether any scrolling is performed (this is result of [trackingView autoscroll:event]).
- (void)trackingView:(NSView*)trackingView didAutoscroll:(BOOL)didScroll withMouseEvent:(NSEvent*)event;

@end
