/*!
    @header NSWindowBTEvents.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2007/04/22
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <AppKit/NSWindow.h>
#import <Foundation/NSGeometry.h>

@class NSEvent;

/*!
    @category NSWindow(BTEvents)
*/
@interface NSWindow(BTEvents)

/*!
    @method BT_nearestLocationEvent
    @abstract Returns nearest event with a valid location.
    @discussion Returns nearest event with a valid location in the event queue starting with the current event. Returns nil if there is no such event.
*/
- (NSEvent*)BT_nearestLocationEvent;

/*!
    @method BT_effectiveMouseLocationWithEvent:
    @abstract Returns the location of `baseEvent', the location of the nearest mouse event or the current location of the mouse as appropriate.
    @param baseEvent The event to query location for. May be nil: the location of the nearest mouse event or the current location of the mouse is returned as appropriate.
    @result Effective mouse location for `baseEvent' in the receiver’s base coordinate system.
*/
- (NSPoint)BT_effectiveMouseLocationWithEvent:(NSEvent*)baseEvent;

/*!
    @method BT_effectiveMouseLocation
    @abstract Returns the location of the nearest mouse event or the current location of the mouse as appropriate.
    @result Effective mouse location in the receiver’s base coordinate system.
*/
- (NSPoint)BT_effectiveMouseLocation;

/*!
    @method BT_nearestModifierFlagsEvent
    @abstract Returns nearest event with valid modifier flags.
    @discussion Returns nearest event with valid modifier flags in the event queue starting with the current event. Returns nil if there is no such event.
*/
- (NSEvent*)BT_nearestModifierFlagsEvent;

/*!
    @method BT_effectiveModifierFlagsWithEvent:
    @abstract Returns modifier flags of `baseEvent', modifier flags of the nearest modifier flags event or the current hardware state of modifier flags as appropriate.
    @param baseEvent The event to query modifier flags for. May be nil: modifier flags of the nearest modifier flags event or the current hardware state of modifier flags as appropriate.
    @result Effective modifier flags for `baseEvent'.
*/
- (NSUInteger)BT_effectiveModifierFlagsWithEvent:(NSEvent*)baseEvent;

/*!
    @method BT_effectiveModifierFlags
    @abstract Returns modifier flags of the nearest modifier flags event or the current hardware state of modifier flags as appropriate.
    @result Effective modifier flags.
*/
- (NSUInteger)BT_effectiveModifierFlags;

/*!
    @method BT_mouseMovedEvent
    @abstract Returns mouse moved event based on either nearest mouse event or current hardware state as appropriate.
*/
- (NSEvent*)BT_mouseMovedEvent;

@end
