/*!
    @header CGEAffineTransform.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 01/11/2007
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
    @abstract Extensions for 2D affine transform:   |a    b   0|
                                                    |c    d   0|
                                                    |tx   ty  1|
*/

#ifndef CGEAffineTransform_H_
#define CGEAffineTransform_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGAffineTransform.h>
#include <SMKCGExtensions/CGEFloat.h>

CG_EXTERN_C_BEGIN

CG_INLINE CGAffineTransform CGEAffineTransformCompound(bool isFlippedHorizontally, bool isFlippedVertically, CGFloat horizontalScaleFactor, CGFloat verticalScaleFactor, CGFloat skewTangent, CGFloat rotationAngle, CGFloat horizontalOffset, CGFloat verticalOffset);
CG_INLINE CGAffineTransform CGEAffineTransformCompose(bool isFlippedHorizontally, bool isFlippedVertically, CGFloat widthScaleFactor, CGFloat heightScaleFactor, CGFloat skewAngle, CGFloat rotationAngle, CGFloat horizontalOffset, CGFloat verticalOffset);

CG_INLINE CGAffineTransform CGEAffineTransformMakeTranslation(CGFloat tx, CGFloat ty); // Same as `CGAffineTransformMakeTranslation' but inline.
CG_INLINE CGAffineTransform CGEAffineTransformMakeScale(CGFloat sx, CGFloat sy); // Same as `CGAffineTransformMakeScale' but inline.
CG_INLINE CGAffineTransform CGEAffineTransformMakeScaleAboutPoint(CGFloat sx, CGFloat sy, CGPoint scaleCenter);
CG_INLINE CGAffineTransform CGEAffineTransformMakeSkew(CGFloat skewTangent);
CG_INLINE CGAffineTransform CGEAffineTransformMakeRotation(CGFloat angle); // Same as `CGAffineTransformMakeRotation' but inline.
CG_INLINE CGAffineTransform CGEAffineTransformMakeRotationAboutPoint(CGFloat angle, CGPoint rotationCenter);
CG_INLINE CGAffineTransform CGEAffineTransformMakeHorizontalFlipWithWidth(CGFloat width);
CG_INLINE CGAffineTransform CGEAffineTransformMakeVerticalFlipWithHeight(CGFloat height);
CG_INLINE CGAffineTransform CGEAffineTransformMakeHorizontalFlipInRect(CGRect rect);
CG_INLINE CGAffineTransform CGEAffineTransformMakeVerticalFlipInRect(CGRect rect);
CG_INLINE CGAffineTransform CGEAffineTransformMakeRectTransition(CGRect sourceRect, CGRect destinationRect);
CG_INLINE CGAffineTransform CGEAffineTransformMakeOrientedRectTransition(CGRect sourceRect, CGPoint destinationOriginCorner, CGPoint destinationFarCorner);


CG_INLINE bool CGEAffineTransformIsFlippedVertically(CGAffineTransform transform, bool isFlippedHorizontally);
CG_INLINE CGFloat CGEAffineTransformGetHorizontalScaleFactor(CGAffineTransform transform);
CG_INLINE CGFloat CGEAffineTransformGetVerticalScaleFactor(CGAffineTransform transform);
CG_INLINE CGFloat CGEAffineTransformGetWidthScaleFactor(CGAffineTransform transform);
CG_INLINE CGFloat CGEAffineTransformGetHeightScaleFactor(CGAffineTransform transform);
CG_INLINE CGFloat CGEAffineTransformGetSkewAngle(CGAffineTransform transform);
CG_INLINE CGFloat CGEAffineTransformGetRotationAngle(CGAffineTransform transform, bool isFlippedHorizontally);

// `isFlippedVerticallyPtr' may be NULL.
// `horizontalScaleFactorPtr' may be NULL.
// `verticalScaleFactorPtr' may be NULL.
// `widthScaleFactorPtr' may be NULL.
// `heightScaleFactorPtr' may be NULL.
// `skewAnglePtr' may be NULL.
// `rotationAnglePtr' may be NULL.
// `horizontalOffsetPtr' may be NULL.
// `verticalOffsetPtr' may be NULL.
CG_INLINE void CGEAffineTransformDecompose(CGAffineTransform transform, bool isFlippedHorizontally, bool* isFlippedVerticallyPtr, CGFloat* horizontalScaleFactorPtr, CGFloat* verticalScaleFactorPtr, CGFloat* widthScaleFactorPtr, CGFloat* heightScaleFactorPtr, CGFloat* skewAnglePtr, CGFloat* rotationAnglePtr, CGFloat* horizontalOffsetPtr, CGFloat* verticalOffsetPtr);

CG_INLINE CGAffineTransform CGEAffineTransformMultiply(CGAffineTransform transform1, CGAffineTransform transform2); // Same as `CGAffineTransformConcat' but inline.

CG_INLINE CGAffineTransform CGEAffineTransformPretranslate(CGAffineTransform transform, CGFloat dx, CGFloat dy); // Same as `CGAffineTransformTranslate' but inline.
CG_INLINE CGAffineTransform CGEAffineTransformPosttranslate(CGAffineTransform transform, CGFloat dx, CGFloat dy);

CG_INLINE CGAffineTransform CGEAffineTransformPrescale(CGAffineTransform transform, CGFloat sx, CGFloat sy); // Same as `CGAffineTransformScale' but inline.
CG_INLINE CGAffineTransform CGEAffineTransformPostscale(CGAffineTransform transform, CGFloat sx, CGFloat sy);
CG_INLINE CGAffineTransform CGEAffineTransformPrescaleAboutPoint(CGAffineTransform transform, CGFloat sx, CGFloat sy, CGPoint scaleCenter);
CG_INLINE CGAffineTransform CGEAffineTransformPostscaleAboutPoint(CGAffineTransform transform, CGFloat sx, CGFloat sy, CGPoint scaleCenter);

CG_INLINE CGAffineTransform CGEAffineTransformSetWidthScaleAboutPoint(CGAffineTransform transform, CGFloat widthScale, CGPoint untransformedScaleCenter);
CG_INLINE CGAffineTransform CGEAffineTransformSetHeightScaleAboutPoint(CGAffineTransform transform, CGFloat heightScale, CGPoint untransformedScaleCenter);
CG_INLINE CGAffineTransform CGEAffineTransformSetSizeScaleAboutPoint(CGAffineTransform transform, CGFloat widthScale, CGFloat heightScale, CGPoint untransformedScaleCenter);

CG_INLINE CGAffineTransform CGEAffineTransformPreskew(CGAffineTransform transform, CGFloat skewTangent);
CG_INLINE CGAffineTransform CGEAffineTransformPostskew(CGAffineTransform transform, CGFloat skewTangent);

CG_INLINE CGAffineTransform CGEAffineTransformPrerotate(CGAffineTransform transform, CGFloat angle); // Same as `CGAffineTransformRotate' but inline.
CG_INLINE CGAffineTransform CGEAffineTransformPostrotate(CGAffineTransform transform, CGFloat angle);
CG_INLINE CGAffineTransform CGEAffineTransformPrerotateAboutPoint(CGAffineTransform transform, CGFloat angle, CGPoint rotationCenter);
CG_INLINE CGAffineTransform CGEAffineTransformPostrotateAboutPoint(CGAffineTransform transform, CGFloat angle, CGPoint rotationCenter);

CG_INLINE CGAffineTransform CGEAffineTransformPreflipHorizontallyWithWidth(CGAffineTransform transform, CGFloat width);
CG_INLINE CGAffineTransform CGEAffineTransformPostflipHorizontallyWithWidth(CGAffineTransform transform, CGFloat width);
CG_INLINE CGAffineTransform CGEAffineTransformPreflipVerticallyWithHeight(CGAffineTransform transform, CGFloat height);
CG_INLINE CGAffineTransform CGEAffineTransformPostflipVerticallyWithHeight(CGAffineTransform transform, CGFloat height);
CG_INLINE CGAffineTransform CGEAffineTransformPreflipHorizontallyInRect(CGAffineTransform transform, CGRect rect);
CG_INLINE CGAffineTransform CGEAffineTransformPostflipHorizontallyInRect(CGAffineTransform transform, CGRect rect);
CG_INLINE CGAffineTransform CGEAffineTransformPreflipVerticallyInRect(CGAffineTransform transform, CGRect rect);
CG_INLINE CGAffineTransform CGEAffineTransformPostflipVerticallyInRect(CGAffineTransform transform, CGRect rect);

CG_INLINE CGAffineTransform CGEAffineTransformInvert(CGAffineTransform transform); // Same as `CGAffineTransformInvert' but inline.

// `removedWidthScaleFactorPtr' may be NULL.
// `removedHeightScaleFactorPtr' may be NULL.
CG_INLINE CGAffineTransform CGEAffineTransformRemoveSizeScale(CGAffineTransform transform, CGFloat* removedWidthScaleFactorPtr, CGFloat* removedHeightScaleFactorPtr);

#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4
    #define CGEAffineTransformIsIdentity CGAffineTransformIsIdentity
#else
    #define CGEAffineTransformIsIdentity __CGEAffineTransformIsIdentity
    
    CG_EXTERN bool CGEAffineTransformIsIdentity(CGAffineTransform transform);
#endif // MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformCompound(bool isFlippedHorizontally, bool isFlippedVertically, CGFloat horizontalScaleFactor, CGFloat verticalScaleFactor, CGFloat skewTangent, CGFloat rotationAngle, CGFloat horizontalOffset, CGFloat verticalOffset)
{
    CGFloat sx = (isFlippedHorizontally) ? -horizontalScaleFactor : horizontalScaleFactor;
    CGFloat sy = (isFlippedVertically) ? -verticalScaleFactor : verticalScaleFactor;
    CGFloat rotationCosine = CGEFloat_cos(rotationAngle);
    CGFloat rotationSine = CGEFloat_sin(rotationAngle);
    
    return CGAffineTransformMake(
        sx * rotationCosine,
        sx * rotationSine,
        sy * (rotationCosine * skewTangent - rotationSine),
        sy * (rotationSine * skewTangent + rotationCosine),
        horizontalOffset,
        verticalOffset
    );
}

CG_INLINE CGAffineTransform CGEAffineTransformCompose(bool isFlippedHorizontally, bool isFlippedVertically, CGFloat widthScaleFactor, CGFloat heightScaleFactor, CGFloat skewAngle, CGFloat rotationAngle, CGFloat horizontalOffset, CGFloat verticalOffset)
{
    CGFloat skewTangent = CGEFloat_tan(skewAngle);
    CGFloat horizontalScaleFactor = widthScaleFactor;
    CGFloat verticalScaleFactor = heightScaleFactor / CGEFloat_hypot(skewTangent, 1.0);
    
    return CGEAffineTransformCompound(isFlippedHorizontally, isFlippedVertically, horizontalScaleFactor, verticalScaleFactor, skewTangent, rotationAngle, horizontalOffset, verticalOffset);
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformMakeTranslation(CGFloat tx, CGFloat ty)
{
    return CGAffineTransformMake(1.0, 0.0, 0.0, 1.0, tx, ty);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeScale(CGFloat sx, CGFloat sy)
{
    return CGAffineTransformMake(sx, 0.0, 0.0, sy, 0.0, 0.0);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeScaleAboutPoint(CGFloat sx, CGFloat sy, CGPoint scaleCenter)
{
    return CGAffineTransformMake(
        sx,
        0.0,
        0.0,
        sy,
        (1.0 - sx) * scaleCenter.x,
        (1.0 - sy) * scaleCenter.y
    );
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeSkew(CGFloat skewTangent)
{
    return CGAffineTransformMake(1.0, 0.0, skewTangent, 1.0, 0.0, 0.0);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeRotation(CGFloat angle)
{
    CGFloat cosine = CGEFloat_cos(angle);
    CGFloat sine = CGEFloat_sin(angle);
    
    return CGAffineTransformMake(cosine, sine, -sine, cosine, 0.0, 0.0);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeRotationAboutPoint(CGFloat angle, CGPoint rotationCenter)
{
    CGFloat cosine = CGEFloat_cos(angle);
    CGFloat sine = CGEFloat_sin(angle);
    CGFloat oneMinusCosine = 1.0 - cosine;
    
    return CGAffineTransformMake(
        cosine,
        sine,
        -sine,
        cosine,
        oneMinusCosine * rotationCenter.x + sine * rotationCenter.y,
        oneMinusCosine * rotationCenter.y - sine * rotationCenter.x
    );
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeHorizontalFlipWithWidth(CGFloat width)
{
    return CGAffineTransformMake(-1.0, 0.0, 0.0, 1.0, width, 0.0);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeVerticalFlipWithHeight(CGFloat height)
{
    return CGAffineTransformMake(1.0, 0.0, 0.0, -1.0, 0.0, height);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeHorizontalFlipInRect(CGRect rect)
{
    return CGEAffineTransformMakeHorizontalFlipWithWidth(rect.size.width + 2.0 * rect.origin.x);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeVerticalFlipInRect(CGRect rect)
{
    return CGEAffineTransformMakeVerticalFlipWithHeight(rect.size.height + 2.0 * rect.origin.y);
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeRectTransition(CGRect sourceRect, CGRect destinationRect)
{
    CGFloat sx = (sourceRect.size.width == 0.0) ? 1.0 : destinationRect.size.width / sourceRect.size.width;
    CGFloat sy = (sourceRect.size.height == 0.0) ? 1.0 : destinationRect.size.height / sourceRect.size.height;
    
    return CGAffineTransformMake(
        sx,
        0.0,
        0.0,
        sy,
        destinationRect.origin.x - sx * sourceRect.origin.x,
        destinationRect.origin.y - sy * sourceRect.origin.y
    );
}

CG_INLINE CGAffineTransform CGEAffineTransformMakeOrientedRectTransition(CGRect sourceRect, CGPoint destinationOriginCorner, CGPoint destinationFarCorner)
{
    CGFloat destinationDeltaX = destinationFarCorner.x - destinationOriginCorner.x;
    CGFloat destinationDeltaY = destinationFarCorner.y - destinationOriginCorner.y;
    CGFloat sx = (sourceRect.size.width == 0.0) ? CGEFloat_copysign(1.0, destinationDeltaX) : destinationDeltaX / sourceRect.size.width;
    CGFloat sy = (sourceRect.size.height == 0.0) ? CGEFloat_copysign(1.0, destinationDeltaY) : destinationDeltaY / sourceRect.size.height;
    
    return CGAffineTransformMake(
        sx,
        0.0,
        0.0,
        sy,
        destinationOriginCorner.x - sx * sourceRect.origin.x,
        destinationOriginCorner.y - sy * sourceRect.origin.y
    );
}

#pragma mark -

CG_INLINE bool CGEAffineTransformIsFlippedVertically(CGAffineTransform transform, bool isFlippedHorizontally)
{
    if ((transform.a * transform.d) >= (transform.b * transform.c))
        return isFlippedHorizontally;
    
    return !isFlippedHorizontally;
}

CG_INLINE CGFloat CGEAffineTransformGetHorizontalScaleFactor(CGAffineTransform transform)
{
    return CGEAffineTransformGetWidthScaleFactor(transform);
}

CG_INLINE CGFloat CGEAffineTransformGetVerticalScaleFactor(CGAffineTransform transform)
{
    CGFloat result = 0.0;
    
    if ((transform.a == 0.0) && (transform.b == 0.0))
        result = CGEAffineTransformGetHeightScaleFactor(transform);
    else result = CGEFloat_abs(transform.a * transform.d - transform.b * transform.c) / CGEAffineTransformGetWidthScaleFactor(transform);
    
    return result;
}

CG_INLINE CGFloat CGEAffineTransformGetWidthScaleFactor(CGAffineTransform transform)
{
    return CGEFloat_hypot(transform.a, transform.b);
}

CG_INLINE CGFloat CGEAffineTransformGetHeightScaleFactor(CGAffineTransform transform)
{
    return CGEFloat_hypot(transform.c, transform.d);
}

CG_INLINE CGFloat CGEAffineTransformGetSkewAngle(CGAffineTransform transform)
{
    return CGEFloat_atan2p(transform.a * transform.c + transform.b * transform.d, transform.a * transform.d - transform.b * transform.c);
}

CG_INLINE CGFloat CGEAffineTransformGetRotationAngle(CGAffineTransform transform, bool isFlippedHorizontally)
{
    CGFloat result = 0.0;
    
    if ((transform.a == 0.0) && (transform.b == 0.0))
    {
        if (isFlippedHorizontally) // Means flipped vertically under condition ((transform.a == 0.0) && (transform.b == 0.0)).
        { // Counter y-flip.
            transform.c = -transform.c;
            transform.d = -transform.d;
        }
        
        result = CGEFloat_atan2(transform.d, transform.c) - M_PI_2;
    }
    else
    {
        if (isFlippedHorizontally)
        { // Counter x-flip.
            transform.a = -transform.a;
            transform.b = -transform.b;
        }
        
        result = CGEFloat_atan2(transform.b, transform.a);
    }
    
    if (result < 0.0)
        result += 2.0 * M_PI;
    
    return result;
}

CG_INLINE void CGEAffineTransformDecompose(CGAffineTransform transform, bool isFlippedHorizontally, bool* isFlippedVerticallyPtr, CGFloat* horizontalScaleFactorPtr, CGFloat* verticalScaleFactorPtr, CGFloat* widthScaleFactorPtr, CGFloat* heightScaleFactorPtr, CGFloat* skewAnglePtr, CGFloat* rotationAnglePtr, CGFloat* horizontalOffsetPtr, CGFloat* verticalOffsetPtr)
{
    bool isEmptyHorizontally = ((transform.a == 0.0) && (transform.b == 0.0));
    CGFloat heightScaleFactor = (((verticalScaleFactorPtr != NULL) && isEmptyHorizontally) || (heightScaleFactorPtr != NULL)) ? CGEAffineTransformGetHeightScaleFactor(transform) : 0.0;
    
    if (isEmptyHorizontally)
    {
        if (isFlippedVerticallyPtr != NULL)
            *isFlippedVerticallyPtr = isFlippedHorizontally;
        
        if (horizontalScaleFactorPtr != NULL)
            *horizontalScaleFactorPtr = 0.0;
        
        if (verticalScaleFactorPtr != NULL)
            *verticalScaleFactorPtr = heightScaleFactor;
        
        if (skewAnglePtr != NULL)
            *skewAnglePtr = 0.0;
        
        if (widthScaleFactorPtr != NULL)
            *widthScaleFactorPtr = 0.0;
    }
    else
    {
        CGFloat widthScaleFactor = ((horizontalScaleFactorPtr != NULL) || (verticalScaleFactorPtr != NULL) || (widthScaleFactorPtr != NULL)) ? CGEAffineTransformGetWidthScaleFactor(transform) : 0.0;
        CGFloat determinant = ((isFlippedVerticallyPtr != NULL) || (verticalScaleFactorPtr != NULL) || (skewAnglePtr != NULL)) ? transform.a * transform.d - transform.b * transform.c : 0.0;
        
        if (isFlippedVerticallyPtr != NULL)
            *isFlippedVerticallyPtr = (determinant >= 0) ? isFlippedHorizontally : !isFlippedHorizontally;
        
        if (horizontalScaleFactorPtr != NULL)
            *horizontalScaleFactorPtr = widthScaleFactor;
        
        if (verticalScaleFactorPtr != NULL)
            *verticalScaleFactorPtr = CGEFloat_abs(determinant) / widthScaleFactor;
        
        if (skewAnglePtr != NULL)
            *skewAnglePtr = CGEFloat_atan2p(transform.a * transform.c + transform.b * transform.d, determinant);
        
        if (widthScaleFactorPtr != NULL)
            *widthScaleFactorPtr = widthScaleFactor;
    }
    
    if (heightScaleFactorPtr != NULL)
        *heightScaleFactorPtr = heightScaleFactor;
    
    if (rotationAnglePtr != NULL)
        *rotationAnglePtr = CGEAffineTransformGetRotationAngle(transform, isFlippedHorizontally);
    
    if (horizontalOffsetPtr != NULL)
        *horizontalOffsetPtr = transform.tx;
    
    if (verticalOffsetPtr != NULL)
        *verticalOffsetPtr = transform.ty;
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformMultiply(CGAffineTransform transform1, CGAffineTransform transform2)
{
    return CGAffineTransformMake(
        transform1.a * transform2.a + transform1.b * transform2.c,
        transform1.a * transform2.b + transform1.b * transform2.d,
        transform1.c * transform2.a + transform1.d * transform2.c,
        transform1.c * transform2.b + transform1.d * transform2.d,
        transform1.tx * transform2.a + transform1.ty * transform2.c + transform2.tx,
        transform1.tx * transform2.b + transform1.ty * transform2.d + transform2.ty
    );
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformPretranslate(CGAffineTransform transform, CGFloat dx, CGFloat dy)
{
    transform.tx += dx * transform.a + dy * transform.c;
    transform.ty += dx * transform.b + dy * transform.d;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPosttranslate(CGAffineTransform transform, CGFloat dx, CGFloat dy)
{
    transform.tx += dx;
    transform.ty += dy;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPrescale(CGAffineTransform transform, CGFloat sx, CGFloat sy)
{
    transform.a *= sx;
    transform.b *= sx;
    transform.c *= sy;
    transform.d *= sy;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPostscale(CGAffineTransform transform, CGFloat sx, CGFloat sy)
{
    transform.a *= sx;
    transform.b *= sy;
    transform.c *= sx;
    transform.d *= sy;
    transform.tx *= sx;
    transform.ty *= sy;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPrescaleAboutPoint(CGAffineTransform transform, CGFloat sx, CGFloat sy, CGPoint scaleCenter)
{
    CGFloat dx = (1.0 - sx) * scaleCenter.x;
    CGFloat dy = (1.0 - sy) * scaleCenter.y;
    
    transform.tx += dx * transform.a + dy * transform.c;
    transform.ty += dx * transform.b + dy * transform.d;
    transform.a *= sx;
    transform.b *= sx;
    transform.c *= sy;
    transform.d *= sy;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPostscaleAboutPoint(CGAffineTransform transform, CGFloat sx, CGFloat sy, CGPoint scaleCenter)
{
    transform.a *= sx;
    transform.b *= sy;
    transform.c *= sx;
    transform.d *= sy;
    transform.tx = transform.tx * sx + (1.0 - sx) * scaleCenter.x;
    transform.ty = transform.ty * sy + (1.0 - sy) * scaleCenter.y;
    return transform;
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformSetWidthScaleAboutPoint(CGAffineTransform transform, CGFloat widthScale, CGPoint untransformedScaleCenter)
{
    CGFloat currentWidthScale = CGEAffineTransformGetWidthScaleFactor(transform);
    
    if (currentWidthScale == 0.0)
    {
        if (widthScale != 0.0)
        { // Set width scale factor from 0.0 to 1.0 by re-composing.
            bool isFlippedVertically = false;
            CGFloat skewAngle = 0.0;
            CGFloat heightScale = 0.0;
            CGFloat rotationAngle = 0.0;
            CGFloat horizontalOffset = 0.0;
            CGFloat verticalOffset = 0.0;
            CGEAffineTransformDecompose(transform, false, &isFlippedVertically, NULL, NULL, NULL, &heightScale, &skewAngle, &rotationAngle, &horizontalOffset, &verticalOffset);
            
            transform = CGEAffineTransformCompose(false, isFlippedVertically, 1.0, heightScale, skewAngle, rotationAngle, horizontalOffset, verticalOffset);
        }
        currentWidthScale = 1.0;
    }
    
    transform = CGEAffineTransformPrescaleAboutPoint(transform, widthScale / currentWidthScale, 1.0, untransformedScaleCenter);
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformSetHeightScaleAboutPoint(CGAffineTransform transform, CGFloat heightScale, CGPoint untransformedScaleCenter)
{
    CGFloat currentHeightScale = CGEAffineTransformGetHeightScaleFactor(transform);
    
    if (currentHeightScale == 0.0)
    {
        if (heightScale != 0.0)
        { // Set height scale factor from 0.0 to 1.0 by re-composing.
            bool isFlippedVertically = false;
            CGFloat skewAngle = 0.0;
            CGFloat widthScale = 0.0;
            CGFloat rotationAngle = 0.0;
            CGFloat horizontalOffset = 0.0;
            CGFloat verticalOffset = 0.0;
            CGEAffineTransformDecompose(transform, false, &isFlippedVertically, NULL, NULL, &widthScale, NULL, &skewAngle, &rotationAngle, &horizontalOffset, &verticalOffset);
            
            transform = CGEAffineTransformCompose(false, isFlippedVertically, widthScale, 1.0, skewAngle, rotationAngle, horizontalOffset, verticalOffset);
        }
        currentHeightScale = 1.0;
    }
    
    transform = CGEAffineTransformPrescaleAboutPoint(transform, 1.0, heightScale / currentHeightScale, untransformedScaleCenter);
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformSetSizeScaleAboutPoint(CGAffineTransform transform, CGFloat widthScale, CGFloat heightScale, CGPoint untransformedScaleCenter)
{
    CGSize currentSizeScale = {CGEAffineTransformGetWidthScaleFactor(transform), CGEAffineTransformGetHeightScaleFactor(transform)};
    bool shouldRecompose = false;
    
    if (currentSizeScale.width == 0.0)
    {
        currentSizeScale.width = 1.0;
        shouldRecompose = (widthScale != 0.0);
    }
    
    if (currentSizeScale.height == 0.0)
    {
        currentSizeScale.height = 1.0;
        if (!shouldRecompose)
            shouldRecompose = (heightScale != 0.0);
    }
    
    if (shouldRecompose)
    { // Set zero size scale factor(s) to 1.0 by re-composing.
        bool isFlippedVertically = false;
        CGFloat skewAngle = 0.0;
        CGFloat rotationAngle = 0.0;
        CGFloat horizontalOffset = 0.0;
        CGFloat verticalOffset = 0.0;
        CGEAffineTransformDecompose(transform, false, &isFlippedVertically, NULL, NULL, NULL, NULL, &skewAngle, &rotationAngle, &horizontalOffset, &verticalOffset);
        
        transform = CGEAffineTransformCompose(false, isFlippedVertically, currentSizeScale.width, currentSizeScale.height, skewAngle, rotationAngle, horizontalOffset, verticalOffset);
    }
    
    transform = CGEAffineTransformPrescaleAboutPoint(transform, widthScale / currentSizeScale.width, heightScale / currentSizeScale.height, untransformedScaleCenter);
    return transform;
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformPreskew(CGAffineTransform transform, CGFloat skewTangent)
{
    transform.c += skewTangent * transform.a;
    transform.d += skewTangent * transform.b;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPostskew(CGAffineTransform transform, CGFloat skewTangent)
{
    transform.a += skewTangent * transform.b;
    transform.c += skewTangent * transform.d;
    transform.tx += skewTangent * transform.ty;
    return transform;
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformPrerotate(CGAffineTransform transform, CGFloat angle)
{
    CGFloat cosine = CGEFloat_cos(angle);
    CGFloat sine = CGEFloat_sin(angle);
    
    return CGAffineTransformMake(
        cosine * transform.a + sine * transform.c,
        cosine * transform.b + sine * transform.d,
        -sine * transform.a + cosine * transform.c,
        -sine * transform.b + cosine * transform.d,
        transform.tx,
        transform.ty
    );
}

CG_INLINE CGAffineTransform CGEAffineTransformPostrotate(CGAffineTransform transform, CGFloat angle)
{
    CGFloat cosine = CGEFloat_cos(angle);
    CGFloat sine = CGEFloat_sin(angle);
    
    return CGAffineTransformMake(
        transform.a * cosine - transform.b * sine,
        transform.a * sine + transform.b * cosine,
        transform.c * cosine - transform.d * sine,
        transform.c * sine + transform.d * cosine,
        transform.tx * cosine - transform.ty * sine,
        transform.tx * sine + transform.ty * cosine
    );
}

CG_INLINE CGAffineTransform CGEAffineTransformPrerotateAboutPoint(CGAffineTransform transform, CGFloat angle, CGPoint rotationCenter)
{
    return CGEAffineTransformMultiply(CGEAffineTransformMakeRotationAboutPoint(angle, rotationCenter), transform);
}

CG_INLINE CGAffineTransform CGEAffineTransformPostrotateAboutPoint(CGAffineTransform transform, CGFloat angle, CGPoint rotationCenter)
{
    return CGEAffineTransformMultiply(transform, CGEAffineTransformMakeRotationAboutPoint(angle, rotationCenter));
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformPreflipHorizontallyWithWidth(CGAffineTransform transform, CGFloat width)
{
    transform.tx += width * transform.a;
    transform.ty += width * transform.b;
    transform.a = -transform.a;
    transform.b = -transform.b;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPostflipHorizontallyWithWidth(CGAffineTransform transform, CGFloat width)
{
    transform.a = -transform.a;
    transform.c = -transform.c;
    transform.tx = width - transform.tx;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPreflipVerticallyWithHeight(CGAffineTransform transform, CGFloat height)
{
    transform.tx += height * transform.c;
    transform.ty += height * transform.d;
    transform.c = -transform.c;
    transform.d = -transform.d;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPostflipVerticallyWithHeight(CGAffineTransform transform, CGFloat height)
{
    transform.b = -transform.b;
    transform.d = -transform.d;
    transform.ty = height - transform.ty;
    return transform;
}

CG_INLINE CGAffineTransform CGEAffineTransformPreflipHorizontallyInRect(CGAffineTransform transform, CGRect rect)
{
    return CGEAffineTransformPreflipHorizontallyWithWidth(transform, rect.size.width + 2.0 * rect.origin.x);
}

CG_INLINE CGAffineTransform CGEAffineTransformPostflipHorizontallyInRect(CGAffineTransform transform, CGRect rect)
{
    return CGEAffineTransformPostflipHorizontallyWithWidth(transform, rect.size.width + 2.0 * rect.origin.x);
}

CG_INLINE CGAffineTransform CGEAffineTransformPreflipVerticallyInRect(CGAffineTransform transform, CGRect rect)
{
    return CGEAffineTransformPreflipVerticallyWithHeight(transform, rect.size.height + 2.0 * rect.origin.y);
}

CG_INLINE CGAffineTransform CGEAffineTransformPostflipVerticallyInRect(CGAffineTransform transform, CGRect rect)
{
    return CGEAffineTransformPostflipVerticallyWithHeight(transform, rect.size.height + 2.0 * rect.origin.y);
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformInvert(CGAffineTransform transform)
{
    CGFloat determinant = transform.a * transform.d - transform.b * transform.c;
    
    if (determinant != 0.0)
    {
        CGFloat temp = transform.d / determinant;
        transform.d = transform.a / determinant;
        transform.a = temp;
        
        transform.b /= -determinant;
        transform.c /= -determinant;
        
        temp = -(transform.a * transform.tx + transform.c * transform.ty);
        transform.ty = -(transform.b * transform.tx + transform.d * transform.ty);
        transform.tx = temp;
    }
    
    return transform;
}

#pragma mark -

CG_INLINE CGAffineTransform CGEAffineTransformRemoveSizeScale(CGAffineTransform transform, CGFloat* removedWidthScaleFactorPtr, CGFloat* removedHeightScaleFactorPtr)
{
    CGFloat widthScaleFactor = CGEAffineTransformGetWidthScaleFactor(transform);
    CGFloat heightScaleFactor = CGEAffineTransformGetHeightScaleFactor(transform);
    
    if (removedWidthScaleFactorPtr != NULL)
        *removedWidthScaleFactorPtr = widthScaleFactor;
    if (removedHeightScaleFactorPtr != NULL)
        *removedHeightScaleFactorPtr = heightScaleFactor;
    
    return CGEAffineTransformPrescale(transform, (widthScaleFactor == 0.0) ? 1.0 : 1.0 / widthScaleFactor, (heightScaleFactor == 0.0) ? 1.0 : 1.0 / heightScaleFactor);
}

CG_EXTERN_C_END

#endif // CGEAffineTransform_H_
