/*!
    @header CGEBitmapContext.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 2007/04/18
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGEBitmapContext_H_
#define CGEBitmapContext_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGBitmapContext.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColorSpace.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>
#include <CoreFoundation/CFBase.h>
#include <SMKCGExtensions/CGEFloat.h>

CG_EXTERN_C_BEGIN

CG_EXTERN size_t CGEBitmapContextGetNumberOfComponents(CGContextRef bitmapContext); // Total number of components: color + alpha.

#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4
    #define CGEBitmapContextGetBitmapInfo CGBitmapContextGetBitmapInfo
#else
    #define CGEBitmapContextGetBitmapInfo __CGEBitmapContextGetBitmapInfo
    
    CG_EXTERN CGBitmapInfo CGEBitmapContextGetBitmapInfo(CGContextRef bitmapContext);
#endif // MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4

// Creates and returns an image that represents content of a bitmap context. Result image does not own underlying bitmap data.
// Note: result image may change as bitmap data changes, the image may cache its data though and therefore there is no guarantee that it reflects bitmap data changes after it was created (re-creating the image is a way to ensure that image reflects changes of bitmap context's data).
// `decode' may be NULL.
CG_EXTERN CGImageRef CGEBitmapContextCreateImageNoCopy(CGContextRef bitmapContext, const CGFloat decode[], bool shouldInterpolate, CGColorRenderingIntent intent);

// Creates and returns an image mask that represents content of a bitmap context: for single-component bitmap contexts mask is created of that component's channel, for multi-component alpha-first bitmap contexts mask is created of alpha channel, for bitmap contexts of other formats result is nil. Result image mask does not own underlying bitmap data.
// Note: result image mask may change as bitmap data changes, the image mask may cache its data though and therefore there is no guarantee that it reflects bitmap data changes after it was created (re-creating the image mask is a way to ensure that image mask reflects changes of bitmap context's data).
// `decode' may be NULL.
CG_EXTERN CGImageRef CGEBitmapContextCreateImageMaskNoCopy(CGContextRef bitmapContext, const CGFloat decode[], bool shouldInterpolate);

CG_EXTERN CFStringRef CGEBitmapContextCopyDescription(CGContextRef bitmapContext);
CG_EXTERN void CGEBitmapContextShow(CGContextRef bitmapContext);

CG_EXTERN_C_END

#endif // CGEBitmapContext_H_
