/*!
    @header CGEBitmapLayer.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 2007/04/11
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
    @abstract Wrapper for CoreGraphics bitmap context & image.
    @discussion Note: CGEBitmapLayerRef is not CF-type.
*/

#ifndef CGEBitmapLayer_H_
#define CGEBitmapLayer_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColorSpace.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>
#include <CoreFoundation/CFBase.h>
#include <CoreFoundation/CFData.h>
#include <SMKCGExtensions/CGEFloat.h>

CG_EXTERN_C_BEGIN

typedef struct CGEBitmapLayer* CGEBitmapLayerRef;

// `data' may be nil: it is allocated.
// `bytesPerRow' may be nil if `data' is nil: it is calculated.
CG_EXTERN CGEBitmapLayerRef CGEBitmapLayerCreate(CFMutableDataRef data, size_t width, size_t height, size_t bitsPerComponent, size_t bytesPerRow, CGColorSpaceRef colorSpace, CGBitmapInfo bitmapInfo);

CG_EXTERN CGEBitmapLayerRef CGEBitmapLayerRetain(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN void CGEBitmapLayerRelease(CGEBitmapLayerRef bitmapLayer);

CG_EXTERN void CGEBitmapLayerBeginEditing(CGEBitmapLayerRef bitmapLayer);

// Returned context & mutable data should only be used between `CGEBitmapLayerBeginEditing' & `CGEBitmapLayerEndEditing' calls.
CG_EXTERN CGContextRef CGEBitmapLayerGetContext(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN CFMutableDataRef CGEBitmapLayerGetMutableData(CGEBitmapLayerRef bitmapLayer);

CG_EXTERN void CGEBitmapLayerEndEditing(CGEBitmapLayerRef bitmapLayer);

CG_EXTERN CFDataRef CGEBitmapLayerGetData(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN size_t CGEBitmapLayerGetWidth(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN size_t CGEBitmapLayerGetHeight(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN size_t CGEBitmapLayerGetBitsPerComponent(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN size_t CGEBitmapLayerGetNumberOfComponents(CGEBitmapLayerRef bitmapLayer); // Total number of components: color + alpha.
CG_EXTERN size_t CGEBitmapLayerGetBitsPerPixel(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN size_t CGEBitmapLayerGetBytesPerRow(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN CGColorSpaceRef CGEBitmapLayerGetColorSpace(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN CGBitmapInfo CGEBitmapLayerGetBitmapInfo(CGEBitmapLayerRef bitmapLayer);

// Creates and returns an image (mask) that represents content of a bitmap layer.
// Note: result image (mask) is not a copy of a bitmap layer and may change as bitmap data changes, the image (mask) may cache its data though and therefore there is no guarantee that it reflects bitmap data changes after it was created (re-creating the image (mask) is a way to ensure that image (mask) reflects changes of bitmap layer's data).
// `decode' may be NULL.
CG_EXTERN CGImageRef CGEBitmapLayerCreateImage(CGEBitmapLayerRef bitmapLayer, const CGFloat decode[], bool shouldInterpolate, CGColorRenderingIntent intent);
CG_EXTERN CGImageRef CGEBitmapLayerCreateImageMask(CGEBitmapLayerRef bitmapLayer, const CGFloat decode[], bool shouldInterpolate);

CG_EXTERN CFStringRef CGEBitmapLayerCopyDescription(CGEBitmapLayerRef bitmapLayer);
CG_EXTERN void CGEBitmapLayerShow(CGEBitmapLayerRef bitmapLayer);

CG_EXTERN_C_END

#endif // CGEBitmapLayer_H_
