/*!
    @header CGEColorSpace.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 01/24/2007
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGEColorSpace_H_
#define CGEColorSpace_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColorSpace.h>
#include <CoreServices/../Frameworks/CarbonCore.framework/Headers/MacTypes.h>
#include <ApplicationServices/../Frameworks/ColorSync.framework/Headers/CMTypes.h>
#include <ApplicationServices/../Frameworks/ColorSync.framework/Headers/CMICCProfile.h>
#include <ApplicationServices/../Frameworks/ColorSync.framework/Headers/CMApplication.h>

CG_EXTERN_C_BEGIN

CG_EXTERN UInt8 CGEColorSpaceProfileGetMajorVersion(CMProfileRef profile, CMAppleProfileHeader* profileHeaderPtr); // `profileHeaderPtr' may be NULL.
CG_EXTERN OSType CGEColorSpaceProfileGetColorSpaceSignature(CMProfileRef profile);
CG_EXTERN CGColorRenderingIntent CGEColorSpaceProfileGetColorRenderingIntent(CMProfileRef profile);

// These return default color space for display use if it can be created and is of requested color space model; otherwise return device color space of requested color space model.
CG_EXTERN CGColorSpaceRef CGEColorSpaceCreateDefaultDisplayGray();
CG_EXTERN CGColorSpaceRef CGEColorSpaceCreateDefaultDisplayRGB();
CG_EXTERN CGColorSpaceRef CGEColorSpaceCreateDefaultDisplayCMYK();

CG_INLINE CGColorRenderingIntent CGEColorRenderingIntentForColorSync1MatchOption(CMMatchOption colorSync1MatchOption);
CG_INLINE CGColorRenderingIntent CGEColorRenderingIntentForColorSync2RenderingIntent(UInt32 colorSync2RenderingIntent);

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

CG_INLINE CGColorRenderingIntent CGEColorRenderingIntentForColorSync1MatchOption(CMMatchOption colorSync1MatchOption)
{
    CGColorRenderingIntent result = kCGRenderingIntentDefault;
    
    switch (colorSync1MatchOption)
    {
        case cmPerceptualMatch/*CMPerceptualMatch*/:
            result = kCGRenderingIntentPerceptual;
            break;
        
        case cmColorimetricMatch/*CMColorimetricMatch*/:
            result = kCGRenderingIntentRelativeColorimetric;
            break;
        
        case cmSaturationMatch/*CMSaturationMatch*/:
            result = kCGRenderingIntentSaturation;
            break;
        
        default:
            break;
    }
    
    return result;
}

CG_INLINE CGColorRenderingIntent CGEColorRenderingIntentForColorSync2RenderingIntent(UInt32 colorSync2RenderingIntent)
{
    CGColorRenderingIntent result = kCGRenderingIntentDefault;
    
    switch (colorSync2RenderingIntent)
    {
        case cmPerceptual:
            result = kCGRenderingIntentPerceptual;
            break;
        
        case cmRelativeColorimetric:
            result = kCGRenderingIntentRelativeColorimetric;
            break;
        
        case cmSaturation:
            result = kCGRenderingIntentSaturation;
            break;
        
        case cmAbsoluteColorimetric:
            result = kCGRenderingIntentAbsoluteColorimetric;
            break;
        
        default:
            break;
    }
    
    return result;
}

CG_EXTERN_C_END

#endif // CGEColorSpace_H_
