/*!
    @header CGEFloat.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 01/11/2007
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGEFloat_H_
#define CGEFloat_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGBase.h>
#include <math.h>

CG_EXTERN_C_BEGIN

#if !CGFLOAT_DEFINED
    typedef float CGFloat;
    #define CGFLOAT_MIN FLT_MIN
    #define CGFLOAT_MAX FLT_MAX
    #define CGFLOAT_IS_DOUBLE 0
    
    #define CGFLOAT_DEFINED 1
#endif // !CGFLOAT_DEFINED

#if CGFLOAT_IS_DOUBLE
    #define CGEFLOAT_MANT_DIG DBL_MANT_DIG
    #define CGEFLOAT_DIG DBL_DIG
    #define CGEFLOAT_MIN_EXP DBL_MIN_EXP
    #define CGEFLOAT_MIN_10_EXP DBL_MIN_10_EXP
    #define CGEFLOAT_MAX_EXP DBL_MAX_EXP
    #define CGEFLOAT_MAX_10_EXP DBL_MAX_10_EXP
    #define CGEFLOAT_EPSILON DBL_EPSILON
#else
    #define CGEFLOAT_MANT_DIG FLT_MANT_DIG
    #define CGEFLOAT_DIG FLT_DIG
    #define CGEFLOAT_MIN_EXP FLT_MIN_EXP
    #define CGEFLOAT_MIN_10_EXP FLT_MIN_10_EXP
    #define CGEFLOAT_MAX_EXP FLT_MAX_EXP
    #define CGEFLOAT_MAX_10_EXP FLT_MAX_10_EXP
    #define CGEFLOAT_EPSILON FLT_EPSILON
#endif // CGFLOAT_IS_DOUBLE

#if MAC_OS_X_VERSION_MIN_REQUIRED > MAC_OS_X_VERSION_10_4
    #define kCFNumberCGEFloatType kCFNumberCGFloatType
#else
    #if CGFLOAT_IS_DOUBLE
        #define kCFNumberCGEFloatType kCFNumberDoubleType
    #else
        #define kCFNumberCGEFloatType kCFNumberFloatType
    #endif // CGFLOAT_IS_DOUBLE
#endif // MAC_OS_X_VERSION_MIN_REQUIRED > MAC_OS_X_VERSION_10_4

CG_INLINE CGFloat CGEFloat_min(CGFloat x, CGFloat y);
CG_INLINE CGFloat CGEFloat_max(CGFloat x, CGFloat y);

CG_INLINE CGFloat CGEFloat_abs(CGFloat x);
CG_INLINE CGFloat CGEFloat_copysign(CGFloat x, CGFloat y);

CG_INLINE CGFloat CGEFloat_nextafter(CGFloat x, CGFloat y);

CG_INLINE CGFloat CGEFloat_rint(CGFloat x);
CG_INLINE CGFloat CGEFloat_round(CGFloat x);
CG_INLINE CGFloat CGEFloat_nearbyint(CGFloat x);
CG_INLINE CGFloat CGEFloat_floor(CGFloat x);
CG_INLINE CGFloat CGEFloat_ceil(CGFloat x);
CG_INLINE CGFloat CGEFloat_trunc(CGFloat x);

CG_INLINE long int CGEFloat_lrint(CGFloat x);
CG_INLINE long long int CGEFloat_llrint(CGFloat x);
CG_INLINE long int CGEFloat_lround(CGFloat x);
CG_INLINE long long int CGEFloat_llround(CGFloat x);

CG_INLINE CGFloat CGEFloat_mod(CGFloat x, CGFloat y);
CG_INLINE CGFloat CGEFloat_modf(CGFloat value, CGFloat* integralPartPtr); // `integralPartPtr' may be nil.

CG_INLINE CGFloat CGEFloat_cos(CGFloat x);
CG_INLINE CGFloat CGEFloat_sin(CGFloat x);
CG_INLINE CGFloat CGEFloat_tan(CGFloat x);

CG_INLINE CGFloat CGEFloat_acos(CGFloat x);
CG_INLINE CGFloat CGEFloat_asin(CGFloat x);
CG_INLINE CGFloat CGEFloat_atan(CGFloat x);
CG_INLINE CGFloat CGEFloat_atan2(CGFloat y, CGFloat x);
CG_INLINE CGFloat CGEFloat_atan2p(CGFloat y, CGFloat x); // Returns principal value of the arc tangent of y/x. The result is in the range [-pi/2, +pi/2].

CG_INLINE CGFloat CGEFloat_cosh(CGFloat x);
CG_INLINE CGFloat CGEFloat_sinh(CGFloat x);
CG_INLINE CGFloat CGEFloat_tanh(CGFloat x);

CG_INLINE CGFloat CGEFloat_acosh(CGFloat x);
CG_INLINE CGFloat CGEFloat_asinh(CGFloat x);
CG_INLINE CGFloat CGEFloat_atanh(CGFloat x);

CG_INLINE CGFloat CGEFloat_pow(CGFloat x, CGFloat y);

CG_INLINE CGFloat CGEFloat_sqrt(CGFloat x);
CG_INLINE CGFloat CGEFloat_cbrt(CGFloat x);

CG_INLINE CGFloat CGEFloat_exp(CGFloat x);
CG_INLINE CGFloat CGEFloat_exp2(CGFloat x);
CG_INLINE CGFloat CGEFloat_expm1(CGFloat x);

CG_INLINE CGFloat CGEFloat_log(CGFloat x);
CG_INLINE CGFloat CGEFloat_log2(CGFloat x);
CG_INLINE CGFloat CGEFloat_log10(CGFloat x);
CG_INLINE CGFloat CGEFloat_log1p(CGFloat x);

CG_INLINE CGFloat CGEFloat_logb(CGFloat x);
CG_INLINE int CGEFloat_ilogb(CGFloat x);
CG_INLINE CGFloat CGEFloat_ldexp(CGFloat x, int n);
CG_INLINE CGFloat CGEFloat_frexp(CGFloat x, int* exponentPtr);

CG_INLINE CGFloat CGEFloat_dim(CGFloat x, CGFloat y); // (x > y) ? x - y : +0
CG_INLINE CGFloat CGEFloat_ma(CGFloat x, CGFloat y, CGFloat z); // x * y + z
CG_INLINE CGFloat CGEFloat_hypot(CGFloat x, CGFloat y); // square root of (x * x + y * y)
CG_INLINE CGFloat CGEFloat_erf(CGFloat x); // 2 / sqrt(pi) * integral from 0 to x of exp(-t * t) dt
CG_INLINE CGFloat CGEFloat_erfc(CGFloat x); // 1 - erf(x)
CG_INLINE CGFloat CGEFloat_lgamma(CGFloat x);
CG_INLINE CGFloat CGEFloat_tgamma(CGFloat x);

#pragma mark -

CG_INLINE long int CGEFloatRoundInteger(CGFloat x);
CG_INLINE long int CGEFloatNearbyInteger(CGFloat x);
CG_INLINE long int CGEFloatFloorInteger(CGFloat x);
CG_INLINE long int CGEFloatCeilInteger(CGFloat x);
CG_INLINE long int CGEFloatTruncateInteger(CGFloat x);

CG_INLINE size_t CGEFloatSizeInteger(CGFloat x);

#pragma mark -

CG_INLINE bool CGEFloatEqualWithDelta(CGFloat x, CGFloat y, CGFloat delta);
CG_INLINE bool CGEFloatLessWithDelta(CGFloat x, CGFloat y, CGFloat delta);
CG_INLINE bool CGEFloatLessOrEqualWithDelta(CGFloat x, CGFloat y, CGFloat delta);
CG_INLINE bool CGEFloatGreaterWithDelta(CGFloat x, CGFloat y, CGFloat delta);
CG_INLINE bool CGEFloatGreaterOrEqualWithDelta(CGFloat x, CGFloat y, CGFloat delta);

CG_INLINE CGFloat CGEFloatComparisonDeltaWithPresicionExponent(const CGFloat* values, size_t count, int presicionExponent);

// These methods calculate comparison delta and are good if powers of `x' & `y' are unknown in advance. Take note that they treat zero as "unique" value (i.e. only zero is equal to zero, any deviation results in inequality).
// `presicionExponent' should be chosen taking into account computations in effect for `x' & `y' and definition of "CGFloat" type ("float" or "double").
CG_INLINE bool CGEFloatEqualWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent);
CG_INLINE bool CGEFloatLessWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent);
CG_INLINE bool CGEFloatLessOrEqualWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent);
CG_INLINE bool CGEFloatGreaterWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent);
CG_INLINE bool CGEFloatGreaterOrEqualWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent);

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

CG_INLINE CGFloat CGEFloat_min(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return fmin(x, y);
#else
    return fminf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_max(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return fmax(x, y);
#else
    return fmaxf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_abs(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return fabs(x);
#else
    return fabsf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_copysign(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return copysign(x, y);
#else
    return copysignf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_nextafter(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return nextafter(x, y);
#else
    return nextafterf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_rint(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return rint(x);
#else
    return rintf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_round(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return round(x);
#else
    return roundf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_nearbyint(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return nearbyint(x);
#else
    return nearbyintf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_floor(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return floor(x);
#else
    return floorf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_ceil(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return ceil(x);
#else
    return ceilf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_trunc(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return trunc(x);
#else
    return truncf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE long int CGEFloat_lrint(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return lrint(x);
#else
    return lrintf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE long long int CGEFloat_llrint(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return llrint(x);
#else
    return llrintf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE long int CGEFloat_lround(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return lround(x);
#else
    return lroundf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE long long int CGEFloat_llround(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return llround(x);
#else
    return llroundf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_mod(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return fmod(x, y);
#else
    return fmodf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_modf(CGFloat value, CGFloat* integralPartPtr)
{
#if CGFLOAT_IS_DOUBLE
    return modf(value, (integralPartPtr != nil) ? integralPartPtr : &value);
#else
    return modff(value, (integralPartPtr != nil) ? integralPartPtr : &value);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_cos(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return cos(x);
#else
    return cosf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_sin(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return sin(x);
#else
    return sinf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_tan(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return tan(x);
#else
    return tanf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_acos(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return acos(x);
#else
    return acosf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_asin(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return asin(x);
#else
    return asinf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_atan(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return atan(x);
#else
    return atanf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_atan2(CGFloat y, CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return atan2(y, x);
#else
    return atan2f(y, x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_atan2p(CGFloat y, CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    CGFloat result = CGEFloat_atan2(y, x);
    if (result < -M_PI_2)
        result += M_PI;
    else if (result > M_PI_2)
        result -= M_PI;
#else // :nav:20070731 Workaround: "float" type is very imprecise, so make sure the result is in range [-pi/2, +pi/2].
    double resultDouble = atan2(y, x);
    if (resultDouble < -M_PI_2)
        resultDouble += M_PI;
    else if (resultDouble > M_PI_2)
        resultDouble -= M_PI;
    
    CGFloat result = CGEFloat_nextafter(resultDouble, 0.0);
#endif // CGFLOAT_IS_DOUBLE
        
    return result;
}

CG_INLINE CGFloat CGEFloat_cosh(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return cosh(x);
#else
    return coshf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_sinh(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return sinh(x);
#else
    return sinhf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_tanh(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return tanh(x);
#else
    return tanhf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_acosh(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return acosh(x);
#else
    return acoshf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_asinh(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return asinh(x);
#else
    return asinhf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_atanh(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return atanh(x);
#else
    return atanhf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_pow(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return pow(x, y);
#else
    return powf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_sqrt(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return sqrt(x);
#else
    return sqrtf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_cbrt(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return cbrt(x);
#else
    return cbrtf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_exp(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return exp(x);
#else
    return expf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_exp2(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return exp2(x);
#else
    return exp2f(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_expm1(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return expm1(x);
#else
    return expm1f(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_log(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return log(x);
#else
    return logf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_log2(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return log2(x);
#else
    return log2f(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_log10(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return log10(x);
#else
    return log10f(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_log1p(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return log1p(x);
#else
    return log1pf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_logb(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return logb(x);
#else
    return logbf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE int CGEFloat_ilogb(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return ilogb(x);
#else
    return ilogbf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_ldexp(CGFloat x, int n)
{
#if CGFLOAT_IS_DOUBLE
    return ldexp(x, n);
#else
    return ldexpf(x, n);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_frexp(CGFloat x, int* exponentPtr)
{
#if CGFLOAT_IS_DOUBLE
    return frexp(x, exponentPtr);
#else
    return frexpf(x, exponentPtr);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_dim(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return fdim(x, y);
#else
    return fdimf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_ma(CGFloat x, CGFloat y, CGFloat z)
{
#if CGFLOAT_IS_DOUBLE
    return fma(x, y, z);
#else
    return fmaf(x, y, z);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_hypot(CGFloat x, CGFloat y)
{
#if CGFLOAT_IS_DOUBLE
    return hypot(x, y);
#else
    return hypotf(x, y);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_erf(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return erf(x);
#else
    return erff(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_erfc(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return erfc(x);
#else
    return erfcf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_lgamma(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return lgamma(x);
#else
    return lgammaf(x);
#endif // CGFLOAT_IS_DOUBLE
}

CG_INLINE CGFloat CGEFloat_tgamma(CGFloat x)
{
#if CGFLOAT_IS_DOUBLE
    return tgamma(x);
#else
    return tgammaf(x);
#endif // CGFLOAT_IS_DOUBLE
}

#pragma mark -

CG_INLINE long int CGEFloatRoundInteger(CGFloat x)
{
    return CGEFloat_lround(x);
}

CG_INLINE long int CGEFloatNearbyInteger(CGFloat x)
{
    return CGEFloatRoundInteger(CGEFloat_nearbyint(x));
}

CG_INLINE long int CGEFloatFloorInteger(CGFloat x)
{
    return CGEFloatRoundInteger(CGEFloat_floor(x));
}

CG_INLINE long int CGEFloatCeilInteger(CGFloat x)
{
    return CGEFloatRoundInteger(CGEFloat_ceil(x));
}

CG_INLINE long int CGEFloatTruncateInteger(CGFloat x)
{
    return CGEFloatRoundInteger(CGEFloat_trunc(x));
}

CG_INLINE size_t CGEFloatSizeInteger(CGFloat x)
{
    if (x <= 0.0)
        return 0;
    
    return CGEFloatCeilInteger(x);
}

#pragma mark -

CG_INLINE bool CGEFloatEqualWithDelta(CGFloat x, CGFloat y, CGFloat delta)
{
	return (x >= (y - delta)) && (x <= (y + delta));
}

CG_INLINE bool CGEFloatLessWithDelta(CGFloat x, CGFloat y, CGFloat delta)
{
	return (x < (y - delta));
}

CG_INLINE bool CGEFloatLessOrEqualWithDelta(CGFloat x, CGFloat y, CGFloat delta)
{
	return (x <= (y + delta));
}

CG_INLINE bool CGEFloatGreaterWithDelta(CGFloat x, CGFloat y, CGFloat delta)
{
	return (x > (y + delta));
}

CG_INLINE bool CGEFloatGreaterOrEqualWithDelta(CGFloat x, CGFloat y, CGFloat delta)
{
	return (x >= (y - delta));
}

#pragma mark -

CG_INLINE CGFloat CGEFloatComparisonDeltaWithPresicionExponent(const CGFloat* values, size_t count, int presicionExponent)
{
    int n = CGEFLOAT_MIN_EXP;
    const CGFloat* valuesEnd = values + count;
    
    for (; values < valuesEnd; values++)
    {
        if (*values != 0.0)
        {
            int n0 = CGEFloat_ilogb(*values);
            
            if (n0 > n)
                n = n0;
        }
    }
    
    return CGEFloat_ldexp(1.0, n + presicionExponent);
}

#pragma mark -

CG_INLINE bool CGEFloatEqualWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent)
{
    CGFloat values[] = {x, y};
    CGFloat delta = CGEFloatComparisonDeltaWithPresicionExponent(values, sizeof(values) / sizeof(CGFloat), presicionExponent);
    
	return CGEFloatEqualWithDelta(x, y, delta);
}

CG_INLINE bool CGEFloatLessWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent)
{
    CGFloat values[] = {x, y};
    CGFloat delta = CGEFloatComparisonDeltaWithPresicionExponent(values, sizeof(values) / sizeof(CGFloat), presicionExponent);
    
	return CGEFloatLessWithDelta(x, y, delta);
}

CG_INLINE bool CGEFloatLessOrEqualWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent)
{
    CGFloat values[] = {x, y};
    CGFloat delta = CGEFloatComparisonDeltaWithPresicionExponent(values, sizeof(values) / sizeof(CGFloat), presicionExponent);
    
	return CGEFloatLessOrEqualWithDelta(x, y, delta);
}

CG_INLINE bool CGEFloatGreaterWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent)
{
    CGFloat values[] = {x, y};
    CGFloat delta = CGEFloatComparisonDeltaWithPresicionExponent(values, sizeof(values) / sizeof(CGFloat), presicionExponent);
    
	return CGEFloatGreaterWithDelta(x, y, delta);
}

CG_INLINE bool CGEFloatGreaterOrEqualWithPresicionExponent(CGFloat x, CGFloat y, int presicionExponent)
{
    CGFloat values[] = {x, y};
    CGFloat delta = CGEFloatComparisonDeltaWithPresicionExponent(values, sizeof(values) / sizeof(CGFloat), presicionExponent);
    
	return CGEFloatGreaterOrEqualWithDelta(x, y, delta);
}

CG_EXTERN_C_END

#endif // CGEFloat_H_
