/*!
    @header CGEImage.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 01/12/2007
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGEImage_H_
#define CGEImage_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>
#include <SMKCGExtensions/CGEFloat.h>
#include <CoreFoundation/CFBase.h>
#include <CoreFoundation/CFData.h>

CG_EXTERN_C_BEGIN

#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4
    #define CGEImageCreateCopy CGImageCreateCopy
#else
    #define CGEImageCreateCopy __CGEImageCreateCopy
    
    CG_EXTERN CGImageRef CGEImageCreateCopy(CGImageRef image);
#endif // MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4

CG_EXTERN CGImageRef CGEImageCreateWithCFData(size_t width, size_t height, size_t bitsPerComponent, size_t bitsPerPixel, size_t bytesPerRow, CGColorSpaceRef colorSpace, CGBitmapInfo bitmapInfo, CFDataRef data, const CGFloat decode[], bool shouldInterpolate, CGColorRenderingIntent intent);

CG_EXTERN CGImageRef CGEImageMaskCreateWithCFData(size_t width, size_t height, size_t bitsPerComponent, size_t bitsPerPixel, size_t bytesPerRow, CFDataRef data, const CGFloat decode[], bool shouldInterpolate);

// Creates and returns an image (mask) with the same attributes as `bitmapContext'. For image mask: for single-component bitmap contexts mask is created of that component's channel, for multi-component alpha-first bitmap contexts mask is created of alpha channel, for bitmap contexts of other formats result is nil.
// `decode' may be NULL.
CG_EXTERN CGImageRef CGEImageCreateAsBitmapContext(CGContextRef bitmapContext, CGDataProviderRef bitmapDataProvider, const CGFloat decode[], bool shouldInterpolate, CGColorRenderingIntent intent);
CG_EXTERN CGImageRef CGEImageMaskCreateAsBitmapContext(CGContextRef bitmapContext, CGDataProviderRef bitmapDataProvider, const CGFloat decode[], bool shouldInterpolate);
CG_EXTERN CGImageRef CGEImageCreateAsBitmapContextWithCFData(CGContextRef bitmapContext, CFDataRef bitmapData, const CGFloat decode[], bool shouldInterpolate, CGColorRenderingIntent intent);
CG_EXTERN CGImageRef CGEImageMaskCreateAsBitmapContextWithCFData(CGContextRef bitmapContext, CFDataRef bitmapData, const CGFloat decode[], bool shouldInterpolate);

#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4
    #define CGEImageGetBitmapInfo CGImageGetBitmapInfo
#else
    #define CGEImageGetBitmapInfo __CGEImageGetBitmapInfo
    
    CG_EXTERN CGBitmapInfo CGEImageGetBitmapInfo(CGImageRef image);
#endif // MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4

CG_INLINE bool CGEAlphaPresenceForAlphaInfo(CGImageAlphaInfo alphaInfo);
CG_INLINE bool CGEAlphaPresenceForBitmapInfo(CGBitmapInfo bitmapInfo);

CG_EXTERN void CGEImageShow(CGImageRef image);
CG_EXTERN void CGEImageShowAs(CGImageRef image, CFStringRef objectTypeName, const void* objectAddress);

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

CG_INLINE bool CGEAlphaPresenceForAlphaInfo(CGImageAlphaInfo alphaInfo)
{
    switch (alphaInfo)
    {
        case kCGImageAlphaPremultipliedLast:
        case kCGImageAlphaPremultipliedFirst:
        case kCGImageAlphaLast:
        case kCGImageAlphaFirst:
        case kCGImageAlphaOnly:
            return TRUE;
        
        case kCGImageAlphaNone:
        case kCGImageAlphaNoneSkipLast:
        case kCGImageAlphaNoneSkipFirst:
            return FALSE;
        
        default:
            return FALSE;
    }
}

CG_INLINE bool CGEAlphaPresenceForBitmapInfo(CGBitmapInfo bitmapInfo)
{
    return CGEAlphaPresenceForAlphaInfo((CGImageAlphaInfo)(bitmapInfo & kCGBitmapAlphaInfoMask));
}

CG_EXTERN_C_END

#endif // CGEImage_H_
