/*!
    @header CGELayer.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 01/12/2007
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
    @abstract Solid interface for caching: implemented using CG layers on Tiger & later and using CGE bitmap layers on pre-Tiger.
    @discussion Note: CGELayerRef is not CF-type.
*/

#ifndef CGELayer_H_
#define CGELayer_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGLayer.h>
#include <SMKCGExtensions/CGEBitmapLayer.h>

CG_EXTERN_C_BEGIN

typedef struct CGELayer* CGELayerRef;

// `referenceContext' & `auxiliaryInfo' are used to create underlying CG layer and are passed to `CGLayerCreateWithContext'.
// `referenceContext' may be NULL: result layer is based on CGE bitmap layer regardless of OS.
// `auxiliaryInfo' may be NULL.
// Note: unlike `CGLayerCreateWithContext' `size' may not be zero-size.
CG_EXTERN CGELayerRef CGELayerCreate(CGSize size, CGContextRef referenceContext, CFDictionaryRef auxiliaryInfo);

CG_EXTERN CGELayerRef CGELayerCreateWithLayer(CGLayerRef layer) AVAILABLE_MAC_OS_X_VERSION_10_4_AND_LATER;
CG_EXTERN CGELayerRef CGELayerCreateWithBitmapLayer(CGEBitmapLayerRef bitmapLayer);

CG_EXTERN CGELayerRef CGELayerRetain(CGELayerRef layer);
CG_EXTERN void CGELayerRelease(CGELayerRef layer);

CG_EXTERN CGSize CGELayerGetSize(CGELayerRef layer);

// Returns NULL if `layer' is not based on CG layer.
CG_EXTERN CGLayerRef CGELayerGetLayer(CGELayerRef layer) AVAILABLE_MAC_OS_X_VERSION_10_4_AND_LATER;

// Returns NULL if `layer' is not based on CGE bitmap layer.
CG_EXTERN CGEBitmapLayerRef CGELayerGetBitmapLayer(CGELayerRef layer);

CG_EXTERN void CGELayerBeginEditing(CGELayerRef layer);

// Returned context should only be used between `CGELayerBeginEditing' & `CGELayerEndEditing' calls.
CG_EXTERN CGContextRef CGELayerGetContext(CGELayerRef layer);

CG_EXTERN void CGELayerEndEditing(CGELayerRef layer);

CG_EXTERN void CGEContextDrawLayerInRect(CGContextRef context, CGRect rect, CGELayerRef layer);
CG_EXTERN void CGEContextDrawLayerAtPoint(CGContextRef context, CGPoint point, CGELayerRef layer);

CG_EXTERN void CGELayerShow(CGELayerRef layer);

CG_EXTERN_C_END

#endif // CGELayer_H_
