/*!
    @header CGELine.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 2007/09/30
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGELine_H_
#define CGELine_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#include <SMKCGExtensions/CGEFloat.h>

CG_EXTERN_C_BEGIN

struct CGELine
{
    CGFloat a;
    CGFloat b;
    CGFloat c;
};
typedef struct CGELine CGELine;

CG_EXTERN const CGELine CGELineUndefined;

CG_INLINE CGELine CGELineMake(CGFloat a, CGFloat b, CGFloat c);
CG_INLINE CGELine CGELineMakeHorizontal(CGFloat y);
CG_INLINE CGELine CGELineMakeVertical(CGFloat x);
CG_INLINE CGELine CGELineMakeWithPoints(CGPoint point1, CGPoint point2);

CG_INLINE bool CGELineIsDefined(CGELine line);

// *** All functions below are for defined lines only ***

CG_INLINE bool CGELineIsHorizontal(CGELine line);
CG_INLINE bool CGELineIsVertical(CGELine line);

// Example:
//      CGFloat y = CGELineGetOrdinateForAbscissa(line, x);
//      if (isfinite(y))
//      {
//          // Line `line' intersects vertical with abscissa `x' at ordinate `y'.
//      }
//      else if (isnan(y))
//      {
//          // Line `line' coincides with vertical with abscissa `x'.
//      }
//      else
//      {
//          // Line `line' does not intersect vertical with abscissa `x'.
//      }
CG_INLINE CGFloat CGELineGetOrdinateForAbscissa(CGELine line, CGFloat x);
CG_INLINE CGFloat CGELineGetAbscissaForOrdinate(CGELine line, CGFloat y);

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

CG_INLINE CGELine CGELineMake(CGFloat a, CGFloat b, CGFloat c)
{
    CGELine result = {a, b, c};
    
    return result;
}

CG_INLINE CGELine CGELineMakeHorizontal(CGFloat y)
{
    return CGELineMake(0.0, 1.0, -y);
}

CG_INLINE CGELine CGELineMakeVertical(CGFloat x)
{
    return CGELineMake(1.0, 0.0, -x);
}

CG_INLINE CGELine CGELineMakeWithPoints(CGPoint point1, CGPoint point2)
{
    return CGELineMake(point1.y - point2.y, point2.x - point1.x, point1.x * point2.y - point2.x * point1.y);
}

#pragma mark -

CG_INLINE bool CGELineIsDefined(CGELine line)
{
    return (line.a != 0.0) || (line.b != 0.0);
}

#pragma mark -

CG_INLINE bool CGELineIsHorizontal(CGELine line)
{
    return (line.a == 0.0);
}

CG_INLINE bool CGELineIsVertical(CGELine line)
{
    return (line.b == 0.0);
}

#pragma mark -

CG_INLINE CGFloat CGELineGetOrdinateForAbscissa(CGELine line, CGFloat x)
{
    return -(line.a * x + line.c) / line.b;
}

CG_INLINE CGFloat CGELineGetAbscissaForOrdinate(CGELine line, CGFloat y)
{
    return -(line.b * y + line.c) / line.a;
}

CG_EXTERN_C_END

#endif // CGELine_H_
