/*!
    @header CGEParallelogram.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 2007/08/29
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGEParallelogram_H_
#define CGEParallelogram_H_

#include <SMKCGExtensions/CGEFloat.h>
#include <SMKCGExtensions/CGEGeometry.h>
#include <SMKCGExtensions/CGEAffineTransform.h>

CG_EXTERN_C_BEGIN

struct CGEParallelogram
{
    CGSize baseSize;
    CGAffineTransform transform;
};
typedef struct CGEParallelogram CGEParallelogram;

CG_EXTERN const CGEParallelogram CGEParallelogramZero;

CG_INLINE CGEParallelogram CGEParallelogramAssemble(CGSize baseSize, CGAffineTransform transform);
CG_INLINE CGEParallelogram CGEParallelogramMake(CGFloat baseWidth, CGFloat baseHeight, CGAffineTransform transform);
CG_INLINE CGEParallelogram CGEParallelogramMakeWithRect(CGRect rect);
CG_INLINE CGEParallelogram CGEParallelogramMakeWithTransformedRect(CGRect rect, CGAffineTransform transform);
CG_INLINE CGEParallelogram CGEParallelogramCompose(CGFloat baseWidth, CGFloat baseHeight, bool isFlippedHorizontally, bool isFlippedVertically, CGFloat widthScaleFactor, CGFloat heightScaleFactor, CGFloat skewAngle, CGFloat rotationAngle, CGFloat horizontalOffset, CGFloat verticalOffset);

CG_INLINE CGFloat CGEParallelogramGetWidth(CGEParallelogram parallelogram);
CG_INLINE CGFloat CGEParallelogramGetHeight(CGEParallelogram parallelogram);
CG_INLINE CGSize CGEParallelogramGetSize(CGEParallelogram parallelogram);
CG_INLINE CGRect CGEParallelogramGetBoundingRect(CGEParallelogram parallelogram);

// `cornersPtr' should point to a memory block large enough to hold 4 points.
CG_INLINE void CGEParallelogramGetCorners(CGEParallelogram parallelogram, CGPoint* cornersPtr);

CG_INLINE CGEParallelogram CGEParallelogramSetBaseSizeAboutRelativePoint(CGEParallelogram parallelogram, CGSize baseSize, CGPoint resizingRelativeCenter);

CG_INLINE CGEParallelogram CGEParallelogramPreresize(CGEParallelogram parallelogram, CGSize resizingFactor);
CG_INLINE CGEParallelogram CGEParallelogramPreresizeAboutPoint(CGEParallelogram parallelogram, CGSize resizingFactor, CGPoint resizingCenter);
CG_INLINE CGEParallelogram CGEParallelogramPreresizeAboutRelativePoint(CGEParallelogram parallelogram, CGSize resizingFactor, CGPoint resizingRelativeCenter);

CG_INLINE CGEParallelogram CGEParallelogramPrescaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat sx, CGFloat sy, CGPoint scaleRelativeCenter);

CG_INLINE CGEParallelogram CGEParallelogramSetWidthScaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat widthScale, CGPoint scaleRelativeCenter);
CG_INLINE CGEParallelogram CGEParallelogramSetHeightScaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat heightScale, CGPoint scaleRelativeCenter);
CG_INLINE CGEParallelogram CGEParallelogramSetSizeScaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat widthScale, CGFloat heightScale, CGPoint scaleRelativeCenter);

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

CG_INLINE CGEParallelogram CGEParallelogramAssemble(CGSize baseSize, CGAffineTransform transform)
{
    CGEParallelogram result = {baseSize, transform};
    
    return result;
}

CG_INLINE CGEParallelogram CGEParallelogramMake(CGFloat baseWidth, CGFloat baseHeight, CGAffineTransform transform)
{
    return CGEParallelogramAssemble(CGSizeMake(baseWidth, baseHeight), transform);
}

CG_INLINE CGEParallelogram CGEParallelogramMakeWithRect(CGRect rect)
{
    return CGEParallelogramAssemble(rect.size, CGEAffineTransformMakeTranslation(rect.origin.x, rect.origin.y));
}

CG_INLINE CGEParallelogram CGEParallelogramMakeWithTransformedRect(CGRect rect, CGAffineTransform transform)
{
    return CGEParallelogramAssemble(rect.size, CGEAffineTransformPretranslate(transform, rect.origin.x, rect.origin.y));
}

CG_INLINE CGEParallelogram CGEParallelogramCompose(CGFloat baseWidth, CGFloat baseHeight, bool isFlippedHorizontally, bool isFlippedVertically, CGFloat widthScaleFactor, CGFloat heightScaleFactor, CGFloat skewAngle, CGFloat rotationAngle, CGFloat horizontalOffset, CGFloat verticalOffset)
{
    return CGEParallelogramAssemble(CGSizeMake(baseWidth, baseHeight), CGEAffineTransformCompose(isFlippedHorizontally, isFlippedVertically, widthScaleFactor, heightScaleFactor, skewAngle, rotationAngle, horizontalOffset, verticalOffset));
}

CG_INLINE CGFloat CGEParallelogramGetWidth(CGEParallelogram parallelogram)
{
    return parallelogram.baseSize.width * CGEAffineTransformGetWidthScaleFactor(parallelogram.transform);
}

CG_INLINE CGFloat CGEParallelogramGetHeight(CGEParallelogram parallelogram)
{
    return parallelogram.baseSize.height * CGEAffineTransformGetHeightScaleFactor(parallelogram.transform);
}

CG_INLINE CGSize CGEParallelogramGetSize(CGEParallelogram parallelogram)
{
    return CGSizeMake(CGEParallelogramGetWidth(parallelogram), CGEParallelogramGetHeight(parallelogram));
}

CG_INLINE CGRect CGEParallelogramGetBoundingRect(CGEParallelogram parallelogram)
{
    return CGERectApplyAffineTransform(CGERectAssemble(CGPointZero, parallelogram.baseSize), parallelogram.transform);
}

CG_INLINE void CGEParallelogramGetCorners(CGEParallelogram parallelogram, CGPoint* cornersPtr)
{
    cornersPtr[0] = CGPointZero;
    cornersPtr[1] = CGPointMake(0.0, parallelogram.baseSize.height);
    cornersPtr[2] = CGPointMake(parallelogram.baseSize.width, parallelogram.baseSize.height);
    cornersPtr[3] = CGPointMake(parallelogram.baseSize.width, 0.0);
    
    size_t i = 0;
    for (; i < 4; i++)
        cornersPtr[i] = CGPointApplyAffineTransform(cornersPtr[i], parallelogram.transform);
}

CG_INLINE CGEParallelogram CGEParallelogramSetBaseSizeAboutRelativePoint(CGEParallelogram parallelogram, CGSize baseSize, CGPoint resizingRelativeCenter)
{
    parallelogram.transform = CGEAffineTransformPretranslate(parallelogram.transform, (parallelogram.baseSize.width - baseSize.width) * resizingRelativeCenter.x, (parallelogram.baseSize.height - baseSize.height) * resizingRelativeCenter.y);
    parallelogram.baseSize = baseSize;
    return parallelogram;
}

CG_INLINE CGEParallelogram CGEParallelogramPreresize(CGEParallelogram parallelogram, CGSize resizingFactor)
{
    parallelogram.transform = CGEAffineTransformPrescale(parallelogram.transform, CGEFloat_copysign(1.0, resizingFactor.width), CGEFloat_copysign(1.0, resizingFactor.height));
    parallelogram.baseSize.width *= CGEFloat_abs(resizingFactor.width);
    parallelogram.baseSize.height *= CGEFloat_abs(resizingFactor.height);
    return parallelogram;
}

CG_INLINE CGEParallelogram CGEParallelogramPreresizeAboutPoint(CGEParallelogram parallelogram, CGSize resizingFactor, CGPoint resizingCenter)
{
    CGSize baseSize = {parallelogram.baseSize.width * CGEFloat_abs(resizingFactor.width), parallelogram.baseSize.height * CGEFloat_abs(resizingFactor.height)};
    CGAffineTransform compensationTransform = {
        CGEFloat_copysign(1.0, resizingFactor.width),
        0.0,
        0.0,
        CGEFloat_copysign(1.0, resizingFactor.height),
        (1.0 - resizingFactor.width) * resizingCenter.x,
        (1.0 - resizingFactor.height) * resizingCenter.y
    };
    
    parallelogram.transform = CGEAffineTransformMultiply(compensationTransform, parallelogram.transform);
    parallelogram.baseSize = baseSize;
    return parallelogram;
}

CG_INLINE CGEParallelogram CGEParallelogramPreresizeAboutRelativePoint(CGEParallelogram parallelogram, CGSize resizingFactor, CGPoint resizingRelativeCenter)
{
    CGSize baseSize = {parallelogram.baseSize.width * CGEFloat_abs(resizingFactor.width), parallelogram.baseSize.height * CGEFloat_abs(resizingFactor.height)};
    CGAffineTransform compensationTransform = {
        CGEFloat_copysign(1.0, resizingFactor.width),
        0.0,
        0.0,
        CGEFloat_copysign(1.0, resizingFactor.height),
        (parallelogram.baseSize.width - CGEFloat_copysign(baseSize.width, resizingFactor.width)) * resizingRelativeCenter.x,
        (parallelogram.baseSize.height - CGEFloat_copysign(baseSize.height, resizingFactor.height)) * resizingRelativeCenter.y
    };
    
    parallelogram.transform = CGEAffineTransformMultiply(compensationTransform, parallelogram.transform);
    parallelogram.baseSize = baseSize;
    return parallelogram;
}

CG_INLINE CGEParallelogram CGEParallelogramPrescaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat sx, CGFloat sy, CGPoint scaleRelativeCenter)
{
    CGPoint scaleCenter = {parallelogram.baseSize.width * scaleRelativeCenter.x, parallelogram.baseSize.height * scaleRelativeCenter.y};
    
    parallelogram.transform = CGEAffineTransformPrescaleAboutPoint(parallelogram.transform, sx, sy, scaleCenter);
    return parallelogram;
}

CG_INLINE CGEParallelogram CGEParallelogramSetWidthScaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat widthScale, CGPoint scaleRelativeCenter)
{
    CGPoint scaleCenter = {parallelogram.baseSize.width * scaleRelativeCenter.x, parallelogram.baseSize.height * scaleRelativeCenter.y};
    
    parallelogram.transform = CGEAffineTransformSetWidthScaleAboutPoint(parallelogram.transform, widthScale, scaleCenter);
    return parallelogram;
}

CG_INLINE CGEParallelogram CGEParallelogramSetHeightScaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat heightScale, CGPoint scaleRelativeCenter)
{
    CGPoint scaleCenter = {parallelogram.baseSize.width * scaleRelativeCenter.x, parallelogram.baseSize.height * scaleRelativeCenter.y};
    
    parallelogram.transform = CGEAffineTransformSetHeightScaleAboutPoint(parallelogram.transform, heightScale, scaleCenter);
    return parallelogram;
}

CG_INLINE CGEParallelogram CGEParallelogramSetSizeScaleAboutRelativePoint(CGEParallelogram parallelogram, CGFloat widthScale, CGFloat heightScale, CGPoint scaleRelativeCenter)
{
    CGPoint scaleCenter = {parallelogram.baseSize.width * scaleRelativeCenter.x, parallelogram.baseSize.height * scaleRelativeCenter.y};
    
    parallelogram.transform = CGEAffineTransformSetSizeScaleAboutPoint(parallelogram.transform, widthScale, heightScale, scaleCenter);
    return parallelogram;
}

CG_EXTERN_C_END

#endif // CGEParallelogram_H_
