/*!
    @header CGEPath.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 2007/03/01
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGEPath_H_
#define CGEPath_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPath.h>
#include <SMKCGExtensions/CGEFloat.h>
#include <SMKCGExtensions/CGEGeometry.h>

CG_EXTERN_C_BEGIN

enum
{
    kCGEVectorizationPerformFullScan = 1 << 0, // Indicates to perform scanning of all bitmap (start point is used only to calculate master color).
    kCGEVectorizationTreatZeroAlphaAsMasterColor = 1 << 1, // Indicates to treat zero alpha as master color.
    kCGEVectorizationInvertMasterColorStatus = 1 << 2, // Indicates to search for non-master color areas (master color indicates holes). Has effect only if kCGEVectorizationPerformFullScan is set.
    kCGEVectorizationIgnoreAlpha = 1 << 3 // Indicates to ignore alpha component of colors: transparent color is treated as separate color, other colors are treated as opaque, alpha component of master color is ignored.
};
typedef uint32_t CGEVectorizationFlags;

// Pixelizes line from `point1' to `point2' with `pixelizationStep' and adds result polyline to `path'.
// `transform' may be nil.
// If `startPointPosition' is kCGEPixelizationPointPositionExternal, then current point of `path' is used as pixelization start point. Otherwise, new sub-path is begun.
// `endPointPosition' may not be kCGEPixelizationPointPositionExternal.
CG_EXTERN void CGEPathAddPixelizedLine(CGMutablePathRef path, const CGAffineTransform* transform, CGPoint point1, CGPoint point2, CGSize pixelizationStep, CGEPixelizationPointPosition startPointPosition, CGEPixelizationPointPosition endPointPosition);

// `transform' may be nil.
// `newSubpathsBoundsPtr' may be nil.
CG_EXTERN bool CGEPathAddPathsFromGrayscale1Bitmap(CGMutablePathRef path, const CGAffineTransform* transform, const void* bitmapData, size_t bitmapWidth, size_t bitmapHeight, size_t bitmapBytesPerRow, CGPoint startPoint, CGFloat tolerance, size_t maxNestingLevel, CGEVectorizationFlags vectorizationFlags, CGRect* newSubpathsBoundsPtr);

// `transform' may be nil.
// `newSubpathsBoundsPtr' may be nil.
// `masterColor' is in 1-bit grayscale format.
// Note: this function always performs full scan regardless of kCGEVectorizationPerformFullScan flag.
CG_EXTERN bool CGEPathAddColoredPathsFromGrayscale1Bitmap(CGMutablePathRef path, const CGAffineTransform* transform, const void* bitmapData, size_t bitmapWidth, size_t bitmapHeight, size_t bitmapBytesPerRow, unsigned char masterColor, CGFloat tolerance, size_t maxNestingLevel, CGEVectorizationFlags vectorizationFlags, CGRect* newSubpathsBoundsPtr);

// `transform' may be nil.
// `newSubpathsBoundsPtr' may be nil.
CG_EXTERN bool CGEPathAddPathsFromARGB8888PremultipliedBitmap(CGMutablePathRef path, const CGAffineTransform* transform, const void* bitmapData, size_t bitmapWidth, size_t bitmapHeight, size_t bitmapBytesPerRow, CGPoint startPoint, CGFloat tolerance, size_t maxNestingLevel, CGEVectorizationFlags vectorizationFlags, CGRect* newSubpathsBoundsPtr);

// `transform' may be nil.
// `newSubpathsBoundsPtr' may be nil.
// `masterColor' is in ARGB8888 format.
// Note: this function always performs full scan regardless of kCGEVectorizationPerformFullScan flag.
CG_EXTERN bool CGEPathAddColoredPathsFromARGB8888PremultipliedBitmap(CGMutablePathRef path, const CGAffineTransform* transform, const void* bitmapData, size_t bitmapWidth, size_t bitmapHeight, size_t bitmapBytesPerRow, const unsigned char masterColor[4], CGFloat tolerance, size_t maxNestingLevel, CGEVectorizationFlags vectorizationFlags, CGRect* newSubpathsBoundsPtr);

CG_EXTERN void CGEPathShow(CGPathRef path, const CGAffineTransform* transform);

CG_EXTERN_C_END

#endif // CGEPath_H_
