/*!
    @header CGERegion.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 2008/04/15
    @copyright Copyright (c) 2008 The Software MacKiev Company. All rights reserved.
    @discussion CGERegion is designed to represent approximate coverage of an area used by some drawings (i.e. region's area always covers represented area). This makes CGERegion suitable for representing invalidation areas. Note: CGERegionRef and CGEMutableRegionRef are not CF-types.
*/

#ifndef CGERegion_H_
#define CGERegion_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGAffineTransform.h>
#include <CoreFoundation/CFData.h>

CG_EXTERN_C_BEGIN

typedef const struct CGERegion* CGERegionRef;
typedef struct CGERegion* CGEMutableRegionRef;

// *** CGERegionRef ***

CG_EXTERN CGERegionRef CGERegionCreateEmpty();
CG_EXTERN CGERegionRef CGERegionCreateWithRect(CGRect rect);
CG_EXTERN CGERegionRef CGERegionCreateCopy(CGERegionRef region);
CG_EXTERN CGERegionRef CGERegionCreateCopyIntersectingRect(CGERegionRef region, CGRect rect);
CG_EXTERN CGERegionRef CGERegionCreateIntersection(CGERegionRef region1, CGERegionRef region2);

CG_EXTERN CGERegionRef CGERegionRetain(CGERegionRef region);
CG_EXTERN void CGERegionRelease(CGERegionRef region);

// If `region' is NULL then result is the same as if it is empty.
// `otherRegion' may be NULL: means empty region.
CG_EXTERN bool CGERegionEqualToRegion(CGERegionRef region, CGERegionRef otherRegion);

CG_EXTERN bool CGERegionIsEmpty(CGERegionRef region); // Result is TRUE if `region' is NULL.

CG_EXTERN CGRect CGERegionGetBoundingRect(CGERegionRef region);
CG_EXTERN CFDataRef CGERegionCopyRectApproximation(CGERegionRef region);

CG_EXTERN bool CGERegionIntersectsRect(CGERegionRef region, CGRect rect);
CG_EXTERN bool CGERegionIntersectsRegion(CGERegionRef region, CGERegionRef otherRegion); // `otherRegion' may be NULL: means empty region.

CG_EXTERN void CGERegionShow(CGERegionRef region);
CG_EXTERN void CGERegionShowAs(CGERegionRef region, CFStringRef objectTypeName, const void* objectAddress);

// *** CGEMutableRegionRef ***

CG_EXTERN CGEMutableRegionRef CGERegionCreateMutable();
CG_EXTERN CGEMutableRegionRef CGERegionCreateMutableCopy(CGERegionRef region);
CG_EXTERN CGEMutableRegionRef CGERegionCreateMutableCopyIntersectingRect(CGERegionRef region, CGRect rect);
CG_EXTERN CGEMutableRegionRef CGERegionCreateMutableIntersection(CGERegionRef region1, CGERegionRef region2);

CG_EXTERN void CGERegionAddRect(CGEMutableRegionRef region, CGRect rect);
CG_EXTERN void CGERegionAddRegion(CGEMutableRegionRef region, CGERegionRef otherRegion); // `otherRegion' may be NULL: means empty region.

CG_EXTERN void CGERegionIntersectRect(CGEMutableRegionRef region, CGRect rect);
CG_EXTERN void CGERegionIntersectRegion(CGEMutableRegionRef region, CGERegionRef otherRegion); // `otherRegion' may be NULL: means empty region.

CG_EXTERN void CGERegionApplyAffineTransform(CGEMutableRegionRef region, CGAffineTransform transform);

CG_EXTERN_C_END

#endif // CGERegion_H_
